

//  it3xl.ru  .



using System;
using System.Linq;
using System.Linq.Expressions;
using System.ServiceModel;
using System.ServiceModel.Activation;
using System.Data;
using BillAnalyserWebApp.Code.Data;
using BillAnalyserWebApp.Code.IFlex.DB;
using CrossCompileSharing.Data;
using Oracle.DataAccess.Client;

namespace BillAnalyserWebApp
{
	[ServiceContract(Namespace = "iFlexAuth")]
	[AspNetCompatibilityRequirements(RequirementsMode = AspNetCompatibilityRequirementsMode.Allowed)]
	public class Auth
	{
		#region CheckPwd

		[OperationContract]
		public ServiceResult<PwdResult> CheckPwd(String login, String pwd
			//, Boolean needToken
			)
		{
			ServiceResult<PwdResult> pwdServiceResult;

			if (IsMock(login, pwd))
			{
				pwdServiceResult = GetMockPwdServiceResult(login, pwd);
			}
			else
			{
				var oCom = new OracleCommand { CommandType = CommandType.StoredProcedure, CommandText = "ba_abonents_iapi.check_pswd" };
				//oCom.BindByName = true;

				oCom.Parameters.Add("i_login", OracleDbType.Varchar2, login, ParameterDirection.Input);
				oCom.Parameters.Add("i_password", OracleDbType.Varchar2, pwd, ParameterDirection.Input);

				oCom.Parameters.Add("o_res", OracleDbType.Int64, ParameterDirection.Output);
				oCom.Parameters.Add("o_role_id", OracleDbType.Int64, ParameterDirection.Output);

				// it3xl.ru         Varchar2   ,  ,       
				//oCom.Parameters.Add("o_name", OracleDbType.Varchar2, ParameterDirection.Output);
				oCom.Parameters.Add("o_name", OracleDbType.Varchar2, 300, null, ParameterDirection.Output);

				oCom.Parameters.Add("o_abonent_id", OracleDbType.Int64, ParameterDirection.Output);

				pwdServiceResult = OracleDbPull.ExecuteNonQuery<PwdResult>(
					oCom,
					serviceResult =>
					{
						serviceResult.DataOrNullOrDefValue = new PwdResult
						{
							Name = oCom.Parameters["o_name"].Value != null ? oCom.Parameters["o_name"].Value.ToString() : "",
							IsAuth = oCom.Parameters["o_res"].Value != null ? oCom.Parameters["o_res"].Value.ToString() == "1" : false,
							AbonentId = oCom.Parameters["o_abonent_id"].Value as Int64?
						};

						var roleId = oCom.Parameters["o_role_id"].Value != null ? oCom.Parameters["o_role_id"].Value.ToString() : null;
						if (roleId == "1")
						{
							serviceResult.DataOrNullOrDefValue.Role = UserRole.Abonent;
						}
						else if (roleId == "2")
						{
							serviceResult.DataOrNullOrDefValue.Role = UserRole.Admin;
						}
						else
						{
							serviceResult.DataOrNullOrDefValue.Role = UserRole.Anonymous;
						}
					}
					);

			}

			if (pwdServiceResult.IsDone == false)
			{

				return pwdServiceResult;
			}

			var getSettingsResult = AnalyserSettings.SettingsGet(0);
			if (getSettingsResult.HasErrors)
			{
				pwdServiceResult.AddResultStates(getSettingsResult);

				return pwdServiceResult;
			}

			// Lock: :       .
			var prName = "bl";
			var oCom2 = new OracleCommand("select BA_ABONENTS_IAPI.is_search_service_from_bill() from dual");
			oCom2.Parameters.Add(prName, OracleDbType.Varchar2, null, ParameterDirection.Output);
			OracleDbPull.GetDataTable<Boolean>(
				oCom2,
				(d, r) =>
				{
					if (d.Rows.Count == 0)
					{
						return;
					}
					var val = d.Rows[0][0].ToString();
					if (0 < val.Length)
					{
						getSettingsResult.DataOrNullOrDefValue.Settings.Add(new AnalyserSetting
						{
							Id = getSettingsResult.DataOrNullOrDefValue.Settings.Max(item => item.Id) + 86,
							SettingName = "bi" + "ll",
							Value_type = "V",
							Value_V = val[0].ToString(),
							ParentId = getSettingsResult.DataOrNullOrDefValue.Settings.First(item => item.SettingName == "").Id,
						});
					}
				}
				);

			pwdServiceResult.DataOrNullOrDefValue.Settings = getSettingsResult.DataOrNullOrDefValue.Settings;

			var settingGroups = AnalyserSettings.SettingGroupsGet();
			if (settingGroups.HasErrors)
			{
				pwdServiceResult.AddResultStates(settingGroups);

				return pwdServiceResult;
			}
			pwdServiceResult.DataOrNullOrDefValue.SettingGroups = settingGroups.DataOrNullOrDefValue;

			var allBillingPeriods = AllBillingPeriods.GetAllBillingPeriods();
			if (allBillingPeriods.IsDone == false)
			{
				pwdServiceResult.AddResultStates(allBillingPeriods);

				return pwdServiceResult;
			}
			pwdServiceResult.DataOrNullOrDefValue.AllBillingPeriods = allBillingPeriods.DataOrNullOrDefValue;

			return pwdServiceResult;
		}

		private static ServiceResult<PwdResult> GetMockPwdServiceResult(string login, string pwd)
		{
			var pwdServiceResult = new ServiceResult<PwdResult>();
			if (IsMockAdmin(login, pwd))
			{
				pwdServiceResult.DataOrNullOrDefValue = new PwdResult
				{
					Name = "test admin",
					IsAuth = true,
					Role = UserRole.Admin,
				};
			}
			else if (IsMockAbonent(login, pwd))
			{
				pwdServiceResult.DataOrNullOrDefValue = new PwdResult
				{
					Name = "test abonent",
					IsAuth = true,
					Role = UserRole.Abonent,
				};
			}
			return pwdServiceResult;
		}

		private static bool IsMock(string login, string password)
		{
#if !DEBUG
			return false;
#else
			return IsMockAbonent(login, password) || IsMockAdmin(login, password);
#endif
		}

		private static bool IsMockAbonent(string login, string password)
		{
			if (login == "0" && password == "0")
			{
				return true;
			}
			return false;
		}

		private static bool IsMockAdmin(string login, string password)
		{
			if (login == "1" && password == "1")
			{
				return true;
			}
			return false;
		}


		#endregion
	}
}