//   .             !

using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using BillAnalyserWebApp.Code.IFlex.DB;
using CrossCompileSharing.Data;
using Oracle.DataAccess.Client;

namespace BillAnalyserWebApp.Code.Data
{
	/// <summary>
	/// Summary description for Abonents
	/// </summary>
	public static class Abonents
	{
		internal static ServiceResult<List<Abonent>> AbonentsGet()
		{
			const string sql = (@"
            SELECT
              A.ID
            , A.ABONENT_ID
            , A.NAME
            , A.IS_GROUP
            , A.EMAIL
            , A.POST
            , A.ROLES_ID
            , A.EXPORT_TYPE
            , A.CODE_1C
            , A.LOGIN
            FROM
              V_BA_ABONENTS A
            ");

			var oCom = new OracleCommand(sql) { CommandType = System.Data.CommandType.Text };

			// it3xl.ru     DataSet   .       .      DataSet,     
			// it3xl.ru          1,  ID1
			return OracleDbPull.GetDataTable<List<Abonent>>(
				oCom,
				(resultDataTable, serviceResult) =>
				{
					var abonentRaw = (
						from row in resultDataTable.AsEnumerable()
						select new
						{
							Id = (Int64)row.Field<Decimal>("ID"),
							ParentId = (Int64?)row.Field<Decimal?>("ABONENT_ID"),
							Name = row.Field<String>("NAME"),
							IsGroup = row.Field<Int16>("IS_GROUP") == 1,
							Email = row.Field<String>("EMAIL"),
							Appointment = row.Field<String>("POST"),
							RoleId = (Int32?)row.Field<Decimal?>("ROLES_ID"),
							ExportType = (Int32?)row.Field<Int16?>("EXPORT_TYPE"),
							Code1C = row.Field<String>("CODE_1C"),
							Login = row.Field<String>("LOGIN")
						}
						).ToList();

					var abonentDict = new Dictionary<Int64, Abonent>();
					foreach (var row in abonentRaw)
					{
						if (
							!abonentDict.ContainsKey(row.Id))
						{
							abonentDict.Add(row.Id, new Abonent
							{
								Id = row.Id,
								ParentId = row.ParentId,
								Name = row.Name,
								IsGroup = row.IsGroup,
								Email = row.Email,
								Appointment = row.Appointment,
								RoleId = row.RoleId,
								ExportType = row.ExportType,
								Code1C = row.Code1C,
								Login = row.Login
							});
						}

					}

					var abonents = new List<Abonent>();
					foreach (var keyVal in abonentDict)
					{
						abonents.Add(keyVal.Value);
					}

					serviceResult.DataOrNullOrDefValue = abonents;
				}
				);


		}

		internal static ServiceResult<List<Abonent>> AbonentGroupsGet()
		{
			const string sql = (@"
            SELECT
              A.ID
            , A.ABONENT_ID
            , A.NAME
            , A.IS_GROUP
            , A.EMAIL
            , A.POST
            , A.ROLES_ID
            , A.EXPORT_TYPE
            , A.CODE_1C
            , A.LOGIN
            FROM
              V_BA_ABONENTS A
            WHERE IS_GROUP = 1
            ");

			var oCom = new OracleCommand(sql) { CommandType = CommandType.Text };

			// it3xl.ru     DataSet   .       .      DataSet,     
			// it3xl.ru          1,  ID1
			return OracleDbPull.GetDataTable<List<Abonent>>(
				oCom,
				(resultDataTable, serviceResult) =>
				{
					var abonentRaw = (
						from row in resultDataTable.AsEnumerable()
						select new
						{
							Id = (Int64)row.Field<Decimal>("ID"),
							ParentId = (Int64?)row.Field<Decimal?>("ABONENT_ID"),
							Name = row.Field<String>("NAME"),
							IsGroup = row.Field<Int16>("IS_GROUP") == 1,
							Email = row.Field<String>("EMAIL"),
							Appointment = row.Field<String>("POST"),
							RoleId = (Int32?)row.Field<Decimal?>("ROLES_ID"),
							ExportType = (Int32?)row.Field<Int16?>("EXPORT_TYPE"),
							Code1C = row.Field<String>("CODE_1C"),
							Login = row.Field<String>("LOGIN")
						}
						).ToList();

					var abonentDict = new Dictionary<Int64, Abonent>();
					foreach (var row in abonentRaw)
					{
						if (
							!abonentDict.ContainsKey(row.Id))
						{
							abonentDict.Add(row.Id, new Abonent
							{
								Id = row.Id,
								ParentId = row.ParentId,
								Name = row.Name,
								IsGroup = row.IsGroup,
								Email = row.Email,
								Appointment = row.Appointment,
								RoleId = row.RoleId,
								ExportType = row.ExportType,
								Code1C = row.Code1C,
								Login = row.Login
							});
						}

					}

					var abonents = new List<Abonent>();
					foreach (var keyVal in abonentDict)
					{
						abonents.Add(keyVal.Value);
					}

					serviceResult.DataOrNullOrDefValue = abonents;
				}
				);

		}

		internal static ServiceResult<List<Abonent>> AbonentGroupsGetWithCompensation(long idCompensation)
		{
			const string sql = (@"
                select ab.*
                ,case
                 when lim.c2 = 0 then
                  -1
                 when lim.c1 = lim.c2 then
                  1
                 when lim.c1 <> lim.c2 then
                  0
                end as checked
                from v_ba_abonents ab
                ,(select a.abonent_id
                      ,count(p.id) c1
                      ,count(c.phones_id) c2
                  from v_ba_phones              p
                      ,v_ba_abonents            a
                      ,v_ba_phones_compensation c
                 where p.ABONENT_ID = a.id
                   and a.is_group = 0
                   and p.ID = c.phones_id(+)
                   and (c.id = :id_compensation or c.id is null)
                 group by a.abonent_id) lim
                where ab.is_group = 1
                and ab.id = lim.abonent_id
            ");

			var oCom = new OracleCommand(sql) { CommandType = CommandType.Text };
			oCom.Parameters.Add("id_compensation", OracleDbType.Decimal, idCompensation, System.Data.ParameterDirection.Input);

			// it3xl.ru     DataSet   .       .      DataSet,     
			// it3xl.ru          1,  ID1
			return OracleDbPull.GetDataTable<List<Abonent>>(
				oCom,
				(resultDataTable, serviceResult) =>
				{
					var abonentRaw = (
						from row in resultDataTable.AsEnumerable()
						select new
						{
							Id = (Int64)row.Field<Decimal>("ID"),
							ParentId = (Int64?)row.Field<Decimal?>("ABONENT_ID"),
							Name = row.Field<String>("NAME"),
							IsGroup = row.Field<Int16>("IS_GROUP") == 1,
							Email = row.Field<String>("EMAIL"),
							Appointment = row.Field<String>("POST"),
							RoleId = (Int32?)row.Field<Decimal?>("ROLES_ID"),
							ExportType = (Int32?)row.Field<Int16?>("EXPORT_TYPE"),
							Code1C = row.Field<String>("CODE_1C"),
							Login = row.Field<String>("LOGIN"),
							Checked = (short)row.Field<Decimal>("CHECKED"),
						}
						).ToList();

					var abonentDict = new Dictionary<Int64, Abonent>();
					foreach (var row in abonentRaw)
					{
						if (
							!abonentDict.ContainsKey(row.Id))
						{
							var na = new Abonent { Id = row.Id, ParentId = row.ParentId, Name = row.Name, IsGroup = row.IsGroup, Email = row.Email, Appointment = row.Appointment, RoleId = row.RoleId, ExportType = row.ExportType, Code1C = row.Code1C, Login = row.Login, Checked = row.Checked };
							if (na.Checked == 1)
								na.HasChild = true;
							else if (na.Checked == -1)
								na.HasChild = false;
							else
								na.HasChild = null;

							abonentDict.Add(row.Id, na);

						}
					}

					var abonents = new List<Abonent>();
					foreach (var keyVal in abonentDict)
					{
						abonents.Add(keyVal.Value);
					}

					serviceResult.DataOrNullOrDefValue = abonents;
				}
				);
		}
	}
}