//   .             !



using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using BillAnalyserWebApp.Code.IFlex.DB;
using CrossCompileSharing.Data;
using CrossCompileSharing.Log;
using CrossCompileSharing.UiStrings;
using Oracle.DataAccess.Client;

namespace BillAnalyserWebApp.Code.Data
{
	/// <summary>
	/// Summary description for Abonents
	/// </summary>
	public static class AggServices
	{
		#region AggServicesGet

		internal static ServiceResult<List<AggService>> AggServicesGet()
		{
			//    
			const string sql = "SELECT AGG_ID, SERVICE_ID, NAME, UNIT_ID, UNIT_NAME, MASK, DSC from v_ba_services_tree";

			var oCom = new OracleCommand(sql);

			return OracleDbPull.GetDataTable<List<AggService>>(
				oCom,
				(resultDataTable, serviceResult) =>
				{
					var serviceRaw = (
						from row in resultDataTable.AsEnumerable()
						select new
						{
							AGG_ID = (Int64?)row.Field<Decimal?>("AGG_ID"),
							SERVICE_ID = (Int64?)row.Field<Decimal?>("SERVICE_ID"),
							NAME = row.Field<String>("NAME"),
							UNIT_ID = (Int64?)row.Field<Decimal?>("UNIT_ID"),
							UNIT_NAME = row.Field<String>("UNIT_NAME"),
							MASK = row.Field<String>("MASK"),
							DSC = row.Field<String>("DSC"),
						}
						).ToList();

					var services = new List<AggService>();

					//     
					foreach (var row in serviceRaw)
					{
						var agg = new AggService
						{
							IdAgg = row.AGG_ID,
							IdService = row.SERVICE_ID,
							Name = row.NAME,
						};

						agg.IsAgg = agg.IdService.HasValue ? false : true;
						agg.IdUnit = row.UNIT_ID;
						agg.NameUnit = row.UNIT_NAME;
						agg.Mask = row.MASK;
						agg.Dsc = row.DSC;
						agg.ChildServices = new List<AggService>();

						services.Add(agg);
					}

					serviceResult.DataOrNullOrDefValue = services;
				}
				);
		}

		#endregion

		#region    

		//--    
		//-- %param i_service_id       
		//-- %param i_service_agg_id  
		//-- %param i_from_date       : 
		//------------------------------------------------------------------
		//  ba_services_iapi.set_service_agg(i_service_id => :i_service_id,
		//                                   i_service_agg_id => :i_service_agg_id,
		//                                   i_from_date => :i_from_date);

		/// <summary>
		///    
		/// %param i_service_id       
		/// %param i_service_agg_id  
		/// %param i_from_date       : 
		/// ------------------------------------------------------------------
		/// ba_services_iapi.set_service_agg(i_service_id => :i_service_id,
		///                                   i_service_agg_id => :i_service_agg_id,
		///                                   i_from_date => :i_from_date);                                  
		/// </summary>
		/// <returns></returns>
		internal static ServiceResult<Boolean> set_service_agg(ServiceMoveInfo[] move_info)
		{
			var result = new ServiceResult<Boolean> { DataOrNullOrDefValue = true };
			foreach (ServiceMoveInfo mi in move_info)
			{
				long service_id = mi.id_service;
				long service_agg_id = mi.id_target_agg;

				OracleCommand command = new OracleCommand("ba_services_iapi.set_service_agg");
				command.CommandType = System.Data.CommandType.StoredProcedure;
				command.Parameters.Add("i_service_id", OracleDbType.Int64, service_id, System.Data.ParameterDirection.Input);
				command.Parameters.Add("i_service_agg_id", OracleDbType.Int64, service_agg_id, System.Data.ParameterDirection.Input);
				command.Parameters.Add("i_from_date", OracleDbType.Date, null, System.Data.ParameterDirection.Input);

				var intResult = OracleDbPull.ExecuteNonQuery<Boolean>(
				command,
				serviceResult =>
				{
					serviceResult.DataOrNullOrDefValue = true;
				}
				);

				if (intResult.IsDone == false)
				{
					return intResult;
				}
			}

			return result;
		}
		#endregion

		#region  

		//------------------------------------------------------------------
		//--  
		//--%param i_service_agg_id    
		//--%param i_unit_id          .
		//--%param i_name             
		//--%param i_mask             
		//--%param i_dsc              
		//--%param i_from_date        : C
		//------------------------------------------------------------------
		//  ba_service_agg_iapi.upd_service_agg(i_service_agg_id => :i_service_agg_id,
		//                                      i_unit_id => :i_unit_id,
		//                                      i_name => :i_name,
		//                                      i_mask => :i_mask,
		//                                      i_dsc => :i_dsc,
		//                                      i_from_date => :i_from_date); 
		//,               2 ,      null.
		internal static ServiceResult<bool> update_agg(long i_service_agg_id, long i_unit_id, string i_name, string i_mask, string i_dsc)
		{
			var command = new OracleCommand("ba_service_agg_iapi.upd_service_agg");
			command.CommandType = System.Data.CommandType.StoredProcedure;
			command.Parameters.Add("i_service_agg_id", OracleDbType.Int64, i_service_agg_id, System.Data.ParameterDirection.Input);
			command.Parameters.Add("i_unit_id", OracleDbType.Int64, i_unit_id, System.Data.ParameterDirection.Input);
			command.Parameters.Add("i_name", OracleDbType.Varchar2, i_name, System.Data.ParameterDirection.Input);
			command.Parameters.Add("i_mask", OracleDbType.Varchar2, i_mask, System.Data.ParameterDirection.Input);
			command.Parameters.Add("i_dsc", OracleDbType.Varchar2, i_dsc, System.Data.ParameterDirection.Input);
			command.Parameters.Add("i_from_date", OracleDbType.Date, null, System.Data.ParameterDirection.Input);

			return OracleDbPull.ExecuteNonQuery<Boolean>(
				command,
				serviceResult =>
				{
					serviceResult.DataOrNullOrDefValue = true;
				}
				);
		}
		#endregion

		#region  

		//------------------------------------------------------------------
		//--  
		//--%param i_service_agg_id    
		//------------------------------------------------------------------
		//  ba_service_agg_iapi.trm_service_agg(i_service_agg_id => :i_service_agg_id);         
		internal static ServiceResult<bool> delete_agg(long i_service_agg_id)
		{
			OracleCommand command = new OracleCommand("ba_service_agg_iapi.trm_service_agg");
			command.CommandType = System.Data.CommandType.StoredProcedure;
			command.Parameters.Add("i_service_agg_id", OracleDbType.Int64, i_service_agg_id, System.Data.ParameterDirection.Input);

			return OracleDbPull.ExecuteNonQuery<Boolean>(
				command,
				serviceResult =>
				{
					serviceResult.DataOrNullOrDefValue = true;
				}
				);
		}
		#endregion

		#region   

		//------------------------------------------------------------------
		//--  
		//-- %param i_unit_id           .
		//-- %param i_name                 
		//-- %param i_mask              
		//-- %param i_dsc                 
		//-- %param i_from_date          
		//-- %param i_to_date            
		//-- %param o_id                  
		//------------------------------------------------------------------
		//ba_service_agg_iapi.procedure ins_ba_service_agg(
		//                             i_unit_id in ba_service_agg.unit_id%type,
		//                             i_name in ba_service_agg.name%type,
		//                             i_mask in ba_service_agg.mask%type,
		//                             i_dsc in ba_service_agg.dsc%type,
		//                             i_from_date in ba_service_agg.from_date%type,
		//                             i_to_date in ba_service_agg.to_date%type,
		//                             o_id out number);         
		internal static ServiceResult<long> InsertAgg(long i_unit_id, string i_name, string i_mask, string i_dsc)
		{
			var command = new OracleCommand("ba_service_agg_iapi.ins_ba_service_agg") { CommandType = CommandType.StoredProcedure };
			command.Parameters.Add("i_unit_id", OracleDbType.Int64, i_unit_id, ParameterDirection.Input);
			command.Parameters.Add("i_name", OracleDbType.Varchar2, i_name, ParameterDirection.Input);
			command.Parameters.Add("i_mask", OracleDbType.Varchar2, i_mask, ParameterDirection.Input);
			command.Parameters.Add("i_dsc", OracleDbType.Varchar2, i_dsc, ParameterDirection.Input);
			command.Parameters.Add("i_from_date", OracleDbType.Date, null, ParameterDirection.Input);
			command.Parameters.Add("i_to_date", OracleDbType.Date, null, ParameterDirection.Input);
			command.Parameters.Add("o_id", OracleDbType.Int64, ParameterDirection.Output);

			return OracleDbPull.ExecuteNonQuery<long>(
				command,
				serviceResult =>
				{
					try
					{
						serviceResult.DataOrNullOrDefValue = Convert.ToInt64(command.Parameters["o_id"].Value);
					}
					catch(Exception ex)
					{
						Logger.Write(ex);
						serviceResult.AddBusinessError(UiMessages.___);
					}
				}
				);

		}
		#endregion

		#region   
		internal static ServiceResult<AggService> GetService(long id)
		{
			//    
			const string sql = "SELECT AGG_ID, SERVICE_ID, NAME, UNIT_ID, UNIT_NAME, MASK, DSC from v_ba_services_tree where AGG_ID = :id AND SERVICE_ID is null";

			var command = new OracleCommand(sql) { CommandType = CommandType.Text };
			command.Parameters.Add("i_service_agg_id", OracleDbType.Int64, id, ParameterDirection.Input);

			return OracleDbPull.GetDataTable<AggService>(
				command,
				(resultDataTable, serviceResult) =>
				{
					var serviceRaw = (
						from row in resultDataTable.AsEnumerable()
						select new
						{
							AGG_ID = (Int64?)row.Field<Decimal?>("AGG_ID"),
							SERVICE_ID = (Int64?)row.Field<Decimal?>("SERVICE_ID"),
							NAME = row.Field<String>("NAME"),
							UNIT_ID = (Int64?)row.Field<Decimal?>("UNIT_ID"),
							UNIT_NAME = row.Field<String>("UNIT_NAME"),
							MASK = row.Field<String>("MASK"),
							DSC = row.Field<String>("DSC"),
						}
						).ToList();

					var services = new List<AggService>();

					//     
					foreach (var row in serviceRaw)
					{
						var agg = new AggService { IdAgg = row.AGG_ID, IdService = row.SERVICE_ID, Name = row.NAME };

						agg.IsAgg = agg.IdService.HasValue ? false : true;
						agg.IdUnit = row.UNIT_ID;
						agg.NameUnit = row.UNIT_NAME;
						agg.Mask = row.MASK;
						agg.Dsc = row.DSC;
						agg.IsSelected = false;
						agg.ChildServices = new List<AggService>();

						services.Add(agg);
					}

					serviceResult.DataOrNullOrDefValue= services.Count != 1 ? null : services[0];
				}
				);
		}

		#endregion
	}
}