//   .             !

using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using BillAnalyserWebApp.Code.IFlex.DB;
using CrossCompileSharing.Data;
using CrossCompileSharing.UiStrings;
using Oracle.DataAccess.Client;

namespace BillAnalyserWebApp.Code.Data
{
	/// <summary>
	/// Summary description
	/// </summary>
	public static class AnalyserSettings
	{
		internal static class SmtpServerPasswordSettingMetadata
		{
			public static readonly String Name = "";
			public static readonly Int32 Id = 84;
		}
		internal static class EmailSubjectSettingMetadata
		{
			public static readonly String Name = "  ";
			public static readonly Int32 Id = 130;
		}

		/// <summary>
		/// it3xl.ru:   0,     .
		/// </summary>
		/// <param name="parentId">it3xl.ru: ? ID    .</param>
		/// <returns></returns>
		internal static ServiceResult<GetSettingsResult> SettingsGet(Int64 parentId)
		{
			var result = UnsafeSettingsGet(parentId, true);

			if (result.DataOrNullOrDefValue == null)
				return result;

			// it3xl.ru:      .
			result.DataOrNullOrDefValue.Settings
				.Where(el => el.SettingName == SmtpServerPasswordSettingMetadata.Name || el.Id == SmtpServerPasswordSettingMetadata.Id)
				.ToList()
				.ForEach(el => el.Value_V = PasswordMark.Value);

			return result;
		}
		/// <summary>
		/// it3xl.ru:   0,     .
		/// TODO: it3xl.ru: .   .    .
		/// </summary>
		/// <param name="Id">it3xl.ru: ? ID    .</param>
		/// <param name="useIdAsParent">
		/// it3xl.ru: ,     ,        ID.
		/// ,              ,      ParentSetting.
		///  ,       .
		/// </param>
		/// <returns></returns>
		internal static ServiceResult<GetSettingsResult> UnsafeSettingsGet(Int64 Id, Boolean useIdAsParent)
		{
			#region  
			String sql;
			OracleCommand oCom;

			if (Id == 0)
			{
				sql = String.Format(
					@"select id, 
setting_id, 
sysname, 
value_type, 
value_v, 
value_n, 
value_d, 
value_list, 
dsc from v_ba_settings t");
				oCom = new OracleCommand(sql);
			}
			else
			{
				sql = String.Format(
					@"select id, 
setting_id, 
sysname, 
value_type, 
value_v, 
value_n, 
value_d, 
value_list, 
dsc from v_ba_settings t where setting_id = :parentId or id = :parentId");
				oCom = new OracleCommand(sql);
				oCom.Parameters.Add("parentId", OracleDbType.Int64, Id, ParameterDirection.Input);
			}

			#endregion

			ServiceResult<GetSettingsResult> result =
				OracleDbPull.GetDataTable<GetSettingsResult>(oCom,
				(resultDataTable, serviceResult) =>
				{
					serviceResult.DataOrNullOrDefValue = new GetSettingsResult { ParentSetting = new AnalyserSetting(), Settings = new List<AnalyserSetting>() };

					var settingsRaw = (
						from row in resultDataTable.AsEnumerable()
						select new
						{
							Id = (Int64)row.Field<Decimal>("id"),
							setting_id = (Int64?)row.Field<Decimal?>("setting_id"),
							sysname = row.Field<String>("sysname"),
							value_type = row.Field<String>("value_type"),
							value_v = row.Field<String>("value_v"),
							value_n = (double?)row.Field<Decimal?>("value_n"),
							value_d = row.Field<DateTime?>("value_d"),
							values = row.Field<string>("value_list"),
							dsc = row.Field<String>("dsc")
						}
						).ToList();

					serviceResult.DataOrNullOrDefValue.Settings.Clear();

					foreach (var row in settingsRaw)
					{
						var ns = new AnalyserSetting { Id = row.Id, ParentId = row.setting_id, SettingName = row.sysname, Value_type = row.value_type, Value_V = row.value_v, Value_N = row.value_n, Value_D = row.value_d };

						if (ns.Value_V == null)
						{
							if (ns.Value_type == "N")
							{
								ns.Value_V = ns.Value_N.HasValue ? ns.Value_N.Value.ToString() : null;
							}
							if (ns.Value_type == "D")
							{
								ns.Value_V = ns.Value_D.HasValue ? ns.Value_D.Value.ToShortDateString() :null;
							}
						}

						ns.Values = row.values;
						ns.Dsc = row.dsc;
						ns.IsReadOnlyValue = true;
						ns.IsVisibleCancel = false;
						if (ns.Values != null)
						{
							ns.IsVisibleList = true;
							ns.IsVisibleBox = false;
						}
						else
						{
							ns.IsVisibleList = false;
							ns.IsVisibleBox = true;
						}

						var listValueResult = GetListValue(ns.Id);
						if (listValueResult.IsDone == false)
						{
							serviceResult.AddResultStates(listValueResult);


							//!!!
							return;
						}

						ns.ListValues = listValueResult.DataOrNullOrDefValue;
						if (ns.ListValues == null)
						{
							ns.ListValues = new List<string>();
						}
						if (ns.ListValues != null && ns.ListValues.Count != 0)
						{
							ns.SelectedIndex = ns.ListValues.IndexOf(ns.Value_V);
						}
						if (ns.ListValues != null)
							if (ns.ListValues.Count == 0)
							{
								ns.SelectedIndex = -1;
							}

						if (ns.Id == Id && useIdAsParent)
						{
							serviceResult.DataOrNullOrDefValue.ParentSetting = ns;
						}
						else
						{
							serviceResult.DataOrNullOrDefValue.Settings.Add(ns);
						}
					}
				}


				);

			return result;
		}

		internal static ServiceResult<List<AnalyserSetting>> SettingGroupsGet()
		{
			var sql = String.Format("select id, setting_id, sysname, value_type, value_v, value_n, value_d, value_list, dsc from v_ba_settings t where setting_id is null");

			var oCom = new OracleCommand(sql);

			return OracleDbPull.GetDataTable<List<AnalyserSetting>>(
				oCom,
				(resultDataTable, serviceResult) =>
				{
					var settingsRaw = (
						from row in resultDataTable.AsEnumerable()
						select new
						{
							Id = (Int64)row.Field<Decimal>("id"),
							setting_id = (Int64?)row.Field<Decimal?>("setting_id"),
							sysname = row.Field<String>("sysname"),
							value_type = row.Field<String>("value_type"),
							value_v = row.Field<String>("value_v"),
							value_n = (double?)row.Field<Decimal?>("value_n"),
							value_d = row.Field<DateTime?>("value_d"),
							values = row.Field<string>("value_list"),
							dsc = row.Field<String>("dsc")
						}
						).ToList();

					var settings = new List<AnalyserSetting>();

					//     
					foreach (var row in settingsRaw)
					{
						var ns = new AnalyserSetting { Id = row.Id, ParentId = row.setting_id, SettingName = row.sysname, Value_type = row.value_type, Value_V = row.value_v, Value_N = row.value_n, Value_D = row.value_d, Values = row.values, Dsc = row.dsc };
						settings.Add(ns);
					}

					serviceResult.DataOrNullOrDefValue = settings;
				}
				);

		}

		#region GetListValue

		/// <summary>
		///   
		/// </summary>
		internal static ServiceResult<List<String>> GetListValue(Int64 idSetting)
		{

			var oCom = new OracleCommand { CommandType = CommandType.Text, CommandText = "select * from table(ba_setting_iapi.get_value_list(:id))" };

			oCom.Parameters.Add("id", idSetting);

			return OracleDbPull.GetDataTable<List<String>>(
				oCom,
				(resultDataTable, serviceResult) =>
				{
					var listValues = new List<string>();
					foreach (DataRow dr in resultDataTable.Rows)
					{
						var currentValue = (string)dr.ItemArray[0];
						listValues.Add(currentValue);
					}

					serviceResult.DataOrNullOrDefValue = listValues;
				}
				);

		}

		#endregion

		/// <summary>
		///  
		/// </summary>
		internal static ServiceResult<SetSettingsValueResult> SetValue(Int64 id, double? vn, String vv, DateTime? vd)
		{
			var command = new OracleCommand("ba_setting_iapi.setVal ") { CommandType = CommandType.StoredProcedure };
			command.Parameters.Add("i_id", OracleDbType.Int32, id, ParameterDirection.Input);
			command.Parameters.Add("i_vn", OracleDbType.Decimal, vn, ParameterDirection.Input);
			command.Parameters.Add("i_vd", OracleDbType.Date, vd, ParameterDirection.Input);
			command.Parameters.Add("i_vv", OracleDbType.Varchar2, vv, ParameterDirection.Input);

			ServiceResult<Boolean> resultUpdate = OracleDbPull.ExecuteNonQuery<Boolean>(
			command,
			serviceResult =>
			{
				serviceResult.DataOrNullOrDefValue = true;
			}
			);

			if (resultUpdate.DataOrNullOrDefValue == true)
			{
				//      ,     SetSettingsValueResult
				//   ,         

				string sql = "select setting_id from v_ba_settings t where id = :SettingId";
				OracleCommand commandForGetParentId = new OracleCommand(sql);
				commandForGetParentId.Parameters.Add("SettingId", OracleDbType.Int64, id, ParameterDirection.Input);

				ServiceResult<SetSettingsValueResult> result = OracleDbPull.GetDataTable<SetSettingsValueResult>(commandForGetParentId,
					(resultDataTable, serviceResult) =>
					{
						serviceResult.DataOrNullOrDefValue = new SetSettingsValueResult();
						var parentIdRaw = (
						from row in resultDataTable.AsEnumerable()
						select new
						{
							setting_id = (Int64?)row.Field<Decimal?>("setting_id")
						}
						).ToList();

						if (parentIdRaw.Count == 1 && parentIdRaw[0].setting_id.HasValue)
						{
							serviceResult.DataOrNullOrDefValue.Id = id;
							serviceResult.DataOrNullOrDefValue.ParentId = parentIdRaw[0].setting_id.Value;
							serviceResult.DataOrNullOrDefValue.Result = true;
							serviceResult.DataOrNullOrDefValue.ErrorMessage = "";
						}
						else
						{
							serviceResult.DataOrNullOrDefValue.Id = id;
							serviceResult.DataOrNullOrDefValue.ParentId = 0;
							serviceResult.DataOrNullOrDefValue.Result = false;
							serviceResult.DataOrNullOrDefValue.ErrorMessage = "    ";
						}
					}
					);

				return result;
			}
			else
			{
				//       
				//   
				var result = new ServiceResult<SetSettingsValueResult> { DataOrNullOrDefValue = { Id = 0, ParentId = 0, Result = false } };
				result.DataOrNullOrDefValue.ErrorMessage = result.ErrorMessageOrMessageFromServer;
				return result;
			}
		}

		#region GetValueCByName

		///// <summary>
		/////  .      
		///// </summary>
		//internal static ServiceResult<String> GetValueCByName(String sysname)
		//{
		//    var command = new OracleCommand("BA_SETTING_IAPI.GET_V")
		//    {
		//        CommandType = CommandType.StoredProcedure,
		//        BindByName = true
		//    };

		//    command.Parameters.Add("i_sysname", OracleDbType.Varchar2, sysname, ParameterDirection.Input);
		//    command.Parameters.Add("ReturnValue", OracleDbType.Varchar2, 2000, null, ParameterDirection.ReturnValue);

		//    return OracleDbPull.ExecuteNonQuery<String>(
		//        command,
		//        serviceResult =>
		//        {
		//            if (command.Parameters["ReturnValue"].Value != DBNull.Value)
		//            {
		//                serviceResult.DataOrDefValue = command.Parameters["ReturnValue"].Value.ToString();
		//            }
		//            else
		//            {
		//                serviceResult.DataOrDefValue = "";
		//            }
		//        }
		//        );
		//}

		#endregion

	}
}