﻿// Создал Андрей Котов. Любые изменения вносимые вами должны быть помечены комментарием идентифицирующим вас и ваши действия!



using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using BillAnalyserWebApp.Code.IFlex.DB;
using CrossCompileSharing.Data;
using Oracle.DataAccess.Client;

namespace BillAnalyserWebApp.Code.Data
{
	/// <summary>
	/// Summary description for Abonents
	/// </summary>
	public static class BillServices
	{
		#region BillServicesGet

		internal static ServiceResult<List<BillService>> BillServicesGet(Int64? idBill, Int64? idPhone)
		{
			var sql = String.Format(
				@"SELECT B.SERVICE_NAME, B.DUR, B.DUR_DIM, B.COST 
FROM v_ba_bill_services B 
WHERE B.BILL_ID = :bill_id and B.PHONE_ID = :phone_id");

			var oCom = new OracleCommand(sql)
			{
				CommandType = CommandType.Text
			};

			oCom.Parameters.Add("bill_id", OracleDbType.Int64, idBill, ParameterDirection.Input);
			oCom.Parameters.Add("phone_id", OracleDbType.Int64, idPhone, ParameterDirection.Input);

			return OracleDbPull.GetDataTable<List<BillService>>(
				oCom,
				(resultDataTable, serviceResult) =>
				{
					var serviceRaw = (
						from row in resultDataTable.AsEnumerable()
						select new
						{
							Name = row.Field<String>("SERVICE_NAME"),
							Duration = (double?)row.Field<Decimal?>("DUR"),
							DurationUnit = row.Field<String>("DUR_DIM"),
							SumAdd = row.Field<double?>("COST")
						}
						).ToList();

					var services = new List<BillService>();

					// Не будем фильтровать по идентификатору
					foreach (var row in serviceRaw)
					{
						var nb = new BillService { Name = row.Name, Duration = row.Duration, DurationUnit = row.DurationUnit, SumAdd = row.SumAdd };
						services.Add(nb);
					}

					serviceResult.DataOrNullOrDefValue = services;
				}
				);
		}

		#endregion

		#region BillChargeServicesGet

		internal static ServiceResult<List<BillService>> BillChargesServicesGet(Int64? idBill, Int64? idPhone)
		{
			String sql = String.Format(
				@"SELECT B.AGG_NAME, B.DUR, B.DUR_DIM, B.sum_add, B.sum_comp, B.sum_ded, B.bill_id, B.phone_id 
FROM v_ba_bill_service_aggs_grid B 
WHERE B.BILL_ID = :bill_id and B.PHONE_ID = :phone_id");

			var oCom = new OracleCommand(sql) { CommandType = CommandType.Text };

			oCom.Parameters.Add("bill_id", OracleDbType.Int64, idBill, ParameterDirection.Input);
			oCom.Parameters.Add("phone_id", OracleDbType.Int64, idPhone, ParameterDirection.Input);

			return OracleDbPull.GetDataTable<List<BillService>>(
				oCom,
				(resultDataTable, serviceResult) =>
				{
					var serviceRaw = (
						from row in resultDataTable.AsEnumerable()
						select new
						{
							Name = row.Field<String>("AGG_NAME"),
							Duration = (double?)row.Field<Decimal?>("DUR"),
							DurationUnit = row.Field<String>("DUR_DIM"),
							SumAdd = row.Field<double?>("sum_add"),
							SumDed = row.Field<double?>("sum_ded"),
							SumComp = (double?)row.Field<Decimal?>("sum_comp"),
							BillId = (Int64)row.Field<Decimal>("BILL_ID"),
							PhoneId = (Int64)row.Field<Decimal>("PHONE_ID")
						}
						).ToList();

					var services = new List<BillService>();

					// Не будем фильтровать по идентификатору
					foreach (var row in serviceRaw)
					{
						var nb = new BillService { Name = row.Name, Duration = row.Duration, DurationUnit = row.DurationUnit, SumAdd = row.SumAdd, SumDed = row.SumDed, SumComp = row.SumComp, BillId = row.BillId, PhoneId = row.PhoneId };

						services.Add(nb);
					}

					serviceResult.DataOrNullOrDefValue = services;
				}
				);

		}

		#endregion

		#region BillServicesGetByPeriod

		internal static ServiceResult<List<BillService>> BillServicesGetByPeriod(Int64 idphone, DateTime period)
		{
			String sql = String.Format(
				@"SELECT b.bill_date, g.phone_id, g.agg_name, g.dur, g.dur_dim, g.sum_add, g.sum_ded, g.sum_comp 
FROM v_ba_bill_service_aggs_grid g, v_ba_bills b 
WHERE b.id = g.bill_id and trunc(b.bill_date, 'MM') = :period and g.phone_id = :phone_id");

			var oCom = new OracleCommand(sql) { CommandType = CommandType.Text };

			oCom.Parameters.Add("period", OracleDbType.Date, period, ParameterDirection.Input);
			oCom.Parameters.Add("phone_id", OracleDbType.Int64, idphone, ParameterDirection.Input);

			return OracleDbPull.GetDataTable<List<BillService>>(
				oCom,
				(resultDataTable, serviceResult) =>
				{
					var serviceRaw = (
						from row in resultDataTable.AsEnumerable()
						select new
						{
							Name = row.Field<String>("agg_name"),
							Duration = (double?)row.Field<Decimal?>("dur"),
							DurationUnit = row.Field<String>("dur_dim"),
							SumAdd = row.Field<double?>("sum_add"),
							SumDed = row.Field<double?>("sum_ded"),
							SumComp = (double?)row.Field<decimal?>("sum_comp"),
						}
						).ToList();

					var services = new List<BillService>();

					// Не будем фильтровать по идентификатору
					foreach (var row in serviceRaw)
					{
						var nb = new BillService { Name = row.Name, Duration = row.Duration, DurationUnit = row.DurationUnit, SumAdd = row.SumAdd, SumDed = row.SumDed, SumComp = row.SumComp };

						services.Add(nb);
					}

					serviceResult.DataOrNullOrDefValue = services;
				}
				);
		}

		#endregion

	}
}