//   .             !



using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using BillAnalyserWebApp.Code.IFlex.DB;
using CrossCompileSharing.Data;
using Oracle.DataAccess.Client;

namespace BillAnalyserWebApp.Code.Data
{
	/// <summary>
	/// Summary description for Abonents
	/// </summary>
	public static class Bills
	{
		#region BillGet

		internal static ServiceResult<List<Bill>> BillGet(DateTime? period, string filter, bool strFilterIsPhone, long? abonentid)
		{
			String sql;
			filter = "%" + filter + "%";

			if (strFilterIsPhone)
			{
				sql = String.Format(
					@"SELECT B.PHONE, B.ABONENT_NAME, B.NACH_SUM, B.BILL_NUMBER, B.BILL_DATE, B.BILL_ID, B.PHONE_ID, B.ABONENT_ID 
FROM V_BA_BILL_TOTAL B 
where trunc(B.BILL_DATE, 'MM') = :period and 
phone like :filter");
			}
			else
			{
				sql = String.Format(
					@"SELECT B.PHONE, B.ABONENT_NAME, B.NACH_SUM, B.BILL_NUMBER, B.BILL_DATE, B.BILL_ID, B.PHONE_ID, B.ABONENT_ID 
FROM V_BA_BILL_TOTAL B 
where trunc(B.BILL_DATE, 'MM') = :period AND 
upper(abonent_name) like upper(:filter)");
			}

			if (abonentid != 0)
			{
				sql = sql + " and b.abonent_id = :abonentid";
			}

			var oCom = new OracleCommand(sql);
			oCom.Parameters.Add("period", OracleDbType.Date, period, ParameterDirection.Input);
			oCom.Parameters.Add("filter", OracleDbType.Varchar2, filter, ParameterDirection.Input);
			if (abonentid != 0)
			{
				oCom.Parameters.Add("abonentid", OracleDbType.Varchar2, abonentid, ParameterDirection.Input);
			}

			return OracleDbPull.GetDataTable<List<Bill>>(
				oCom,
				(resultDataTable, serviceResult) =>
				{
					var billRaw = (
						from row in resultDataTable.AsEnumerable()
						select new
						{
							Phone = row.Field<String>("PHONE"),
							Abonent = row.Field<String>("ABONENT_NAME"),
							harge = (double)row.Field<Decimal>("NACH_SUM"),
							BillNumber = row.Field<String>("BILL_NUMBER"),
							BillDate = row.Field<DateTime>("BILL_DATE").ToShortDateString(),
							BillId = (Int64)row.Field<Decimal>("BILL_ID"),
							PhoneId = (Int64)row.Field<Decimal>("PHONE_ID"),
							AbonentId = (Int64?)row.Field<Decimal?>("ABONENT_ID")
						}
						).ToList();

					var bills = new List<Bill>();

					//     
					foreach (var row in billRaw)
					{
						var nb = new Bill { Phone = row.Phone, Abonent = row.Abonent, Charge = row.harge, BillNumber = row.BillNumber, BillDate = row.BillDate, BillId = row.BillId, PhoneId = row.PhoneId, AbonentId = row.AbonentId };

						bills.Add(nb);
					}

					serviceResult.DataOrNullOrDefValue = bills;
				}
				);

		}

		#endregion

		#region BillChargesGet

		internal static ServiceResult<List<Bill>> BillChargesGet(DateTime? period, string filter, bool strFilterIsPhone, long? abonentid)
		{
			String sql = @"
SELECT B.PHONE, B.ABONENT_NAME, B.sum_add, B.sum_comp, B.sum_ded, 
B.BILL_NUMBER, B.BILL_DATE, B.BILL_ID, B.PHONE_ID, B.ABONENT_ID, 
 bill_charges_id 
FROM V_BA_BILL_CHARGES_GRID B 
where trunc(B.BILL_DATE, 'MM') = :period
{0}
";
			filter = "%" + filter + "%";

			if (strFilterIsPhone)
			{
				sql = String.Format(sql, " and phone like :filter");
			}
			else
			{
				sql = String.Format(sql, " and upper(abonent_name) like upper(:filter)");
			}

			if (abonentid != 0)
			{
				sql = sql + " and b.abonent_id = :abonentid";
			}

			var oCom = new OracleCommand(sql);
			oCom.Parameters.Add("period", OracleDbType.Date, period, ParameterDirection.Input);
			oCom.Parameters.Add("filter", OracleDbType.Varchar2, filter, ParameterDirection.Input);
			if (abonentid != 0)
			{
				oCom.Parameters.Add("abonentid", OracleDbType.Varchar2, abonentid, ParameterDirection.Input);
			}


			return OracleDbPull.GetDataTable<List<Bill>>(
				oCom,
				(resultDataTable, serviceResult) =>
				{
					var billRaw = (
						from row in resultDataTable.AsEnumerable()
						select new
						{
							Phone = row.Field<String>("PHONE"),
							Abonent = row.Field<String>("ABONENT_NAME"),
							harge = (double?)row.Field<Decimal?>("sum_add"),
							Compensated = (double?)row.Field<Decimal?>("sum_comp"),
							Retantion = (double?)row.Field<Decimal?>("sum_ded"),
							BillNumber = row.Field<String>("BILL_NUMBER"),
							BillDate = row.Field<DateTime>("BILL_DATE").ToShortDateString(),
							BillId = (Int64)row.Field<Decimal>("BILL_ID"),
							PhoneId = (Int64)row.Field<Decimal>("PHONE_ID"),
							AbonentId = (Int64?)row.Field<Decimal?>("ABONENT_ID"),
							BillChargesId = (Int64)row.Field<Decimal>("bill_charges_id")
						}
						).ToList();

					var bills = new List<Bill>();

					//     
					foreach (var row in billRaw)
					{
						var nb = new Bill { Phone = row.Phone, Abonent = row.Abonent, Charge = row.harge, Compensated = row.Compensated, Retantion = row.Retantion.HasValue ? row.Retantion.ToString() : "", BillNumber = row.BillNumber, BillDate = row.BillDate, BillId = row.BillId, PhoneId = row.PhoneId, AbonentId = row.AbonentId, BillChargesId = row.BillChargesId, IsReadOnly = true, IsVisibleCancel = false };

						bills.Add(nb);
					}

					serviceResult.DataOrNullOrDefValue = bills;
				}
				);

		}

		#endregion
	}
}