//   .             !


using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Web;
using BillAnalyserWebApp.Code.IFlex.DB;
using CrossCompileSharing.Data;
using Oracle.DataAccess.Client;
using CrossCompileSharing.UiStrings;

namespace BillAnalyserWebApp.Code.Data
{
	/// <summary>
	/// Summary description for Abonents
	/// </summary>
	public static class Calls
	{
		#region CallGet

		/// <summary>
		///       ,    .
		/// </summary>
		public enum ViewModFor
		{
			NotInited,
			/// <summary>
			///   ,    ,       .
			/// </summary>
			Date,
			/// <summary>
			///             .
			/// </summary>
			Period,
		}

		/// <summary>
		///      .
		/// </summary>
		public enum FilterBy
		{
			NotInited,
			/// <summary>
			///   .
			/// </summary>
			Phone,
			/// <summary>
			///   .
			/// </summary>
			Abonent,
		}

		internal static ServiceResult<List<Call>> CallGet(DateTime? period, string filter_, FilterBy filterBy, String phoneBFilter_, ViewModFor viewModFor, long? abonentid)
		{

			var filterPrepared = "%" + filter_ + "%";
			var phoneBFilterPrepared = "%" + phoneBFilter_ + "%";

			String sql = @"
SELECT B.id, B.phones_id, B.call_date, B.phone_number, B.service_name, B.zone_ps, B.zone_vtk, B.dur, B.cost, B.cost_no_vat, B.abonent_name, B.phone, B.abonent_id, B.period_start_date, B.period_end_date 
FROM V_BA_CALLS_GRID B 
where
1=1
{0}
{1}
{2}
";

			String filterModeSql;
			switch (viewModFor)
			{
				case ViewModFor.Date:
					filterModeSql = "and trunc(B.call_date, 'MM') = :period";
					break;
				case ViewModFor.Period:
					filterModeSql = "and trunc(B.period_start_date, 'MM') = :period";
					break;
				default:
					throw new Exception(string.Format("  {0}     .", typeof(ViewModFor).Name));
			}

			String filterSql;
			switch (filterBy)
			{
				case FilterBy.Phone:
					filterSql = "and phone like :filter";
					break;
				case FilterBy.Abonent:
					filterSql = "and upper(abonent_name) like upper(:filter)";
					break;
				default:
					throw new Exception(string.Format("  {0}     .", typeof(FilterBy).Name));
			}

			var phoneBFilterSql = "and B.phone_number like :filterPhoneB";

			sql = String.Format(sql, filterModeSql, filterSql, phoneBFilterSql);

			if (abonentid != 0)
			{
				sql = sql + "\n and b.abonent_id = :abonentid";
			}

			var oCom = new OracleCommand(sql)
			{
				BindByName =  true
			};

			oCom.Parameters.Add("period", OracleDbType.Date, period, ParameterDirection.Input);
			oCom.Parameters.Add("filter", OracleDbType.Varchar2, filterPrepared, ParameterDirection.Input);
			oCom.Parameters.Add("filterPhoneB", OracleDbType.Varchar2, phoneBFilterPrepared, ParameterDirection.Input);
			if (abonentid != 0)
			{
				oCom.Parameters.Add("abonentid", OracleDbType.Varchar2, abonentid, ParameterDirection.Input);
			}

			return OracleDbPull.GetDataTable<List<Call>>(
				oCom,
				(resultDataTable, serviceResult) =>
				{
					var callRaw = (
						from row in resultDataTable.AsEnumerable()
						select new
						{
							id = (Int64)row.Field<Decimal>("id"),
							phones_id = (Int64)row.Field<Decimal>("phones_id"),
							call_date = row.Field<DateTime>("call_date")
								//.ToString("dd.MM.yy HH:mm")
								,
							phone_number = row.Field<String>("phone_number"),
							service_name = row.Field<String>("service_name"),
							zone_ps = row.Field<String>("zone_ps"),
							zone_vtk = row.Field<String>("zone_vtk"),
							dur = row.Field<String>("dur"),
							cost = row.Field<double>("cost"),
							cost_no_vat = row.Field<double>("cost_no_vat"),
							abonent_name = row.Field<String>("abonent_name"),
							phone = row.Field<String>("phone"),
							abonent_id = (Int64?)row.Field<Decimal?>("abonent_id")
						}
						).ToList();

					var calls = new List<Call>();

					//     
					foreach (var row in callRaw)
					{
						var nc = new Call { Id = row.id, PhoneId = row.phones_id, CallDate = row.call_date, PhoneB = row.phone_number, ServiceName = row.service_name, ZonePS = row.zone_ps, ZoneVTK = row.zone_vtk, Dur = row.dur, Cost = row.cost, CostNotVat = row.cost_no_vat, Abonent = row.abonent_name, AbonentPhone = row.phone, AbonentId = row.abonent_id };
						calls.Add(nc);
					}

					serviceResult.DataOrNullOrDefValue = calls;
				}
				);

		}

		#endregion

		#region CallGetByPhoneId

		internal static ServiceResult<List<Call>> CallGetByPhoneId(Int64 phoneId, DateTime period, Calls.ViewModFor viewModFor)
		{
			var sql = @"
SELECT B.id, B.phones_id, B.call_date, B.phone_number, B.service_name, B.zone_ps, B.zone_vtk, B.dur, B.cost, B.cost_no_vat, B.abonent_name, B.phone, B.abonent_id, B.period_start_date, B.period_end_date 
FROM V_BA_CALLS_GRID B 
where {0}
AND phones_id = :phoneId

order by B.call_date
";

			String filterModeSql;
			switch (viewModFor)
			{
				case ViewModFor.Date:
					filterModeSql = "trunc(B.call_date, 'MM') = :period";
					break;
				case ViewModFor.Period:
					filterModeSql = "trunc(B.period_start_date, 'MM') = :period";
					break;
				default:
					throw new Exception(string.Format("  {0}     .", typeof(ViewModFor).Name));
			}
			sql = String.Format(sql, filterModeSql);

			var oCom = new OracleCommand(sql);
			oCom.Parameters.Add("period", OracleDbType.Date, period, ParameterDirection.Input);
			oCom.Parameters.Add("phoneId", OracleDbType.Varchar2, phoneId, ParameterDirection.Input);

			return OracleDbPull.GetDataTable<List<Call>>(
				oCom,
				(resultDataTable, serviceResult) =>
				{
					var callRaw = (
						from row in resultDataTable.AsEnumerable()
						select new
						{
							id = (Int64)row.Field<Decimal>("id"),
							phones_id = (Int64)row.Field<Decimal>("phones_id"),
							call_date = row.Field<DateTime>("call_date")
								//.ToShortDateString()
								,
							phone_number = row.Field<String>("phone_number"),
							service_name = row.Field<String>("service_name"),
							zone_ps = row.Field<String>("zone_ps"),
							zone_vtk = row.Field<String>("zone_vtk"),
							dur = row.Field<String>("dur"),
							cost = row.Field<double>("cost"),
							cost_no_vat = row.Field<double>("cost_no_vat"),
							abonent_name = row.Field<String>("abonent_name"),
							phone = row.Field<String>("phone"),
							abonent_id = (Int64?)row.Field<Decimal?>("abonent_id")
						}
						).ToList();

					var calls = new List<Call>();

					//     
					foreach (var row in callRaw)
					{
						var nc = new Call { Id = row.id, PhoneId = row.phones_id, CallDate = row.call_date, PhoneB = row.phone_number, ServiceName = row.service_name, ZonePS = row.zone_ps, ZoneVTK = row.zone_vtk, Dur = row.dur, Cost = row.cost, CostNotVat = row.cost_no_vat, Abonent = row.abonent_name, AbonentPhone = row.phone, AbonentId = row.abonent_id };
						calls.Add(nc);
					}

					serviceResult.DataOrNullOrDefValue = calls;
				}
				);

		}

		#endregion
	}
}