//   .             !


using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using BillAnalyserWebApp.Code.IFlex.DB;
using CrossCompileSharing.Data;
using Oracle.DataAccess.Client;

namespace BillAnalyserWebApp.Code.Data
{
	public static class CompensationUnits
	{
		#region CompensationUnitsGet

		internal static ServiceResult<List<CompensationUnit>> CompensationUnitsGet(Int64 compensation_id)
		{
			const string sql = @"
                    select 
                        sc.id as service_comp_id, 
                        sa.id as service_agg_id, 
                        sc.SUMMA, 
                        sc.PERCENT, 
                        sa.NAME as service_agg_name
                    from (select *  
                            from v_ba_service_comps 
                            where compensations_id = :id_compensation) SC,
                        v_ba_service_agg sa 
                    WHERE sc.SERVICE_AGG_ID(+) = sa.id";

			var oCom = new OracleCommand(sql) { CommandType = CommandType.Text };
			oCom.Parameters.Add("id_compensation", OracleDbType.Int64, compensation_id, System.Data.ParameterDirection.Input);

			return OracleDbPull.GetDataTable<List<CompensationUnit>>(
				oCom,
				(resultDataTable, serviceResult) =>
				{
					var unitRow = (
						from row in resultDataTable.AsEnumerable()
						select new 
						{
							IdLinkToCompensation = (Int64?)row.Field<Decimal?>("service_comp_id"),
							IdAgg = (Int64?)row.Field<Decimal?>("service_agg_id"),
							Summa = (double?)row.Field<Decimal?>("SUMMA"),
							Percent = row.Field<Int16?>("PERCENT"),
							Name = row.Field<String>("service_agg_name")
						}
						).ToList();

					var compensationUnits = new List<CompensationUnit>();

					foreach(var row in unitRow)
					{
						var unit = new CompensationUnit { IdLinkToCompensation = row.IdLinkToCompensation, IdAgg = row.IdAgg, IdCompensation = compensation_id, Summa = row.Summa, Percent = row.Percent, Name = row.Name };
						compensationUnits.Add(unit);
					}

					serviceResult.DataOrNullOrDefValue = compensationUnits;
				}
				);
		}

		#endregion


	}
}