//   .             !


using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Reflection;
using System.Web;
using BillAnalyserWebApp.Code.IFlex.DB;
using BillAnalyserWebApp.Code.IFlex.DB.Oracle;
using CrossCompileSharing.Data;
using CrossCompileSharing.Log;
using CrossCompileSharing.UiStrings;
using Oracle.DataAccess.Client;
using System.Globalization;

namespace BillAnalyserWebApp.Code.Data
{
	public static class Compensations
	{
		#region    

		internal static ServiceResult<List<Compensation>> CompensationsGet(Int64 phoneId)
		{
			var oCom = new OracleCommand(@"
                select
                  ID
                , NAME
                , DSC
                , SUMMA
                , PERCENT
                , PHONES_ID AS PHONE_ID
                , phone_comps_id
                from
                  V_BA_PHONES_COMPENSATION
                WHERE
                  PHONES_ID = :PHONE_ID
                ");

			oCom.Parameters.Add("PHONE_ID", phoneId);

			return OracleDbPull.GetDataTable<List<Compensation>>(
				oCom,
				(resultDataTable, serviceResult) =>
				{
					serviceResult.DataOrNullOrDefValue = (
						from row in resultDataTable.AsEnumerable()
						select new Compensation
						{
							Id = (Int64)row.Field<Decimal>("ID"),
							Name = row.Field<String>("NAME"),
							Description = row.Field<String>("DSC"),
							Summa = (double?)row.Field<Decimal?>("SUMMA"),
							Percent = row.Field<Int16?>("PERCENT"),
							PhoneId = (Int64)row.Field<Decimal>("PHONE_ID"),
							PhoneCompsId = (Int64)row.Field<Decimal>("phone_comps_id"),
						}
						).ToList();
				}
				);
		}

		#endregion

		#region  
		internal static ServiceResult<Boolean> CalcCompensation(long id, DateTime period, bool recalc)
		{
			CompensationForCalc._countPosted++;
			//var dt = DateTime.Parse(period, new CultureInfo("Ru-RU", true));

			//         
			var command = new OracleCommand("ba_bills_iapi.calc_compensation") { CommandType = CommandType.StoredProcedure };

			command.Parameters.Add("i_abonent_id", OracleDbType.Int64, id, ParameterDirection.Input);
			command.Parameters.Add("i_date", OracleDbType.Date, period, ParameterDirection.Input);
			command.Parameters.Add("i_mode_id", OracleDbType.Int32, recalc, ParameterDirection.Input);

			return OracleDbPull.ExecuteNonQuery<Boolean>(
				command,
				serviceResult =>
				{
					serviceResult.DataOrNullOrDefValue = true;
					serviceResult.MessageFromServer = UiMessages.;
				}
				);
		}
		#endregion

		#region   
		internal static ServiceResult<List<Compensation>> GroupCompensationsGet()
		{
			var oCom = new OracleCommand(@"
                select
                  ID
                , NAME
                , DSC
                , SUMMA
                , PERCENT
                from
                  V_BA_COMPENSATIONS
                ");

			return OracleDbPull.GetDataTable<List<Compensation>>(
				oCom,
				(resultDataTable, serviceResult) =>
				{
					var compensations = (
						from row in resultDataTable.AsEnumerable()
						select new Compensation
						{
							Id = (Int64)row.Field<Decimal>("ID"),
							Name = row.Field<String>("NAME"),
							Description = row.Field<String>("DSC"),
							Summa = (double?)row.Field<Decimal?>("SUMMA"),
							Percent = row.Field<Int16?>("PERCENT")
						}
						).ToList();

					serviceResult.DataOrNullOrDefValue = compensations;
				}
				);
		}

		#endregion

		#region   
		//    BA_COMPENSATIONS_IAPI.ins_ba_compensations
		//------------------------------------------------------------------
		//--   
		//-- %param i_name         
		//-- %param i_dsc          
		//-- %param i_summa         
		//-- %param i_percent       
		//-- %param i_from_date    
		//-- %param i_to_date      
		//-- %param o_id            
		internal static ServiceResult<Int64> ins_ba_compensations()
		{
			const string dbName = "  ";

			var command = new OracleCommand("BA_COMPENSATIONS_IAPI.ins_ba_compensations") { CommandType = CommandType.StoredProcedure };

			command.Parameters.Add("i_name", OracleDbType.Varchar2, dbName, ParameterDirection.Input);
			command.Parameters.Add("i_dsc", OracleDbType.Varchar2, "", ParameterDirection.Input);
			command.Parameters.Add("i_summa", OracleDbType.Decimal, null, ParameterDirection.Input);
			command.Parameters.Add("i_percent", OracleDbType.Decimal, null, ParameterDirection.Input);
			command.Parameters.Add("i_from_date", OracleDbType.Date, null, ParameterDirection.Input);
			command.Parameters.Add("i_to_date", OracleDbType.Date, null, ParameterDirection.Input);
			command.Parameters.Add("i_id", OracleDbType.Int64, ParameterDirection.Output);

			return OracleDbPull.ExecuteNonQuery<Int64>(
				command,
				serviceResult =>
				{
					//   .
					serviceResult.DataOrNullOrDefValue = 0;

					if (command.Parameters["i_id"].Value != DBNull.Value)
					{
						try
						{
							serviceResult.DataOrNullOrDefValue = Convert.ToInt32(command.Parameters["i_id"].Value);
							serviceResult.MessageFromServer = UiMessages.____;
						}
						catch (Exception ex)
						{
							Logger.Write(ex);
							serviceResult.BusinessErrorNotNulled.Add(UiMessages.__________);
						}
					}
					else
					{
						Logger.Write(UiMessages.______________);
						serviceResult.BusinessErrorNotNulled.Add(UiMessages.______________);
					}
				},
				serviceResultOnError =>
				{
					serviceResultOnError.DataOrNullOrDefValue = 0;
					serviceResultOnError.BusinessErrorNotNulled.Add(UiMessages.______);
				}
				);

		}
		#endregion

		#region  

		//       
		//begin
		//  -- Call the procedure
		//  ba_compensations_iapi.trm_compensation_full(i_compensation_id => :i_compensation_id,
		//                                              i_from_date => :i_from_date);
		//end;
		internal static ServiceResult<Boolean> TrmCompensation(long id)
		{
			var command = new OracleCommand("ba_compensations_iapi.trm_compensation_full") { CommandType = CommandType.StoredProcedure };

			command.Parameters.Add("i_id", OracleDbType.Decimal, id, System.Data.ParameterDirection.Input);
			command.Parameters.Add("i_from_date", OracleDbType.Date, null, System.Data.ParameterDirection.Input);

			return OracleDbPull.ExecuteNonQuery<Boolean>(
				command,
				serviceResult =>
				{
					serviceResult.DataOrNullOrDefValue = true;
				},
				errorServiceResult =>
				{
					errorServiceResult.DataOrNullOrDefValue = false;
					errorServiceResult.BusinessErrorNotNulled.Add(UiMessages._____);
				}
				);
		}
		#endregion

		#region  
		//        begin
		//  -- Call the procedure
		//  ba_compensations_iapi.upd_compensation(i_compensation_id => :i_compensation_id,
		//                                         i_name => :i_name,
		//                                         i_summa => :i_summa,
		//                                         i_percent => :i_percent,
		//                                         i_dsc => :i_dsc,
		//                                         i_from_date => :i_from_date);
		//end; 
		internal static ServiceResult<Boolean> UpdCompensation(CommonCompensationData commonCompensationData, OracleDbPullInTransaction dbPullInTransaction)
		{
			var command = new OracleCommand("ba_compensations_iapi.upd_compensation") { CommandType = CommandType.StoredProcedure };

			command.Parameters.Add("i_compensations_id", OracleDbType.Decimal, commonCompensationData.Id, ParameterDirection.Input);
			command.Parameters.Add("i_name", OracleDbType.Varchar2, commonCompensationData.Name, ParameterDirection.Input);
			command.Parameters.Add("i_summa", OracleDbType.Decimal, commonCompensationData.Summa, ParameterDirection.Input);
			command.Parameters.Add("i_percent", OracleDbType.Decimal, commonCompensationData.Percent, ParameterDirection.Input);
			command.Parameters.Add("i_dsc", OracleDbType.Varchar2, commonCompensationData.Description, ParameterDirection.Input);
			command.Parameters.Add("i_from_date", OracleDbType.Date, null, ParameterDirection.Input);

			return dbPullInTransaction.ExecuteNonQuery<Boolean>(
				command,
				serviceResult =>
				{
					serviceResult.MessageFromServer = String.Format("  : \"{0}\"  ", commonCompensationData.Name);
					serviceResult.DataOrNullOrDefValue = true;
				},
				serviceResultOnError =>
				{
					serviceResultOnError.MessageFromServer = String.Format("  : \"{0}\"    :\n{1}", commonCompensationData.Name, serviceResultOnError.BusinessErrorNotNulled.GetAllErrorsText());
					serviceResultOnError.DataOrNullOrDefValue = false;
				}
				);
		}

		#endregion

		#region    
		//        begin
		//  -- Call the procedure
		//  ba_phone_comps_iapi.ins_ba_phone_comps(i_phones_id => :i_phones_id,
		//                                         i_compensations_id => :i_compensations_id,
		//                                         i_from_date => :i_from_date,
		//                                         i_to_date => :i_to_date,
		//                                         o_id => :o_id);
		//end;
		internal static ServiceResult<Int64> InsBaPhoneComps(long? phoneId, long? compensationId)
		{
			var command = new OracleCommand("ba_phone_comps_iapi.ins_ba_phone_comps") { CommandType = CommandType.StoredProcedure };

			command.Parameters.Add("i_phones_id", OracleDbType.Decimal, phoneId, ParameterDirection.Input);
			command.Parameters.Add("i_compensations_id", OracleDbType.Decimal, compensationId, ParameterDirection.Input);
			command.Parameters.Add("i_from_date", OracleDbType.Date, null, ParameterDirection.Input);
			command.Parameters.Add("i_to_date", OracleDbType.Date, null, ParameterDirection.Input);
			command.Parameters.Add("o_id", OracleDbType.Int64, ParameterDirection.Output);

			return OracleDbPull.ExecuteNonQuery<Int64>(
				command,
				serviceResult =>
				{
					try
					{
						serviceResult.DataOrNullOrDefValue = Convert.ToInt64(command.Parameters["o_id"].Value);
					}
					catch(Exception ex)
					{
						Logger.Write(ex);
						serviceResult.BusinessErrorNotNulled.Add(UiMessages.___);
					}
				}
				);
		}

		#endregion

		#region    
		//begin
		//  -- Call the procedure
		//  ba_phone_comps_iapi.Trm_Phone_Comps(i_phone_comps_id => :i_phone_comps_id,
		//                                      i_from_date => :i_from_date);
		//end;        
		internal static ServiceResult<bool> TrmPhoneComps(long? phoneCompID)
		{
			var command = new OracleCommand("ba_phone_comps_iapi.Trm_Phone_Comps") { CommandType = CommandType.StoredProcedure };

			command.Parameters.Add("i_phone_comps_id", OracleDbType.Decimal, phoneCompID, ParameterDirection.Input);
			command.Parameters.Add("i_from_date", OracleDbType.Date, null, ParameterDirection.Input);

			return OracleDbPull.ExecuteNonQuery<Boolean>(
				command,
				serviceResult =>
				{
					serviceResult.DataOrNullOrDefValue = true;
				}
				);
		}

		#endregion

		#region    
		//begin
		//  -- Call the procedure
		//  ba_service_comps_iapi.ins_ba_service_comps(i_compensations_id => :i_compensations_id,
		//                                             i_service_agg_id => :i_service_agg_id,
		//                                             i_summa => :i_summa,
		//                                             i_percent => :i_percent,
		//                                             i_from_date => :i_from_date,
		//                                             i_to_date => :i_to_date,
		//                                             o_id => :o_id);
		//end;
		internal static ServiceResult<CompensationUnit> InsBaServiceComps(CompensationUnit unit, OracleDbPullInTransaction dbPullInTransaction)
		{
			var command = new OracleCommand("ba_service_comps_iapi.ins_ba_service_comps") { CommandType = CommandType.StoredProcedure };

			command.Parameters.Add("i_compensations_id", OracleDbType.Decimal, unit.IdCompensation, ParameterDirection.Input);
			command.Parameters.Add("i_service_agg_id", OracleDbType.Decimal, unit.IdAgg, ParameterDirection.Input);
			command.Parameters.Add("i_summa", OracleDbType.Decimal, unit.Summa, ParameterDirection.Input);
			command.Parameters.Add("i_percent", OracleDbType.Decimal, unit.Percent, ParameterDirection.Input);
			command.Parameters.Add("i_from_date", OracleDbType.Date, null, ParameterDirection.Input);
			command.Parameters.Add("i_to_date", OracleDbType.Date, null, ParameterDirection.Input);
			command.Parameters.Add("o_id", OracleDbType.Int64, ParameterDirection.Output);

			return dbPullInTransaction.ExecuteNonQuery<CompensationUnit>(
				command,
				serviceResult =>
				{
					serviceResult.DataOrNullOrDefValue = unit;
					serviceResult.MessageFromServer = UiMessages._____;
					try
					{
						serviceResult.DataOrNullOrDefValue.IdLinkToCompensation = Convert.ToInt64(command.Parameters["o_id"].Value);
					}
					catch (Exception ex)
					{
						Logger.Write(ex);
						serviceResult.AddBusinessError(UiMessages.___);
						serviceResult.MessageFromServer = serviceResult.BusinessErrorNotNulled.GetAllErrorsText();
					}
				},
				serviceResultOnError =>
				{
					serviceResultOnError.DataOrNullOrDefValue = unit;
					serviceResultOnError.MessageFromServer = serviceResultOnError.BusinessErrorNotNulled.GetAllErrorsText();
				}
				);
		}

		#endregion

		#region    
		//        begin
		//  -- Call the procedure
		//  ba_service_comps_iapi.upd_service_comp(i_service_comp_id => :i_service_comp_id,
		//                                         i_compensations_id => :i_compensations_id,
		//                                         i_serivce_group_id => :i_serivce_group_id,
		//                                         i_summa => :i_summa,
		//                                         i_percent => :i_percent,
		//                                         i_from_date => :i_from_date);
		//end;
		internal static ServiceResult<CompensationUnit> UpdServiceComp(CompensationUnit unit, OracleDbPullInTransaction dbPullInTransaction)
		{
			var command = new OracleCommand("ba_service_comps_iapi.upd_service_comp") { CommandType = CommandType.StoredProcedure };

			command.Parameters.Add("i_service_comp_id", OracleDbType.Decimal, unit.IdLinkToCompensation, ParameterDirection.Input);
			command.Parameters.Add("i_compensations_id", OracleDbType.Decimal, unit.IdCompensation, ParameterDirection.Input);
			command.Parameters.Add("i_serivce_group_id", OracleDbType.Decimal, unit.IdAgg, ParameterDirection.Input);
			command.Parameters.Add("i_summa", OracleDbType.Decimal, unit.Summa, ParameterDirection.Input);
			command.Parameters.Add("i_percent", OracleDbType.Decimal, unit.Percent, ParameterDirection.Input);
			command.Parameters.Add("i_from_date", OracleDbType.Date, null, ParameterDirection.Input);

			return dbPullInTransaction.ExecuteNonQuery<CompensationUnit>(
				command,
				serviceResult =>
				{
					serviceResult.DataOrNullOrDefValue = unit;
					serviceResult.MessageFromServer = UiMessages.___;
				},
				serviceResultOnError =>
				{
					serviceResultOnError.DataOrNullOrDefValue = unit;
					serviceResultOnError.MessageFromServer = serviceResultOnError.BusinessErrorNotNulled.GetAllErrorsText();
				}
				);
		}

		#endregion

		#region    
		//begin
		//  -- Call the procedure
		//  ba_service_comps_iapi.trm_service_comp(i_service_comp_id => :i_service_comp_id,
		//                                         i_from_date => :i_from_date);
		//end;  
		internal static ServiceResult<CompensationUnit> TrmServiceComp(CompensationUnit compensationUnit, OracleDbPullInTransaction dbPullInTransaction)
		{
			var command = new OracleCommand("ba_service_comps_iapi.trm_service_comp") { CommandType = CommandType.StoredProcedure };
			command.Parameters.Add("i_service_comp_id", OracleDbType.Decimal, compensationUnit.IdLinkToCompensation, ParameterDirection.Input);
			command.Parameters.Add("i_from_date", OracleDbType.Date, null, ParameterDirection.Input);

			return dbPullInTransaction.ExecuteNonQuery<CompensationUnit>(
				command,
				serviceResult =>
				{
					serviceResult.DataOrNullOrDefValue = compensationUnit;
					serviceResult.MessageFromServer = UiMessages._____;
				},
				serviceResultOnError =>
				{
					serviceResultOnError.DataOrNullOrDefValue = compensationUnit;
					serviceResultOnError.MessageFromServer = serviceResultOnError.BusinessErrorNotNulled.GetAllErrorsText();
				}
				);
		}

		#endregion
	}
}