/// <summary>
/// Tuzova L. Processing group objects information.
/// </summary>

using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using BillAnalyserWebApp.Code.IFlex.DB;
using CrossCompileSharing.Data;
using Oracle.DataAccess.Client;

namespace BillAnalyserWebApp.Code.Data
{
	public static class GroupReports
	{
		#region GroupGettingMethod

		internal static ServiceResult<List<GroupReport>> GroupsGettingReport()
		{
			/* Define select command */
			var queryData = new OracleCommand(@"SELECT G.ID as ID, G.ABONENT_ID as ABONENT_ID, NVL(G.NAME,'') as NAME from V_BA_GROUPS G");
			
			/* Getting data */
			return OracleDbPull.GetDataTable<List<GroupReport>>(
				queryData,
				(resultDataTable, serviceResult) =>
				{
					/* List of Groups */
					var groupRaw = (
						from row in resultDataTable.AsEnumerable()
						select new
						{
							Id = (Int64)row.Field<Decimal>("ID"),
							ParentId = (Int64?)row.Field<Decimal?>("ABONENT_ID"),
							Name = row.Field<String>("NAME")
						}
						).ToList();


					var groupDict = new Dictionary<Int64, GroupReport>();

					foreach (var row in groupRaw)
					{
						if (
							!groupDict.ContainsKey(row.Id))
						{
							groupDict.Add(row.Id, new GroupReport
							{
								Id = row.Id,
								ParentId = row.ParentId,
								Name = row.Name,
							});
						}
					}

					var groups = new List<GroupReport>();
					foreach (var keyVal in groupDict)
					{
						groups.Add(keyVal.Value);
					}

					serviceResult.DataOrNullOrDefValue = groups;
				}
				);
		}
		#endregion
	}
}