using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using BillAnalyserWebApp.Code.Data;
using BillAnalyserWebApp.Code.IFlex.DB;
using CrossCompileSharing.Data;
using CrossCompileSharing.UiStrings;
using Oracle.DataAccess.Client;
using Oracle.DataAccess.Types;
using System.Data;

//      
//   ..

namespace BillAnalyserWebApp.Code.Interop1C
{
    public class ImportEmp
    {
        #warning     !!!
        //       ,     .
		// it3xl.ru            UI,           .      .       ,      ,       .

        private static List<ImportEmployeesReport> _listForInserting = new List<ImportEmployeesReport>();
        private static long _numberItems;

		/// <summary>
		///    ... . ( : .   ).
		/// </summary>
		/// <param name="fileUniqueKey"></param>
		/// <returns></returns>
		public static ServiceResult<List<Employee>> ProcessEmployeesFromUploadedFile(String fileUniqueKey)
        {
			var fileInfo = FileManager.GetFileInfo(fileUniqueKey);

			var empList = ParseEmployeesFromFile(fileInfo);
			FileManager.DeleteFile(fileUniqueKey);

			return SaveEmpListToDB(empList);
        }

		private static List<Employee> ParseEmployeesFromFile(FileInfo fileInfo)
		{
			var employees = new List<Employee>();
			using (var streamReader = new StreamReader(fileInfo.OpenRead()))
			{
				do
				{
					var line = streamReader.ReadLine();
					if (line == null)
					{
						break;
					}
					var split = line.Split(';');

					employees.Add(new Employee
					{
						IsGroup = Int32.Parse(split[0]),
						Code = split[1],
						ParentCode = split[2],

						// BA-113     1 !!!!
						//   ,    ,    .
						Name = String.IsNullOrEmpty(split[3]) ? UiMessages. : split[3],

						Post = split[4],
						Phone = split[5],
					});

				} while (true);
			}

			return employees;
		}

		/// <summary>
		///         _listForInserting
		/// </summary>
		/// <returns></returns>
		public static ServiceResult<List<Employee>> SaveToDbFromStaticList()
        {
            if (_listForInserting == null)
                return null;

            ServiceResult<List<Employee>> listEmployeeResult;
            try
            {
                var listEmployeeInput = new List<Employee>();
				foreach (var e in _listForInserting)
				{
					var ne = new Employee
					{
						IsGroup = 0,
						Code = "",
						ParentCode = "",
						Name = e.Name,
						Post = e.Post,
						Phone = e.Phone,

						Email = e.Email,

						UserLogin = e.UserLogin,
						UserPassword = e.UserPassword,
						CostCenter = e.CostCenter,
						ArbitraryParameter1 = e.ArbitraryParameter1,
						ArbitraryParameter2 = e.ArbitraryParameter2,
						ArbitraryParameter3 = e.ArbitraryParameter3,
						ArbitraryParameter4 = e.ArbitraryParameter4,
						LimitRuble = e.LimitRuble,
						LimitPercent = e.LimitPercent,
					};
					listEmployeeInput.Add(ne);
				}

                listEmployeeResult = SaveEmpListToDB(listEmployeeInput);
            }
            catch (Exception ex)
            {
                listEmployeeResult = null;
            }
            finally
            {
                //    
                _listForInserting = null;
            }

            return listEmployeeResult;
        }



        #region         
        public static ServiceResult<Boolean> PrepareInsert(long numberItems)
        {
            _listForInserting = new List<ImportEmployeesReport>();
            _numberItems = numberItems;

            return new ServiceResult<bool> { DataOrNullOrDefValue = true, };
        }

        #endregion

        #region         
        public static ServiceResult<bool> AddPartToList(List<ImportEmployeesReport> partList)
        {
            if (_listForInserting == null || partList == null)
            {
                return new ServiceResult<bool> { DataOrNullOrDefValue = false };
            }

            _listForInserting.AddRange(partList);

            //      
            if (_listForInserting.Count == _numberItems)
            {
                return new ServiceResult<bool> { DataOrNullOrDefValue = true };
            }

            return new ServiceResult<bool> { DataOrNullOrDefValue = false };
        }
        #endregion

        #region         .     EmpList
		/// <summary>
		///         .     EmpList.
		/// </summary>
		/// <param name="empList"></param>
		/// <returns></returns>
        private static ServiceResult<List<Employee>> SaveEmpListToDB(List<Employee> empList)
        {
            //   
            var oCom = new OracleCommand();
            oCom.CommandType = CommandType.StoredProcedure;
            oCom.BindByName = true;
            oCom.CommandText = "BA_ABONENTS_IMP_IAPI.ImportAbonents";


            ///////////////////////////////////
            var isGroupList = new List<int>();
            var codeList = new List<String>();
            var parentList = new List<String>();
            var nameList = new List<String>();
            var postList = new List<String>();
            var phoneList = new List<String>();
			var emailList = new List<String>();

			var userLoginList = new List<String>();
			var userPasswordList = new List<String>();
			var costCenterList = new List<String>();
			var arbitraryParameter1List = new List<String>();
			var arbitraryParameter2List = new List<String>();
			var arbitraryParameter3List = new List<String>();
			var arbitraryParameter4List = new List<String>();
			var limitRubleList = new List<Int32?>();
			var limitPercentList = new List<Int32?>();

            foreach (Employee item in empList)
            {
                isGroupList.Add(item.IsGroup);
                codeList.Add(item.Code);
                parentList.Add(item.ParentCode);
                nameList.Add(item.Name);
                postList.Add(item.Post);
                phoneList.Add(item.Phone);
				emailList.Add(item.Email);

				userLoginList.Add(item.UserLogin);
				userPasswordList.Add(item.UserPassword);
				costCenterList.Add(item.CostCenter);
				arbitraryParameter1List.Add(item.ArbitraryParameter1);
				arbitraryParameter2List.Add(item.ArbitraryParameter2);
				arbitraryParameter3List.Add(item.ArbitraryParameter3);
				arbitraryParameter4List.Add(item.ArbitraryParameter4);
				limitRubleList.Add(item.LimitRuble);
				limitPercentList.Add(item.LimitPercent);
            }

            //  plsql 
			oCom.Parameters.Add(new OracleParameter
			{
				ParameterName = "i_isGroupList",
				OracleDbType = OracleDbType.Int32,
				Direction = ParameterDirection.Input,
				CollectionType = OracleCollectionType.PLSQLAssociativeArray,
				Value = isGroupList.ToArray(),
				Size = isGroupList.Count,
			});

			oCom.Parameters.Add(new OracleParameter
			{
				ParameterName = "i_CodeList",
				OracleDbType = OracleDbType.Varchar2,
				Direction = ParameterDirection.Input,
				CollectionType = OracleCollectionType.PLSQLAssociativeArray,
				Value = codeList.ToArray(),
				Size = codeList.Count,
			});

			oCom.Parameters.Add(new OracleParameter
			{
				ParameterName = "i_NameList",
				OracleDbType = OracleDbType.Varchar2,
				Direction = ParameterDirection.Input,
				CollectionType = OracleCollectionType.PLSQLAssociativeArray,
				Value = nameList.ToArray(),
				Size = nameList.Count,
			});

			oCom.Parameters.Add(new OracleParameter
			{
				ParameterName = "i_ParentList",
				OracleDbType = OracleDbType.Varchar2,
				Direction = ParameterDirection.Input,
				CollectionType = OracleCollectionType.PLSQLAssociativeArray,
				Value = parentList.ToArray(),
				Size = parentList.Count,
			});

			oCom.Parameters.Add(new OracleParameter
			{
				ParameterName = "i_PostList",
				OracleDbType = OracleDbType.Varchar2,
				Direction = ParameterDirection.Input,
				CollectionType = OracleCollectionType.PLSQLAssociativeArray,
				Value = postList.ToArray(),
				Size = postList.Count,
			});

			oCom.Parameters.Add(new OracleParameter
			{
				ParameterName = "i_PhoneList",
				OracleDbType = OracleDbType.Varchar2,
				Direction = ParameterDirection.Input,
				CollectionType = OracleCollectionType.PLSQLAssociativeArray,
				Value = phoneList.ToArray(),
				Size = phoneList.Count,
			});

			oCom.Parameters.Add(new OracleParameter
			{
				ParameterName = "i_EmailList",
				OracleDbType = OracleDbType.Varchar2,
				Direction = ParameterDirection.Input,
				CollectionType = OracleCollectionType.PLSQLAssociativeArray,
				Value = emailList.ToArray(),
				Size = emailList.Count,
			});

			oCom.Parameters.Add(new OracleParameter
			{
				ParameterName = "i_login",
				OracleDbType = OracleDbType.Varchar2,
				Direction = ParameterDirection.Input,
				CollectionType = OracleCollectionType.PLSQLAssociativeArray,
				Value = userLoginList.ToArray(),
				Size = userLoginList.Count,
			});

			oCom.Parameters.Add(new OracleParameter
			{
				ParameterName = "i_password",
				OracleDbType = OracleDbType.Varchar2,
				Direction = ParameterDirection.Input,
				CollectionType = OracleCollectionType.PLSQLAssociativeArray,
				Value = userPasswordList.ToArray(),
				Size = userPasswordList.Count,
			});

			oCom.Parameters.Add(new OracleParameter
			{
				ParameterName = "i_virtual_group",
				OracleDbType = OracleDbType.Varchar2,
				Direction = ParameterDirection.Input,
				CollectionType = OracleCollectionType.PLSQLAssociativeArray,
				Value = costCenterList.ToArray(),
				Size = costCenterList.Count,
			});

			oCom.Parameters.Add(new OracleParameter
			{
				ParameterName = "i_add_field1",
				OracleDbType = OracleDbType.Varchar2,
				Direction = ParameterDirection.Input,
				CollectionType = OracleCollectionType.PLSQLAssociativeArray,
				Value = arbitraryParameter1List.ToArray(),
				Size = arbitraryParameter1List.Count,
			});

			oCom.Parameters.Add(new OracleParameter
			{
				ParameterName = "i_add_field2",
				OracleDbType = OracleDbType.Varchar2,
				Direction = ParameterDirection.Input,
				CollectionType = OracleCollectionType.PLSQLAssociativeArray,
				Value = arbitraryParameter2List.ToArray(),
				Size = arbitraryParameter2List.Count,
			});

			oCom.Parameters.Add(new OracleParameter
			{
				ParameterName = "i_add_field3",
				OracleDbType = OracleDbType.Varchar2,
				Direction = ParameterDirection.Input,
				CollectionType = OracleCollectionType.PLSQLAssociativeArray,
				Value = arbitraryParameter3List.ToArray(),
				Size = arbitraryParameter3List.Count,
			});

			oCom.Parameters.Add(new OracleParameter
			{
				ParameterName = "i_add_field4",
				OracleDbType = OracleDbType.Varchar2,
				Direction = ParameterDirection.Input,
				CollectionType = OracleCollectionType.PLSQLAssociativeArray,
				Value = arbitraryParameter4List.ToArray(),
				Size = arbitraryParameter4List.Count,
			});

			oCom.Parameters.Add(new OracleParameter
			{
				ParameterName = "i_limit_rub",
				OracleDbType = OracleDbType.Int32,
				Direction = ParameterDirection.Input,
				CollectionType = OracleCollectionType.PLSQLAssociativeArray,
				Value = limitRubleList.ToArray(),
				Size = limitRubleList.Count,
			});

			oCom.Parameters.Add(new OracleParameter
			{
				ParameterName = "i_limit_per",
				OracleDbType = OracleDbType.Int32,
				Direction = ParameterDirection.Input,
				CollectionType = OracleCollectionType.PLSQLAssociativeArray,
				Value = limitPercentList.ToArray(),
				Size = limitPercentList.Count,
			});


            //    Output PLSQL-
            OracleParameter o_ResList = oCom.Parameters.Add("o_ResList", OracleDbType.Varchar2, ParameterDirection.Output);
            o_ResList.CollectionType = OracleCollectionType.PLSQLAssociativeArray;
            o_ResList.Size = empList.Count;
            o_ResList.ArrayBindSize = new Int32[empList.Count];
            o_ResList.ArrayBindStatus = new OracleParameterStatus[empList.Count];
            for (int i = 0; i < empList.Count; i++)
            {
                o_ResList.ArrayBindSize[i] = 2000;
                o_ResList.ArrayBindStatus[i] = OracleParameterStatus.Success;
            }

            OracleParameter o_IdList = oCom.Parameters.Add("o_IdList", OracleDbType.Varchar2, ParameterDirection.Output);
            o_IdList.CollectionType = OracleCollectionType.PLSQLAssociativeArray;
            o_IdList.Size = empList.Count;
            o_IdList.ArrayBindSize = new Int32[empList.Count];
            o_IdList.ArrayBindStatus = new OracleParameterStatus[empList.Count];
            for (int i = 0; i < empList.Count; i++)
            {
                o_IdList.ArrayBindSize[i] = 32;
                o_IdList.ArrayBindStatus[i] = OracleParameterStatus.Success;
            }

            ServiceResult<List<Employee>> result = OracleDbPull.ExecuteNonQuery<List<Employee>>(
            oCom,
            serviceResult =>
            {
                serviceResult.DataOrNullOrDefValue = empList;
                //       
                for (var i = 0; i <= ((OracleString[])o_ResList.Value).Length - 1; i++)
                {
                    if (!((OracleString[])o_ResList.Value)[i].IsNull
                        && ((OracleString[])o_ResList.Value)[i].Length > 0
                            && ((OracleString[])o_ResList.Value)[i] != "")
                    {
                        serviceResult.DataOrNullOrDefValue[i].Dsc = ((OracleString[])o_ResList.Value)[i].Value;
                    }
                    if (!((OracleString[])o_IdList.Value)[i].IsNull
                        && ((OracleString[])o_IdList.Value)[i].Length > 0
                            && ((OracleString[])o_IdList.Value)[i] != "")
                    {
                        serviceResult.DataOrNullOrDefValue[i].Id = ((OracleString[])o_IdList.Value)[i].Value;
                    }
                }
            },
			errorResult =>
			{
				errorResult.DataOrNullOrDefValue = new List<Employee> { new Employee { Name = errorResult.BusinessErrorNotNulled.GetAllErrorsText()} };
				//errorResult.
			}
            );

            return result;
        }
        #endregion
    }
}