

//  it3xl.ru  .


using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using BillAnalyserWebApp.Code.IFlex.DB;
using BillAnalyserWebApp.Code.IFlex.DB.Oracle;
using CrossCompileSharing.Log;
using Oracle.DataAccess.Client;
using CrossCompileSharing.UiStrings;
using CrossCompileSharing.Data;

namespace BillAnalyserWebApp.Code.Data
{
	public static class OrgUnits
	{
		internal static ServiceResult<List<OrgUnit>> OrgUnitsGet()
		{
			var oCom = new OracleCommand(@"
SELECT
  A.ID
, A.ABONENT_ID
, A.NAME
, A.IS_GROUP
, A.EMAIL
, A.POST
, A.ROLES_ID
, A.EXPORT_TYPE
--     V_BA_ABONENTS, ..   A.CODE_1     
, A.CODE_1C
, A.LOGIN

, P.ID AS PHONE_ID
, P.PHONE
, P.TARIF

, A.VIRTUAL_GROUP

, A.ADD_FIELD1
, A.ADD_FIELD2
, A.ADD_FIELD3
, A.ADD_FIELD4

FROM
  V_BA_ABONENTS A
, V_BA_PHONES P
WHERE
  A.ID = P.ABONENT_ID (+)
");
			// it3xl.ru     DataSet   .       .      DataSet,     
			// it3xl.ru          1,  ID1
			return OracleDbPull.GetDataTable<List<OrgUnit>>(
				oCom,
				(resultDataTable, serviceResult) =>
				{
					// it3xl.ru        - 
					var orgUnitRawMultiByPhones = (
						from row in resultDataTable.AsEnumerable()
						select new
						{
							Id = (Int64)row.Field<Decimal>("ID"),
							ParentId = (Int64?)row.Field<Decimal?>("ABONENT_ID"),
							Name = row.Field<String>("NAME"),
							IsGroup = row.Field<Int16>("IS_GROUP") == 1,
							Email = row.Field<String>("EMAIL"),
							Appointment = row.Field<String>("POST"),
							RoleId = (Int32?)row.Field<Decimal?>("ROLES_ID"),
							ExportType = (Int32?)row.Field<Int16?>("EXPORT_TYPE"),
							Code1C = row.Field<String>("CODE_1C"),
							Login = row.Field<String>("LOGIN"),

							PhoneId = (Int64?)row.Field<Decimal?>("PHONE_ID"),
							Phone = row.Field<String>("PHONE"),
							PhoneTarif = row.Field<String>("TARIF"),

							CostCenter = row.Field<String>("VIRTUAL_GROUP"),

							ArbitraryParameter1 = row.Field<String>("ADD_FIELD1"),
							ArbitraryParameter2 = row.Field<String>("ADD_FIELD2"),
							ArbitraryParameter3 = row.Field<String>("ADD_FIELD3"),
							ArbitraryParameter4 = row.Field<String>("ADD_FIELD4"),
						}
						).ToList();

					// it3xl.ru              
					var orgUnitsDict = new Dictionary<Int64, OrgUnit>();
					foreach (var row in orgUnitRawMultiByPhones)
					{
						if (
							!orgUnitsDict.ContainsKey(row.Id))
						{
							orgUnitsDict.Add(row.Id, new OrgUnit
							{
								Id = row.Id,
								ParentId = row.ParentId,
								Name = row.Name,
								IsGroup = row.IsGroup,
								Email = row.Email,
								Appointment = row.Appointment,
								RoleId = row.RoleId,
								ExportType = row.ExportType,
								Code1C = row.Code1C,
								Login = row.Login,

								CostCenter = row.CostCenter,

								ArbitraryParameter1 = row.ArbitraryParameter1,
								ArbitraryParameter2 = row.ArbitraryParameter2,
								ArbitraryParameter3 = row.ArbitraryParameter3,
								ArbitraryParameter4 = row.ArbitraryParameter4,
							});
						}

						if (row.PhoneId.HasValue)
						{
							orgUnitsDict[row.Id].Phones.Add(new Phone
							{
								Id = row.PhoneId.Value,
								PhoneStr = row.Phone,
								Tarif = row.PhoneTarif,
							});
						}
					}

					// it3xl.ru   .         ,     .      .
					//List<OrgUnit> orgUnits = new List<OrgUnit>();
					//foreach (KeyValuePair<Int64, OrgUnit> keyVal in orgUnitsDict)
					//{
					//    orgUnits.Add(keyVal.Value);
					//}

					// it3xl.ru  .        ,      ,        .
					// it3xl.ru       
					var orgUnits = (
						from el in orgUnitsDict.Values
						//    http://iliya-tretyakov.spaces.live.com/blog/cns!6DDCA30C11FA5050!503.entry
						let dummy = el.Phones = el.Phones.OrderBy(p => p.PhoneStr).ToList()
						orderby el.ParentId,
							// it3xl.ru   false   true,   descending
							el.IsGroup descending,
							el.Name
						select el
						).ToList();


					serviceResult.DataOrNullOrDefValue = orgUnits;
				}
				);

		}


		/// <summary>
		///    .
		///     .
		/// </summary>
		/// <param name="orgUnit"></param>
		/// <param name="dbInTransaction"></param>
		/// <returns></returns>
		internal static ServiceResult<Boolean> OrgUnitUpdate(OrgUnit orgUnit, OracleDbPullInTransaction dbInTransaction)
		{
			// it3xl.ru   ,       UI     

			var oCom = new OracleCommand { CommandType = CommandType.StoredProcedure, CommandText = "BA_ABONENTS_IAPI.upd_abonent" };
			//oCom.BindByName = true;

			// . 
			oCom.Parameters.Add(new OracleParameter("i_id", OracleDbType.Decimal, orgUnit.Id, ParameterDirection.Input));
			//  - 
			oCom.Parameters.Add(new OracleParameter("i_abonent_id", OracleDbType.Decimal, orgUnit.ParentId, ParameterDirection.Input));
			//    
			oCom.Parameters.Add(new OracleParameter("i_name", OracleDbType.Varchar2, orgUnit.Name, ParameterDirection.Input));
			//  
			oCom.Parameters.Add(new OracleParameter("i_is_group", OracleDbType.Int32, orgUnit.IsGroup, ParameterDirection.Input));
			//  - 
			oCom.Parameters.Add(new OracleParameter("i_roles_id", OracleDbType.Int32, orgUnit.RoleId, ParameterDirection.Input));
			// 
			oCom.Parameters.Add(new OracleParameter("i_post", OracleDbType.Varchar2, orgUnit.Appointment, ParameterDirection.Input));
			// E-Mail
			oCom.Parameters.Add(new OracleParameter("i_email", OracleDbType.Varchar2, orgUnit.Email, ParameterDirection.Input));
			//    
			oCom.Parameters.Add(new OracleParameter("i_pass_num", OracleDbType.Varchar2, null, ParameterDirection.Input));
			//   
			oCom.Parameters.Add(new OracleParameter("i_pass_date", OracleDbType.Date, null, ParameterDirection.Input));
			//  
			oCom.Parameters.Add(new OracleParameter("i_pass_who", OracleDbType.Varchar2, null, ParameterDirection.Input));
			//  
			oCom.Parameters.Add(new OracleParameter("i_address", OracleDbType.Varchar2, null, ParameterDirection.Input));
			//    1
			oCom.Parameters.Add(new OracleParameter("i_code_1", OracleDbType.Varchar2, orgUnit.Code1C, ParameterDirection.Input));
			//   (1-  ,2-)
			oCom.Parameters.Add(new OracleParameter("i_export_type", OracleDbType.Int32, orgUnit.ExportType, ParameterDirection.Input));
			// 
			oCom.Parameters.Add(new OracleParameter("i_login", OracleDbType.Varchar2, orgUnit.Login, ParameterDirection.Input));
			//  
			oCom.Parameters.Add(new OracleParameter("i_from_date", OracleDbType.Date, null, ParameterDirection.Input));

			oCom.Parameters.Add(new OracleParameter("i_virtual_group", OracleDbType.Varchar2, orgUnit.CostCenter, ParameterDirection.Input));

			oCom.Parameters.Add(new OracleParameter("i_add_field1", OracleDbType.Varchar2, orgUnit.ArbitraryParameter1, ParameterDirection.Input));
			oCom.Parameters.Add(new OracleParameter("i_add_field2", OracleDbType.Varchar2, orgUnit.ArbitraryParameter2, ParameterDirection.Input));
			oCom.Parameters.Add(new OracleParameter("i_add_field3", OracleDbType.Varchar2, orgUnit.ArbitraryParameter3, ParameterDirection.Input));
			oCom.Parameters.Add(new OracleParameter("i_add_field4", OracleDbType.Varchar2, orgUnit.ArbitraryParameter4, ParameterDirection.Input));

			var updateAbonentResult = dbInTransaction.ExecuteNonQuery<Boolean>(oCom, null);
			if (updateAbonentResult.IsDone == false)
			{
				// !!!

				return updateAbonentResult;
			}

			if (orgUnit.IsGroup == false
					&& String.IsNullOrEmpty(orgUnit.Password) == false)
			{
				var oComPwd = new OracleCommand { CommandType = CommandType.StoredProcedure, CommandText = "BA_ABONENTS_IAPI.set_pswd" };

				oComPwd.Parameters.Add("i_id", orgUnit.Id);
				oComPwd.Parameters.Add("i_password", orgUnit.Password);

				oComPwd.Parameters.Add("i_from_date", null);

				var r = dbInTransaction.ExecuteNonQuery<Boolean>(oComPwd, null);
				if (r.IsDone == false)
				{

					return updateAbonentResult;
				}
			}

			return ServiceResult.GetTrueServiceResult();
		}

		internal static ServiceResult<OrgUnit> OrgUnitCreate(OrgUnit orgUnit)
		{
			orgUnit.Name = UiMessages.;
			if (
				!orgUnit.IsGroup)
			{
				//       
				orgUnit.RoleId = 1;
			}


			var oCom = new OracleCommand { CommandType = CommandType.StoredProcedure, CommandText = "BA_ABONENTS_IAPI.ins_ba_abonents" };

			//  - 
			oCom.Parameters.Add(new OracleParameter("i_abonent_id", OracleDbType.Decimal, orgUnit.ParentId, ParameterDirection.Input));
			//    
			oCom.Parameters.Add(new OracleParameter("i_name", OracleDbType.Varchar2, orgUnit.Name, ParameterDirection.Input));
			//  
			oCom.Parameters.Add(new OracleParameter("i_is_group", OracleDbType.Int32, orgUnit.IsGroup, ParameterDirection.Input));
			//  - 
			oCom.Parameters.Add(new OracleParameter("i_roles_id", OracleDbType.Int32, orgUnit.RoleId, ParameterDirection.Input));
			// 
			oCom.Parameters.Add(new OracleParameter("i_post", OracleDbType.Varchar2, null, ParameterDirection.Input));
			// E-Mail
			oCom.Parameters.Add(new OracleParameter("i_email", OracleDbType.Varchar2, null, ParameterDirection.Input));
			//    
			oCom.Parameters.Add(new OracleParameter("i_pass_num", OracleDbType.Varchar2, null, ParameterDirection.Input));
			//   
			oCom.Parameters.Add(new OracleParameter("i_pass_date", OracleDbType.Date, null, ParameterDirection.Input));
			//  
			oCom.Parameters.Add(new OracleParameter("i_pass_who", OracleDbType.Varchar2, null, ParameterDirection.Input));
			//  
			oCom.Parameters.Add(new OracleParameter("i_address", OracleDbType.Varchar2, null, ParameterDirection.Input));
			//    1
			oCom.Parameters.Add(new OracleParameter("i_code_1", OracleDbType.Varchar2, null, ParameterDirection.Input));
			//   (1-  ,2-)
			oCom.Parameters.Add(new OracleParameter("i_export_type", OracleDbType.Int32, null, ParameterDirection.Input));
			// 
			oCom.Parameters.Add(new OracleParameter("i_login", OracleDbType.Varchar2, null, ParameterDirection.Input));

			// it3xl.ru    
			// 
			oCom.Parameters.Add(new OracleParameter("i_password", OracleDbType.Varchar2, null, ParameterDirection.Input));

			//  
			oCom.Parameters.Add(new OracleParameter("i_from_date", OracleDbType.Date, null, ParameterDirection.Input));
			//  
			oCom.Parameters.Add(new OracleParameter("i_to_date", OracleDbType.Date, null, ParameterDirection.Input));

			oCom.Parameters.Add(new OracleParameter("i_virtual_group", OracleDbType.Varchar2, null, ParameterDirection.Input));

			oCom.Parameters.Add(new OracleParameter("i_add_field1", OracleDbType.Varchar2, null, ParameterDirection.Input));
			oCom.Parameters.Add(new OracleParameter("i_add_field2", OracleDbType.Varchar2, null, ParameterDirection.Input));
			oCom.Parameters.Add(new OracleParameter("i_add_field3", OracleDbType.Varchar2, null, ParameterDirection.Input));
			oCom.Parameters.Add(new OracleParameter("i_add_field4", OracleDbType.Varchar2, null, ParameterDirection.Input));

			//   
			oCom.Parameters.Add(new OracleParameter("o_id", OracleDbType.Int64, ParameterDirection.Output));

			return OracleDbPull.ExecuteNonQuery<OrgUnit>(
				oCom,
				serviceResult =>
				{
					serviceResult.DataOrNullOrDefValue = orgUnit;
					//  Id  
					try
					{
						orgUnit.Id = (Int64)oCom.Parameters["o_id"].Value;
					}
					catch (Exception ex)
					{
						Logger.Write(ex);
						serviceResult.AddBusinessError(UiMessages.___);
					}
				}
				);
		}

		internal static ServiceResult<Boolean> OrgUnitDelete(IEnumerable<OrgUnit> orgUnitsForDeleting)
		{
			// it3xl.ru   ,       UI  ,   .
			var db = new OracleDbPullInTransaction();

			using (db)
			{
				foreach (var orgUnit in orgUnitsForDeleting)
				{
					var unsetPhoneResult = Phones.UserPhoneLinksSet(orgUnit.Phones, null, db);
					if (unsetPhoneResult.IsDone == false)
					{
						//!!!

						return unsetPhoneResult;
					}

					//    
					var oCom = new OracleCommand
					{
						CommandType = CommandType.StoredProcedure,
						CommandText = "BA_ABONENTS_IAPI.trm_abonent",
					};
					oCom.Parameters.Add("i_id", orgUnit.Id);
					oCom.Parameters.Add("i_from_date", null);
					var deleteAbonentResult = db.ExecuteNonQuery<Boolean>(oCom, null);
					if (deleteAbonentResult.IsDone == false)
					{
						//!!!

						return unsetPhoneResult;
					}
				}

				db.Commit();
			}

			return new ServiceResult<Boolean> { DataOrNullOrDefValue = true, };
		}

		internal static ServiceResult<Boolean> OrgUnitAddToChilds(IEnumerable<OrgUnit> childOrgUnit, OrgUnit parentOrgUnit)
		{
			using (var dbInTransaction = new OracleDbPullInTransaction())
			{
				foreach (var orgUnit in childOrgUnit)
				{
					if (orgUnit.ParentId.HasValue == false)
					{

						return new ServiceResult<bool>(UiMessages.______);
					}

					orgUnit.ParentId = parentOrgUnit.Id;
					var lastResult = OrgUnitUpdate(orgUnit, dbInTransaction);
					if (lastResult.IsDone == false)
					{

						return lastResult;
					}
				}

				dbInTransaction.Commit();
			}

			return ServiceResult.GetTrueServiceResult();
		}
	}
}