/// <summary>
/// Tuzova L. Summary description for Periods
/// </summary>
/// 

using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using BillAnalyserWebApp.Code.IFlex.DB;
using CrossCompileSharing.Data;
using Oracle.DataAccess.Client;


namespace BillAnalyserWebApp.Code.Data
{
	public class PeriodReports
	{
		#region PeriodsGettingMethod
		internal static ServiceResult<List<PeriodReport>> PeriodsGettingReport()
		{
			/* Define select command */
			var queryData = new OracleCommand(@"SELECT P.BILL_DATE, NVL(P.PERIOD_NAME,'') as PERIOD_NAME FROM V_USED_PERIODS P ORDER BY P.BILL_DATE");

			/* Getting data */
			return OracleDbPull.GetDataTable<List<PeriodReport>>(
				queryData,
				(resultDataTable, serviceResult) =>
				{
					/* List of Services */
					var periodRaw = (
						from row in resultDataTable.AsEnumerable()
						select new
						{
							Period = row.Field<DateTime>("BILL_DATE"),
							PeriodName = row.Field<String>("PERIOD_NAME")
						}
						).ToList();

					var periods = new List<PeriodReport>();

					foreach (var item in periodRaw)
					{
						var currPeriod = new PeriodReport();
						currPeriod.Period = item.Period;
						currPeriod.PeriodName = item.PeriodName;
						periods.Add(currPeriod);
					}

					serviceResult.DataOrNullOrDefValue = periods;
				}
				);
		}
		#endregion
	}
}