

//  it3xl.ru  .


using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using BillAnalyserWebApp.Code.IFlex.DB;
using CrossCompileSharing.Data;
using Oracle.DataAccess.Client;

namespace BillAnalyserWebApp.Code.Data
{
	public static class Phones
	{
		#region UserPhoneLinkSet

		/// <summary>
		///       
		/// </summary>
		internal static ServiceResult<Boolean> UserPhoneLinkSet(Phone phone, OrgUnit orgUnit)
		{
			var oCom = new OracleCommand
			{
				CommandType = CommandType.StoredProcedure,
				CommandText = "BA_PHONES_IAPI.cr_user_phone_link"
			};

			oCom.Parameters.Add("i_phone_id", phone.Id);

			oCom.Parameters.Add("i_abonent_id"
				, orgUnit != null ?
				//     
					(Int64?)orgUnit.Id
				//         
					: null);

			oCom.Parameters.Add("i_from_date", null);

			return OracleDbPull.ExecuteNonQuery<Boolean>(
				oCom,
				serviceResult =>
				{
					serviceResult.DataOrNullOrDefValue = true;
				}
				);
		}

		#endregion


		#region UserPhoneLinksSet

		/// <summary>
		///       
		/// </summary>
		internal static ServiceResult<Boolean> UserPhoneLinksSetInTransaction(List<Phone> phones, OrgUnit orgUnit)
		{
			var db = new OracleDbPullInTransaction();

			// it3xl.ru   ,       UI     
			using (db)
			{
				var setPhonesResult = UserPhoneLinksSet(phones, orgUnit, db);
				if (setPhonesResult.IsDone == false)
				{
					// !!!

					return setPhonesResult;
				}
				db.Commit();
			}

			return new ServiceResult<bool> { DataOrNullOrDefValue = true, };
		}

		internal static ServiceResult<Boolean> UserPhoneLinksSet(List<Phone> phones, OrgUnit orgUnitOrNullForDelete, OracleDbPullInTransaction db)
		{
			foreach (var phone in phones)
			{
				var oCom = new OracleCommand { CommandType = CommandType.StoredProcedure, CommandText = "BA_PHONES_IAPI.cr_user_phone_link" };

				oCom.Parameters.Add("i_phone_id", phone.Id);

				oCom.Parameters.Add("i_abonent_id"
					, orgUnitOrNullForDelete != null ?
					//     
						(Int64?)orgUnitOrNullForDelete.Id
					//         
						: null);

				oCom.Parameters.Add("i_from_date", null);

				var serviceResult = db.ExecuteNonQuery<Boolean>(oCom, null);
				if (serviceResult.IsDone == false)
				{

					return serviceResult;
				}
			}

			return ServiceResult.GetTrueServiceResult();
		}

		#endregion

		#region UserPhones

		/// <summary>
		///     
		/// </summary>
		internal static ServiceResult<List<Phone>> UserPhones(Int64 idAbonent)
		{
			var sql = String.Format(
				@"SELECT B.ID, B.PHONE 
FROM V_BA_PHONES B 
WHERE B.ABONENT_ID = :idAbonent"
				);
			var oCom = new OracleCommand(sql);

			oCom.Parameters.Add("idAbonent", idAbonent);

			return OracleDbPull.GetDataTable<List<Phone>>(
				oCom,
				(resultDataTable, serviceResult) =>
				{
					var phonesRaw = (
						from row in resultDataTable.AsEnumerable()
						select new
						{
							Id = (Int64)row.Field<Decimal>("ID"),
							Phone = row.Field<String>("PHONE"),
						}
						).ToList();

					var phones = new List<Phone>();

					//     
					foreach (var row in phonesRaw)
					{
						var np = new Phone { Id = row.Id, PhoneStr = row.Phone };

						phones.Add(np);
					}

					serviceResult.DataOrNullOrDefValue = phones;
				}
				);

		}

		#endregion
	}
}