/// <summary>
/// Tuzova L. Report#1 "By employees"
/// </summary>

using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Web;
using BillAnalyserWebApp.Code.IFlex.DB;
using CrossCompileSharing.Data;
using Oracle.DataAccess.Client;

namespace BillAnalyserWebApp.Code.Data
{
	public class ReportByEmps
	{
		#region ReportByEmpGettingMethod
		internal static ServiceResult<ReportByEmp> ReportByEmpGetting(Int64? depGroupID, String accGroupID, Int64? subsID, Int64? serviceID, DateTime? startPeriod, DateTime? endPeriod, Dictionary<String, String> listColumn)
		{
			/* Get group IDs */
			String _accGroupID = accGroupID;
			Int64? _depGroupID = depGroupID;

			var _listColumn = listColumn;
			var _subsID = subsID;
			var _serviceID = serviceID;

			var _startPeriod = startPeriod;
			var _endPeriod = endPeriod;

			String startDate = String.Format("{0:dd.MM.yyyy}", _startPeriod);
			String endDate = String.Format("{0:dd.MM.yyyy}", _endPeriod);

			/* Check if service group is defined */
			if (serviceID == null)
			{
				/* Remove column "dur" from listColumns*/
				listColumn.Remove("dur");
			}

			/* Define selected columns row */
			String sqlColumns = "";

			foreach (var item in listColumn)
			{
				if (item.Key.Contains("department"))
				{

					sqlColumns = sqlColumns + ", ba_reports_iapi.get_abonent_group(abonent_id) as " + @"""" + item.Value + @"""";
				}
				else if (item.Key.Contains("phone"))
				{
					/* Phone ID IS defined */
					if (_subsID != null)
					{
						sqlColumns = sqlColumns + ", at.phone as " + @"""" + item.Value + @"""";
					}
					/* Department group */
					else if (_depGroupID != null)
					{
						sqlColumns = sqlColumns + ", ba_reports_iapi.get_phone_list_by_abonent(abonent_id) as " + @"""" + item.Value + @"""";
					}
					/* Account group */
					else
					{
						sqlColumns = sqlColumns + ", ba_reports_iapi.get_phone_list_by_abonent_acc(abonent_id, account) as " + @"""" + item.Value + @"""";
					}
				}
				else if (item.Key.Contains("compensation_group"))
				{
					/* Phone ID IS defined */
					if (_subsID != null)
					{
						sqlColumns = sqlColumns + ", ba_reports_iapi.get_comp_by_phone(phone_id) as " + @"""" + item.Value + @"""";
					}
					/* Department group */
					else if (_depGroupID != null)
					{
						sqlColumns = sqlColumns + ", ba_reports_iapi.get_comp_by_abonent(abonent_id) as " + @"""" + item.Value + @"""";
					}
					/* Account group */
					else
					{
						sqlColumns = sqlColumns + ", ba_reports_iapi.get_comp_by_abonent_acc(abonent_id, account) as " + @"""" + item.Value + @"""";
					}
				}
				else if (item.Key.Contains("tarif"))
				{
					/* Phone ID IS defined */
					if (_subsID != null)
					{
						sqlColumns = sqlColumns + ", at.tarif as " + @"""" + item.Value + @"""";
					}
					/* Department group */
					else if (_depGroupID != null)
					{
						sqlColumns = sqlColumns + ", ba_reports_iapi.get_tarifs_by_abonent(abonent_id) as " + @"""" + item.Value + @"""";
					}
					/* Account group */
					else
					{
						sqlColumns = sqlColumns + ", ba_reports_iapi.get_tarifs_by_abonent_acc(abonent_id,account) as " + @"""" + item.Value + @"""";
					}
				}
				else
				{
					sqlColumns = sqlColumns + ", at." + item.Key + " as " + @"""" + item.Value + @"""";
				}
			}

			/* Remove first symbols */
			sqlColumns = sqlColumns.Remove(0, 2);

			/* Define query */
			var oCom = new OracleCommand();
			oCom.CommandType = CommandType.Text;
			oCom.BindByName = true;

			/* Subscriber IS defined */
			if (_subsID != null)
			{
				oCom.CommandText =
					"SELECT " + sqlColumns + (@" FROM
					(SELECT a.abonent_id as abonent_id, 
                            nvl(a.abonent_name,'') as abonent_name,  
							nvl(SUM(a.SUM_ADD),0) as sum_add,
							nvl(SUM(a.SUM_DED),0) as sum_ded, 
                            nvl(SUM(a.SUM_COMP),0) as sum_comp,
							nvl(sum(a.dur),0) as dur, 
                            nvl(a.email,'') as email, 
							decode(a.user_role,1,'','') as user_role, 
                            nvl(a.post,'') as post, 
							nvl(a.phone,'') as phone,
                            a.phone_id as phone_id, 
							nvl(a.tarif_name,'') as tarif, 
                            nvl(a.account_name,'') as account,
                            nvl(a.VIRTUAL_GROUP,'') as VIRTUAL_GROUP, 
                            nvl(a.ADD_FIELD1,'') as ADD_FIELD1, 
                            nvl(a.ADD_FIELD2,'') as ADD_FIELD2,
                            nvl(a.ADD_FIELD3,'') as ADD_FIELD3,
                            nvl(a.ADD_FIELD4,'') as ADD_FIELD4,
                            round(sum(a.add_percent),2)*100 as add_percent
                    FROM (select z.*,
                                (case
                                    when sum(z.sum_add) over() = 0 then
                                    100
                                    else
                                    z.sum_add / sum(z.sum_add) over()
                                end) as add_percent
                            from v_rep_services_abonents z
                            WHERE z.phone_id = (:phone_id)
                            and --    
                                z.agg_id = nvl(:agg_id, z.agg_id)
                            and --       
                                TRUNC(z.bill_date) between
                                TO_DATE(:start_date, 'dd.mm.yyyy') and
                                TO_DATE(:end_date, 'dd.mm.yyyy') --     
                        ) a
                    GROUP BY a.abonent_id,
                            a.abonent_name,
                            a.email,
                            a.user_role,
                            a.post,
                            a.phone,
                            a.tarif_name,
                            a.account_name,
                            a.phone_id,
                            a.virtual_group, 
                            a.add_field1, 
                            a.add_field2, 
                            a.add_field3, 
                            a.add_field4
					ORDER BY UPPER(abonent_name))at");
				oCom.Parameters.Add("phone_id", OracleDbType.Int64, _subsID, ParameterDirection.Input);
				oCom.Parameters.Add("agg_id", OracleDbType.Int64, _serviceID, ParameterDirection.Input);
				oCom.Parameters.Add("start_date", OracleDbType.Varchar2, startDate, ParameterDirection.Input);
				oCom.Parameters.Add("end_date", OracleDbType.Varchar2, endDate, ParameterDirection.Input);
			}
			/* Subscriber is NOT defined */
			else
			{
				/* Department group */
				if (_depGroupID != null)
				{
					oCom.CommandText =
						"SELECT " + sqlColumns + (@" FROM
						(SELECT a.abonent_id, 
                                nvl(a.ABONENT_NAME,'') as abonent_name, 
								nvl(SUM(a.SUM_ADD),0) as sum_add,
                                nvl(SUM(a.SUM_DED),0) as sum_ded,
                                nvl(SUM(a.SUM_COMP),0) as sum_comp,
                                nvl(sum(a.dur),0) as dur,
								nvl(a.email,'') as email, 
                                decode(a.user_role,1,'','') as user_role, 
                                nvl(a.post,'') as post,
                                a.virtual_group, 
                                a.add_field1, 
                                a.add_field2, 
                                a.add_field3, 
                                a.add_field4,
                                round(sum(a.add_percent),2)*100 as add_percent
                                  FROM (select z.*,
                                               (case
                                                 when sum(z.sum_add) over() = 0 then
                                                  100
                                                 else
                                               z.sum_add / sum(z.sum_add) over()
                                               end) as add_percent
                                          from v_rep_services_abonents z,
                                               table(ba_reports_iapi.get_allabonents_by_group(:org_id)) t --  ,   
                                         where z.agg_id = nvl(:agg_id, z.agg_id)
                                                                 and --       
                                                                                           TRUNC(z.bill_date) between
                                                                                           TO_DATE(:start_date, 'dd.mm.yyyy') and
                                                                                           TO_DATE(:end_date, 'dd.mm.yyyy') --     
                                            
                                           and z.abonent_id = t.n) a
                                 GROUP BY a.abonent_id, a.ABONENT_NAME, a.email, a.user_role, a.post, a.virtual_group, 
                        a.add_field1, 
                        a.add_field2, 
                        a.add_field3, 
                        a.add_field4
						ORDER BY UPPER(abonent_name))at");
					oCom.Parameters.Add("org_id", OracleDbType.Int64, _depGroupID, ParameterDirection.Input);
					oCom.Parameters.Add("agg_id", OracleDbType.Int64, _serviceID, ParameterDirection.Input);
					oCom.Parameters.Add("start_date", OracleDbType.Varchar2, startDate, ParameterDirection.Input);
					oCom.Parameters.Add("end_date", OracleDbType.Varchar2, endDate, ParameterDirection.Input);
				}
				/* Account group */
				else
				{
					oCom.CommandText =
						"SELECT " + sqlColumns + (@" FROM
							(SELECT a.abonent_id as abonent_id, 
                                    nvl(a.ABONENT_NAME,'') as abonent_name,
									nvl(a.email,'') as email, 
                                    decode(a.user_role,1,'','') as user_role, 
									nvl(a.post,'') as post, 
                                    a.account_name as account,
                                    nvl(a.VIRTUAL_GROUP,'') as VIRTUAL_GROUP, 
                                    nvl(a.ADD_FIELD1,'') as ADD_FIELD1, 
                                    nvl(a.ADD_FIELD2,'') as ADD_FIELD2,
                                    nvl(a.ADD_FIELD3,'') as ADD_FIELD3,
                                    nvl(a.ADD_FIELD4,'') as ADD_FIELD4,
                                    nvl(SUM(a.SUM_ADD),0) as sum_add,
									nvl(SUM(a.SUM_DED),0) as sum_ded, 
									nvl(SUM(a.SUM_COMP),0) as sum_comp,
                                    nvl(sum(a.dur),0) as dur,
                                    round(sum(a.add_percent),2)*100 as add_percent
                                FROM (select z.*,
                                            (case
                                                when sum(z.sum_add) over() = 0 then
                                                100
                                                else
                                                round(z.sum_add / sum(z.sum_add) over(), 2) * 100
                                            end) as add_percent
                                        from v_rep_services_abonents z
                                        WHERE z.agg_id = nvl(:agg_id, z.agg_id)
                                        and TRUNC(z.bill_date) between
                                            TO_DATE(:start_date, 'dd.mm.yyyy') and
                                            TO_DATE(:end_date, 'dd.mm.yyyy')
                                            and z.account_name = :account_id) a
							GROUP BY a.abonent_id, a.ABONENT_NAME, a.email, a.user_role, a.post, a.account_name, a.VIRTUAL_GROUP, a.ADD_FIELD1, a.ADD_FIELD2,a.ADD_FIELD3,a.ADD_FIELD4
							ORDER BY UPPER(abonent_name))at ");

					oCom.Parameters.Add("agg_id", OracleDbType.Int64, _serviceID, ParameterDirection.Input);
					oCom.Parameters.Add("start_date", OracleDbType.Varchar2, startDate, ParameterDirection.Input);
					oCom.Parameters.Add("end_date", OracleDbType.Varchar2, endDate, ParameterDirection.Input);
					oCom.Parameters.Add("account_id", OracleDbType.Varchar2, _accGroupID, ParameterDirection.Input);
				}
			}
			/* Getting data */
			return OracleDbPull.GetDataTable<ReportByEmp>(
				oCom,
				(resultDataTable, serviceResult) =>
				{
					/* Output report data */

					/* Final output structure */
					var reportDict = new ReportByEmp();

					/* List of record values list */
					var recordValues = new List<List<Object>>();

					/* List of columns */
					var columnList = new List<String>();

					/* Column values for all records */
					foreach (DataRow dr in resultDataTable.Rows)
					{
						/* Values of current row */
						var currRecordValues = new List<Object>();
						foreach (DataColumn dc in resultDataTable.Columns)
						{
							/* Value of current column */
							Object recordValue = dr[dc];
							if (recordValue is DBNull)
							{
								currRecordValues.Add("");
							}
							else
							{
								currRecordValues.Add(recordValue);
							}
						}
						recordValues.Add(currRecordValues);
					}


					/* Define list of column names */
					for (int i = 0; i < resultDataTable.Columns.Count; i++)
					{
						var columnName = resultDataTable.Columns[i].ColumnName;
						columnList.Add(columnName.ToString());
					}

					/* Define output structure */
					reportDict.ColumnNames = columnList;
					reportDict.AbonentReportData = recordValues;

					serviceResult.DataOrNullOrDefValue = reportDict;
				}
				);

		}
		#endregion
	}
}