/// <summary>
/// Tuzova L. Summary description for ReportByGroups
/// </summary>

using System;
using System.Collections.Generic;
using System.Data;
using BillAnalyserWebApp.Code.IFlex.DB;
using CrossCompileSharing.Data;
using Oracle.DataAccess.Client;

namespace BillAnalyserWebApp.Code.Data
{
	public class ReportByGroups
	{
		#region ReportByGroupGetting
		internal static ServiceResult<ReportByEmp> ReportByGroupGetting(String accGroupID, Int64? depGroupID, Int64? serviceID, DateTime? startPeriod, DateTime? endPeriod, Dictionary<String, String> listColumn, Boolean groupLevel)
		{
			/* Variables */
			var _listColumn = listColumn;
			var _accGroupID = accGroupID;

			var _depGroupID = depGroupID;
			var _serviceID = serviceID;

			var _startPeriod = startPeriod;
			var _endPeriod = endPeriod;

			var startDate = String.Format("{0:dd.MM.yyyy}", _startPeriod);
			var endDate = String.Format("{0:dd.MM.yyyy}", _endPeriod);

			/* Check if service group is defined */
			if (serviceID == null)
			{
				/* Remove column "dur" from listColumns*/
				listColumn.Remove("dur");
			}

			/* Define selected columns row */
			var sqlColumns = "";
			foreach (var item in listColumn)
			{
				sqlColumns = sqlColumns + ", at." + item.Key + " as " + @"""" + item.Value + @"""";
			}

			/* Remove first symbols */
			sqlColumns = sqlColumns.Remove(0, 2);

			/* Define query */

			var oCom = new OracleCommand { CommandType = CommandType.Text, BindByName = true };

			/* GROUP TYPE DEFINITION */

			/* DEPARTMENT group*/
			if (_depGroupID != null)
			{

				/* Data WITH next level */
				if (groupLevel == true)
				{
					/* Service IS defined */
					if (_serviceID != null)
					{
						oCom.CommandText = String.Format(
							@"
SELECT {0} 
FROM (SELECT gr.name as ABONENT_NAME, SUM(a.SUM_ADD) as sum_add, 
sum(a.sum_ded) as sum_ded, sum(a.sum_add - a.sum_ded) as sum_comp,sum(a.dur) as dur 
FROM v_rep_services_Abonents a, 
v_ba_groups gr, 
(select b.id, substr(gr_id,instr(gr_id, ';', 1, 2) + 1, 
decode(instr(gr_id, ';', 1, 3),0,length(gr_id) + 1,instr(gr_id, ';', 1, 3)) - instr(gr_id, ';', 1, 2) - 1) as gr_id, 
b.is_group, LEV from (select a.id, level as lev, sys_connect_by_path(id, ';') gr_id, is_group 
FROM v_ba_abonents a 
connect by prior id = abonent_id 
start with id = :depGroupID) b 
where b.lev > 1) lim 
where lim.gr_id = gr.id 
and lim.is_group = 0 
and a.abonent_id = lim.id 
and a.agg_id = :agg_id 
and a.bill_date between TO_DATE(:start_date,'dd.mm.yyyy') and TO_DATE(:end_date,'dd.mm.yyyy') 
group by gr.name) at", sqlColumns);

						oCom.Parameters.Add("depGroupID", OracleDbType.Int64, _depGroupID, ParameterDirection.Input);
						oCom.Parameters.Add("agg_id", OracleDbType.Int64, _serviceID, ParameterDirection.Input);
						oCom.Parameters.Add("start_date", OracleDbType.Varchar2, startDate, ParameterDirection.Input);
						oCom.Parameters.Add("end_date", OracleDbType.Varchar2, endDate, ParameterDirection.Input);
					}

						/* Service is NOT defined */
					else
					{
						oCom.CommandText = String.Format(
							@"SELECT {0} 
FROM (SELECT gr.name as ABONENT_NAME, SUM(a.SUM_ADD) as sum_add, 
sum(a.sum_ded) as sum_ded, sum(a.sum_add - a.sum_ded) as sum_comp,sum(a.dur) as dur 
FROM v_rep_services_Abonents a, 
v_ba_groups gr, 
(select b.id, substr(gr_id,instr(gr_id, ';', 1, 2) + 1, 
decode(instr(gr_id, ';', 1, 3),0,length(gr_id) + 1,instr(gr_id, ';', 1, 3)) - instr(gr_id, ';', 1, 2) - 1) as gr_id, 
b.is_group, LEV from (select a.id, level as lev, sys_connect_by_path(id, ';') gr_id, is_group 
FROM v_ba_abonents a 
connect by prior id = abonent_id 
start with id = :depGroupID) b 
where b.lev > 1) lim 
where lim.gr_id = gr.id 
and lim.is_group = 0 
and a.abonent_id = lim.id 
and a.bill_date between TO_DATE(:start_date,'dd.mm.yyyy') and TO_DATE(:end_date,'dd.mm.yyyy') 
group by gr.name) at", sqlColumns);

						oCom.Parameters.Add("depGroupID", OracleDbType.Int64, _depGroupID, ParameterDirection.Input);
						oCom.Parameters.Add("start_date", OracleDbType.Varchar2, startDate, ParameterDirection.Input);
						oCom.Parameters.Add("end_date", OracleDbType.Varchar2, endDate, ParameterDirection.Input);
					}
				}


					/* Data WITHOUT next level */

				else
				{
					/* Service IS defined */
					if (_serviceID != null)
					{
						oCom.CommandText = String.Format(
							@"SELECT {0} 
FROM (SELECT g.name as ABONENT_NAME, SUM(a.SUM_ADD) as sum_add, 
sum(a.sum_ded) as sum_ded,sum(a.sum_add - a.sum_ded) as sum_comp,sum(a.dur) as dur 
from v_rep_services_Abonents a, 
table(ba_reports_iapi.get_allabonents_by_group(:depGroupID)) t, 
v_ba_groups g 
WHERE a.abonent_id = t.n 
and a.agg_id = :agg_id 
and a.bill_date between TO_DATE(:start_date,'dd.mm.yyyy') and TO_DATE(:end_date,'dd.mm.yyyy') 
and g.id =:depGroupID 
GROUP BY g.name) at", sqlColumns);

						oCom.Parameters.Add("depGroupID", OracleDbType.Int64, _depGroupID, ParameterDirection.Input);
						oCom.Parameters.Add("agg_id", OracleDbType.Int64, _serviceID, ParameterDirection.Input);
						oCom.Parameters.Add("start_date", OracleDbType.Varchar2, startDate, ParameterDirection.Input);
						oCom.Parameters.Add("end_date", OracleDbType.Varchar2, endDate, ParameterDirection.Input);
					}

						/* Service is NOT defined */

					else
					{
						oCom.CommandText = String.Format(
							@"SELECT {0} 
FROM (SELECT g.name as ABONENT_NAME, SUM(a.SUM_ADD) as sum_add, 
sum(a.sum_ded) as sum_ded,sum(a.sum_add - a.sum_ded) as sum_comp,sum(a.dur) as dur 
from v_rep_services_Abonents a, 
table(ba_reports_iapi.get_allabonents_by_group(:depGroupID)) t, 
v_ba_groups g 
WHERE a.abonent_id = t.n 
and a.bill_date between TO_DATE(:start_date,'dd.mm.yyyy') and TO_DATE(:end_date,'dd.mm.yyyy') 
and g.id =:depGroupID 
GROUP BY g.name) at", sqlColumns);

						oCom.Parameters.Add("depGroupID", OracleDbType.Int64, _depGroupID, ParameterDirection.Input);
						oCom.Parameters.Add("start_date", OracleDbType.Varchar2, startDate, ParameterDirection.Input);
						oCom.Parameters.Add("end_date", OracleDbType.Varchar2, endDate, ParameterDirection.Input);
					}
				}
			}
			/* ACCOUNT group*/
			else if (_accGroupID != null)
			{
				/* Data WITH next level */
				if (groupLevel == true)
				{
					/* Service IS defined */
					if (_serviceID != null)
					{
						oCom.CommandText = String.Format(
							@"SELECT {0} 
FROM (SELECT SUM(a.SUM_ADD) as sum_add, 
SUM(a.SUM_DED) as sum_ded, 
sum(a.SUM_ADD-a.SUM_ded) as sum_comp, 
sum(a.dur) as dur, 
a.account as account 
FROM v_rep_service_acc a 
WHERE a.service_agg_id = :agg_id 
and a.bill_date between TO_DATE(:start_date,'dd.mm.yyyy') and TO_DATE(:end_date,'dd.mm.yyyy') 
GROUP BY a.account) at", sqlColumns);
						oCom.Parameters.Add("agg_id", OracleDbType.Int64, _serviceID, ParameterDirection.Input);
						oCom.Parameters.Add("start_date", OracleDbType.Varchar2, startDate, ParameterDirection.Input);
						oCom.Parameters.Add("end_date", OracleDbType.Varchar2, endDate, ParameterDirection.Input);
					}

						/* Service is NOT defined */
					else
					{
						oCom.CommandText = String.Format(
							@"SELECT {0} 
FROM (SELECT SUM(a.SUM_ADD) as sum_add, 
SUM(a.SUM_DED) as sum_ded, 
sum(a.SUM_ADD-a.SUM_ded) as sum_comp, 
sum(a.dur) as dur, 
a.account as account 
FROM v_rep_service_acc a 
WHERE 
a.bill_date between TO_DATE(:start_date,'dd.mm.yyyy') and TO_DATE(:end_date,'dd.mm.yyyy') " +

																"GROUP BY a.account) at", sqlColumns);
						oCom.Parameters.Add("start_date", OracleDbType.Varchar2, startDate, ParameterDirection.Input);
						oCom.Parameters.Add("end_date", OracleDbType.Varchar2, endDate, ParameterDirection.Input);
					}
				}

					/* Data WITHOUT next level */
				else
				{
					/* Service IS defined */
					if (_serviceID != null)
					{
						/* Account name IS  */
						if (_accGroupID.Contains("  "))
						{
							oCom.CommandText = String.Format(
								@"SELECT {0} 
FROM (SELECT SUM(a.SUM_ADD) as sum_add, 
SUM(a.SUM_DED) as sum_ded, 
sum(a.SUM_ADD-a.SUM_ded) as sum_comp, 
sum(a.dur) as dur, 
'  ' as account 
FROM v_rep_service_acc a 
WHERE a.service_agg_id = :agg_id 
and a.bill_date between TO_DATE(:start_date,'dd.mm.yyyy') and TO_DATE(:end_date,'dd.mm.yyyy')) at", sqlColumns);
							oCom.Parameters.Add("agg_id", OracleDbType.Int64, _serviceID, ParameterDirection.Input);
							oCom.Parameters.Add("start_date", OracleDbType.Varchar2, startDate, ParameterDirection.Input);
							oCom.Parameters.Add("end_date", OracleDbType.Varchar2, endDate, ParameterDirection.Input);
						}

						/* Account name IS NOT  */
						else
						{
							oCom.CommandText = String.Format(
								@"SELECT {0} 
FROM (SELECT SUM(a.SUM_ADD) as sum_add, 
SUM(a.SUM_DED) as sum_ded, 
sum(a.SUM_ADD-a.SUM_ded) as sum_comp, 
sum(a.dur) as dur, 
a.account as account 
FROM v_rep_service_acc a 
WHERE a.service_agg_id = :agg_id 
and a.account = :account_name 
and a.bill_date between TO_DATE(:start_date,'dd.mm.yyyy') and TO_DATE(:end_date,'dd.mm.yyyy') 
GROUP BY a.account) at", sqlColumns);
							oCom.Parameters.Add("agg_id", OracleDbType.Int64, _serviceID, ParameterDirection.Input);
							oCom.Parameters.Add("account_name", OracleDbType.Varchar2, _accGroupID, ParameterDirection.Input);
							oCom.Parameters.Add("start_date", OracleDbType.Varchar2, startDate, ParameterDirection.Input);
							oCom.Parameters.Add("end_date", OracleDbType.Varchar2, endDate, ParameterDirection.Input);
						}
					}

						/* Service is NOT defined */
					else
					{
						/* Account name IS  */
						if (_accGroupID.Contains("  "))
						{
							oCom.CommandText = String.Format(
								@"SELECT {0} 
FROM (SELECT SUM(a.SUM_ADD) as sum_add, 
SUM(a.SUM_DED) as sum_ded, 
sum(a.SUM_ADD-a.SUM_ded) as sum_comp, 
sum(a.dur) as dur, 
'  ' as account 
FROM v_rep_service_acc a 
WHERE 
a.bill_date between TO_DATE(:start_date,'dd.mm.yyyy') and TO_DATE(:end_date,'dd.mm.yyyy')) at", sqlColumns);
							oCom.Parameters.Add("start_date", OracleDbType.Varchar2, startDate, ParameterDirection.Input);
							oCom.Parameters.Add("end_date", OracleDbType.Varchar2, endDate, ParameterDirection.Input);
						}

							/* Account name IS NOT  */
						else
						{
							oCom.CommandText = String.Format(
								@"SELECT {0} 
FROM (SELECT SUM(a.SUM_ADD) as sum_add, 
SUM(a.SUM_DED) as sum_ded, 
sum(a.SUM_ADD-a.SUM_ded) as sum_comp, 
sum(a.dur) as dur, 
a.account as account 
FROM v_rep_service_acc a 
WHERE 
a.account = :account_name 
and a.bill_date between TO_DATE(:start_date,'dd.mm.yyyy') and TO_DATE(:end_date,'dd.mm.yyyy') 
GROUP BY a.account) at", sqlColumns);
							oCom.Parameters.Add("account_name", OracleDbType.Varchar2, _accGroupID, ParameterDirection.Input);
							oCom.Parameters.Add("start_date", OracleDbType.Varchar2, startDate, ParameterDirection.Input);
							oCom.Parameters.Add("end_date", OracleDbType.Varchar2, endDate, ParameterDirection.Input);
						}
					}
				}
			}

			/* Getting data */
			return OracleDbPull.GetDataTable<ReportByEmp>(
				oCom,
				(resultDataTable, serviceResult) =>
				{
					/* Output report data */

					/* Final output structure */
					var reportDict = new ReportByEmp();

					/* List of record values list */

					var recordValues = new List<List<Object>>();

					/* List of columns */
					var columnList = new List<String>();

					/* Column values for all records */
					foreach (DataRow dr in resultDataTable.Rows)
					{
						/* Values of current row */
						var currRecordValues = new List<Object>();
						foreach (DataColumn dc in resultDataTable.Columns)
						{
							/* Value of current column */
							var recordValue = dr[dc];
							if (recordValue is DBNull)
							{
								currRecordValues.Add("");
							}
							else
							{
								currRecordValues.Add(recordValue);
							}
						}
						recordValues.Add(currRecordValues);
					}


					/* Define list of column names */

					for (int i = 0; i < resultDataTable.Columns.Count; i++)
					{
						var columnName = resultDataTable.Columns[i].ColumnName;
						columnList.Add(columnName.ToString());
					}

					/* Define output structure */
					reportDict.ColumnNames = columnList;
					reportDict.AbonentReportData = recordValues;

					serviceResult.DataOrNullOrDefValue = reportDict;
				}
				);

		#endregion
		}
	}
}