/// <summary>
/// Tuzova L. Summary description for ReportByServices
/// </summary>

using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Web;
using BillAnalyserWebApp.Code.IFlex.DB;
using CrossCompileSharing.Data;
using Oracle.DataAccess.Client;

namespace BillAnalyserWebApp.Code.Data
{
	public class ReportByServices
	{
		internal static ServiceResult<ReportByEmp> ReportByServiceGetting(Int64? depGroupID, String accGroupID, Int64? subsID, Int64? serviceID, DateTime? startPeriod, DateTime? endPeriod, Dictionary<String, String> listColumn)
		{
			/* Get group IDs */
			String _accGroupID = accGroupID;
			Int64? _depGroupID = depGroupID;

			/* Variables */
			var _listColumn = listColumn;
			var _subsID = subsID;
			var _serviceID = serviceID;

			var _startPeriod = startPeriod;
			var _endPeriod = endPeriod;

			String startDate = String.Format("{0:dd.MM.yyyy}", _startPeriod);
			String endDate = String.Format("{0:dd.MM.yyyy}", _endPeriod);

			/* Define selected columns row */
			String sqlColumns = "";
			foreach (var item in listColumn)
			{
				sqlColumns = sqlColumns + ", at." + item.Key + " as " + @"""" + item.Value + @"""";
			}

			/* Remove first symbols */
			sqlColumns = sqlColumns.Remove(0, 2);

			/* Define query */

			/* Check if service is defined*/

			var oCom = new OracleCommand { CommandType = CommandType.Text, BindByName = true };

			/* Subscriber IS defined */
			if (_subsID != null)
			{
				oCom.CommandText =
					"SELECT " + sqlColumns + (@" FROM
						(SELECT nvl(SUM(a.SUM_ADD), 0) as sum_add,
								nvl(a.agg_name,'') as agg_name
							FROM v_rep_services_abonents a
							WHERE a.phone_id = (:phone_id) and --    
								  a.agg_id = nvl(:agg_id, a.agg_id)and --       
								  a.bill_date between TO_DATE(:start_date, 'dd.mm.yyyy') and
       TO_DATE(:end_date, 'dd.mm.yyyy') --     
							GROUP BY a.agg_name
							ORDER BY sum_add desc)at");

				oCom.Parameters.Add("phone_id", OracleDbType.Int64, _subsID, ParameterDirection.Input);
				oCom.Parameters.Add("agg_id", OracleDbType.Int64, _serviceID, ParameterDirection.Input);
				oCom.Parameters.Add("start_date", OracleDbType.Varchar2, startDate, ParameterDirection.Input);
				oCom.Parameters.Add("end_date", OracleDbType.Varchar2, endDate, ParameterDirection.Input);
			}
			/* Subscriber is NOT defined */
			else
			{
				/* Department group */
				if (_depGroupID != null)
				{
					oCom.CommandText =
						"SELECT " + sqlColumns + (@" FROM
						(SELECT nvl(SUM(a.SUM_ADD), 0) as sum_add, nvl(a.agg_name,'') as agg_name
						 FROM v_rep_services_Abonents a, table(ba_reports_iapi.get_allabonents_by_group(:org_id)) t --  ,   
						WHERE a.abonent_id = t.n and 
								a.agg_id = nvl(:agg_id, a.agg_id) and --       
								a.bill_date between TO_DATE(:start_date, 'dd.mm.yyyy') and
       TO_DATE(:end_date, 'dd.mm.yyyy') --     
						GROUP BY a.agg_name
						ORDER BY sum_add desc )at");

					oCom.Parameters.Add("org_id", OracleDbType.Int64, _depGroupID, ParameterDirection.Input);
					oCom.Parameters.Add("agg_id", OracleDbType.Int64, _serviceID, ParameterDirection.Input);
					oCom.Parameters.Add("start_date", OracleDbType.Varchar2, startDate, ParameterDirection.Input);
					oCom.Parameters.Add("end_date", OracleDbType.Varchar2, endDate, ParameterDirection.Input);
				}
				/* Account group */
				else
				{
					oCom.CommandText =
						"SELECT " + sqlColumns + (@" FROM
						(SELECT nvl(SUM(a.SUM_ADD), 0) as sum_add, nvl(a.agg_name,'') as agg_name
						 FROM v_rep_services_Abonents a
						 WHERE a.agg_id = nvl(:agg_id, a.agg_id) and 
							   a.bill_date between TO_DATE(:start_date, 'dd.mm.yyyy') and
       TO_DATE(:end_date, 'dd.mm.yyyy') and 
							   a.account_name = :account_id
						 GROUP BY a.agg_name
						 ORDER BY sum_add desc)at");
					oCom.Parameters.Add("agg_id", OracleDbType.Int64, _serviceID, ParameterDirection.Input);
					oCom.Parameters.Add("start_date", OracleDbType.Varchar2, startDate, ParameterDirection.Input);
					oCom.Parameters.Add("end_date", OracleDbType.Varchar2, endDate, ParameterDirection.Input);
					oCom.Parameters.Add("account_id", OracleDbType.Varchar2, _accGroupID, ParameterDirection.Input);

				}
			}
			/* Getting data */
			return OracleDbPull.GetDataTable<ReportByEmp>(
				oCom,
				(resultDataTable, serviceResult) =>
				{
					/* Output report data */

					/* Final output structure */
					var reportDict = new ReportByEmp();

					/* List of record values list */

					var recordValues = new List<List<Object>>();

					/* List of columns */
					var columnList = new List<String>();

					/* Column values for all records */
					foreach (DataRow dr in resultDataTable.Rows)
					{
						/* Values of current row */
						var currRecordValues = new List<Object>();
						//for (int i = 0; i < reportResult.Columns.Count; i++)
						foreach (DataColumn dc in resultDataTable.Columns)
						{
							/* Value of current column */
							var recordValue = dr[dc];
							if (recordValue is DBNull)
							{
								currRecordValues.Add("");
							}
							else
							{
								currRecordValues.Add(recordValue);
							}
						}
						recordValues.Add(currRecordValues);
					}


					/* Define list of column names */

					for (int i = 0; i < resultDataTable.Columns.Count; i++)
					{
						var columnName = resultDataTable.Columns[i].ColumnName;
						columnList.Add(columnName.ToString());
					}

					/* Define output structure */
					reportDict.ColumnNames = columnList;
					reportDict.AbonentReportData = recordValues;

					serviceResult.DataOrNullOrDefValue = reportDict;
				}
				);

		}
	}
}