/// <summary>
/// Tuzova L. Summary description for ReportGeneral
/// </summary>

using System;
using System.Collections.Generic;
using System.Data;
using BillAnalyserWebApp.Code.IFlex.DB;
using CrossCompileSharing.Data;
using Oracle.DataAccess.Client;

namespace BillAnalyserWebApp.Code.Data
{
	public class ReportGenerals
	{
		internal static ServiceResult<ReportByEmp> ReportGeneralGetting(DateTime? startPeriod, DateTime? endPeriod, Dictionary<String, String> listColumn)
		{

			#region ReportByEmpGettingMethod

			var startDate = String.Format("{0:dd.MM.yyyy}", startPeriod);
			var endDate = String.Format("{0:dd.MM.yyyy}", endPeriod);

			/* Define selected columns row */
			var sqlColumns = "";
			foreach (var item in listColumn)
			{
				if (item.Key.Contains("abonent_group"))
				{

					sqlColumns = sqlColumns + ", BA_REPORTS_IAPI.get_abonent_group(abonent_id) as " + @"""" + item.Value + @"""";
				}
				else
				{
					sqlColumns = sqlColumns + ", at." + item.Key + " as " + @"""" + item.Value + @"""";
				}
			}

			/* Remove first symbols */
			sqlColumns = sqlColumns.Remove(0, 2);

			/* Define query */

			/* Check if service is defined*/

			var oCom = new OracleCommand { CommandType = CommandType.Text, BindByName = true, CommandText = "SELECT " + sqlColumns + (@" FROM 
				(SELECT NVL(gr.agg_name,'') as agg_name, 
                 NVL(gr.dur,0) as dur, 
                 NVL(gr.dur_dim,'') as dur_dim, 
					       NVL(gr.sum_add,0) as sum_add, 
                 NVL(gr.sum_ded,0) as sum_ded, 
                 NVL(gr.sum_comp,0) as sum_comp, 
					       NVL(ph.phone,'') as phone, 
                 NVL(ph.tarif,'') as tariff, 
                 NVL(ph.account,'') as account,
                 ab.id as abonent_id, 
                 NVL(ab.name,'') as abonent_name, 
                 NVL(ab.post,'') as post, 
                 NVL(ab.email,'') as email,
					       to_char(bi.bill_date,'MM.RRRR') as period,
					       NVL(c.name,'') as compensation_group,
					       NVL(decode(ab.roles_id,1,'',''),0) as user_role
					FROM v_ba_bill_service_aggs_grid gr,
               v_ba_phones ph ,
               v_ba_abonents ab, 
               v_ba_bills bi, 
               v_ba_phones_compensation c
					WHERE ph.id = gr.phone_id
					and ph.abonent_id = ab.id
					and ph.id = c.phones_id(+)
					and gr.bill_id = bi.id
					and bi.bill_date between TO_DATE(:start_date,'dd.mm.yyyy') and TO_DATE(:end_date,'dd.mm.yyyy'))at") };

			oCom.Parameters.Add("start_date", OracleDbType.Varchar2, startDate, ParameterDirection.Input);
			oCom.Parameters.Add("end_date", OracleDbType.Varchar2, endDate, ParameterDirection.Input);

			/* Getting data */
			return OracleDbPull.GetDataTable<ReportByEmp>(
				oCom,
				(resultDataTable, serviceResult) =>
				{
					/* Output report data */

					/* Final output structure */
					var reportDict = new ReportByEmp();

					/* List of record values list */

					var recordValues = new List<List<Object>>();

					/* List of columns */
					var columnList = new List<String>();

					/* Column values for all records */
					foreach (DataRow dr in resultDataTable.Rows)
					{
						/* Values of current row */
						var currRecordValues = new List<Object>();
						//for (int i = 0; i < reportResult.Columns.Count; i++)
						foreach (DataColumn dc in resultDataTable.Columns)
						{
							/* Value of current column */
							Object recordValue = dr[dc];
							if (recordValue is DBNull)
							{
								currRecordValues.Add("");
							}
							else
							{
								currRecordValues.Add(recordValue);
							}
						}
						recordValues.Add(currRecordValues);
					}


					/* Define list of column names */

					for (int i = 0; i < resultDataTable.Columns.Count; i++)
					{
						var columnName = resultDataTable.Columns[i].ColumnName;
						columnList.Add(columnName.ToString());
					}

					/* Define output structure */
					reportDict.ColumnNames = columnList;
					reportDict.AbonentReportData = recordValues;

					serviceResult.DataOrNullOrDefValue = reportDict;
				}
				);

			#endregion
		}
	}
}