/// <summary>
/// Tuzova L. Processing service group objects information.
/// </summary>

using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Web;
using BillAnalyserWebApp.Code.IFlex.DB;
using CrossCompileSharing.Data;
using Oracle.DataAccess.Client;

namespace BillAnalyserWebApp.Code.Data
{
	public class ServiceReports
	{
		#region ServiceGettingMethod

		internal static ServiceResult<List<ServiceReport>> ServicesGettingReport()
		{
			/* Define select command */
			var queryData = new OracleCommand(@"SELECT S.ID, S.NAME FROM V_BA_SERVICE_AGG S ORDER BY S.NAME");

			/* Getting data */
			return OracleDbPull.GetDataTable<List<ServiceReport>>(
				queryData,
				(resultDataTable, serviceResult) =>
				{
					/* List of Services */
					var serviceRaw = (
						from row in resultDataTable.AsEnumerable()
						select new
						{
							Id = (Int64)row.Field<Decimal>("ID"),
							Name = row.Field<String>("NAME")
						}
						).ToList();


					var serviceDict = new Dictionary<Int64, ServiceReport>();

					foreach (var row in serviceRaw)
					{
						if (
							!serviceDict.ContainsKey(row.Id))
						{
							serviceDict.Add(row.Id, new ServiceReport
							{
								Id = row.Id,
								Name = row.Name
							});
						}
					}

					var services = new List<ServiceReport>();
					
					/* Other Itmes Definition*/
					foreach (KeyValuePair<Int64, ServiceReport> keyVal in serviceDict)
					{
						services.Add(keyVal.Value);
					}

					serviceResult.DataOrNullOrDefValue = services;
				}
				);
		}
		#endregion
	}
}