﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Web;
using System.Web.Hosting;

namespace BillAnalyserWebApp.Code.Data
{
	/// <summary>
	/// Минимизирует обращения к файловой системе, кэширует шаблон eMail сообщения,
	/// <para/>
	///  следит за изменениями шаблона и подгружает новый шаблон при появлении, позволяя администратору изменять шаблон.
	/// </summary>
	public static class SmtpMailTemplate
	{
		private readonly static Object _targetDataLocker = new Object();
		private readonly static String _targerTemplateFilePath;
		private const String TargerTemplateFileName = "Mail_Template.xml";

		static SmtpMailTemplate()
		{
			var folderPath = HostingEnvironment.MapPath("~") + "\\App_Data\\";
			_targerTemplateFilePath = folderPath + TargerTemplateFileName;
			var fileSystemWatcher = new FileSystemWatcher
			{
				Path = folderPath,
				EnableRaisingEvents = true,
				Filter = TargerTemplateFileName,
				IncludeSubdirectories = false,
			};
			fileSystemWatcher.Changed += (o, args) => setMailTemplate();
			fileSystemWatcher.Created += (o, args) => setMailTemplate();
			setMailTemplate();
		}

		private static void setMailTemplate()
		{
			lock (_targetDataLocker)
			{
				if (File.Exists(_targerTemplateFilePath))
				{
					_value = File.ReadAllText(_targerTemplateFilePath);
				}
				else
				{
					// Нет файла, тогда останется старая закэшированная версия.
					// Если файла нет при запуске - отправка почты не будет работать, пока не появится файл шаблона.
				}
			}
		}


		public static void PrepareMailTemplate()
		{
			// Запускает статический конструктор и функционал инициализации.
			// Не обязателен, только для предварительной загрузки, т.к. весь функционал и так отработает при первом обращении.
		}

		public static Boolean TemplateExist
		{
			get
			{
				return _value != null;
			}
		}

		/// <summary>
		/// it3xl.ru<para/>
		/// Отказался от HTML, т.к. использовать неправильный DOCTYPE в XmlDocument не получится, а правильный задерживает на 3 и более минут загрузку XmlDocument,
		///  потому, что выполняет по сети всегда загрузку DTD из DOCTYPE.<para/>
		/// Обычный XML работает на порядки быстрее.<para/>
		/// Также часто ловятся исключения из-за загрузки этого DTD по сети, например "принудительные разрывы сокетов".<para/>
		/// <para/>
		/// При отказе от HTML с DOCTYPE перестает работать GetElementById в XmlDocument.
		///  Решение для этого в http://stackoverflow.com/questions/2003185/getelementbyid-not-finding-the-tag
		/// string query = string.Format("//*[@id='{0}']", id);// or "//book[@id='{0}']"
		/// XmlElement el = (XmlElement)doc.SelectSingleNode(query);
		/// <para/>
		/// + Перенес обратно в файл, чтоб клинеты могли править шаблоны сами.
		/// </summary>
		public static String Value
		{
			get
			{
				if(_value == null)
				{
					setMailTemplate();
				}
				lock (_targetDataLocker)
				{
					return _value;
				}
			}
		}

		public static String TemplateFilePath
		{
			get
			{
				return _targerTemplateFilePath;
			}
		}

		private static string _value;
		// Есил XML задавать здесь и перед объявлением XML вставить перенос строки, то XmlDocument порвет. Возможно работает как XML и без этого объявления.
//@"<?xml version=""1.0"" encoding=""utf-8""?>
//<html>
//<head>
//    <title>Счет</title>
//</head>
//<body>
//    <table id='table_1' style='font-style:italic; width:640px;'>
//        <tr>
//            <td id='period' style='text-align:center;'>
//                Счет за <span style='font-weight:bold'>[mm] [yy]</span>
//            </td>
//        </tr>
//        <tr>
//            <td id='abonent' style='text-align:center;'>
//                Абонент: <span style='font-weight:bold'>[abonent]</span>
//            </td>
//        </tr>
//    </table>
//
//    <table id='table_result_all' border='0' cellpadding='5' cellspacing='0' style='width:640px;'>
//        <tr>
//            <td>
//                <h4 style='font-style:italic; margin:0;'>ВСЕГО ПО АБОНЕНТУ</h4>
//            </td>
//        </tr>
//        <tr>
//            <td>
//                Начислено за период: <span style='font-weight:bold'>[result_abonent_charge]</span> рублей.
//            </td>
//        </tr>
//        <tr>
//            <td>
//                В том числе компенсировано компанией: <span style='font-weight:bold'>[result_abonent_compensation]</span> рублей.
//            </td>
//        </tr>
//        <tr>
//            <td>
//            </td>
//        </tr>
//        <tr>
//            <td>
//                <span style='font-weight:bold'>Всего к оплате за текущий период: [result_abonent_retantion] рублей.</span>
//            </td>
//        </tr>
//    </table>
//    <hr align='left' style='width:640px;'/>
//	(Внимание! Идущие далее слишком длинные детализации могут быть обрезаны почтовым сервером.)
//
//    <table id='table_phone_0' style='width:640px;'>
//        <tr>
//            <td>
//				<hr align='left'/>
//                <table>
//                    <tr>
//                        <td id='abonent_phone'>
//                            <span style='font-weight:bold'>Абонентский номер: [abonent_phone]</span>
//                        </td>
//                    </tr>
//                </table>
//            </td>
//        </tr>
//
//        <tr>
//            <td>
//                <table id='table_services' style='width:640px;' border='0' cellpadding='5' cellspacing='0'>
//                    <tr>
//                        <td colspan='4'>
//                            <span style='font-weight:bold'>Всего по услугам:</span>
//                        </td>
//                    </tr>
//                    <tr>
//						<td style='border:1px solid #ccc;'>
//							Группа услуг
//						</td>
//						<td style='border:1px solid #ccc;'>
//							Величина
//						</td>
//						<td style='border:1px solid #ccc;'>
//							ед. измерения
//						</td>
//						<td style='border:1px solid #ccc;'>
//							Сумма
//						</td>
//                    </tr>
//                </table>
//            </td>
//        </tr>
//        <tr>
//            <td>
//                <table id='table_result' style='width:640px' border='0' cellpadding='5' cellspacing='0'>
//                    <tr>
//                        <td colspan='4'>
//                            Начислено за период: <span style='font-weight:bold'>[number_charge]</span> рублей.
//                        </td>
//                    </tr>
//                    <tr>
//                        <td colspan='4'>
//                            В том числе компенсировано компанией: <span style='font-weight:bold'>[number_compensation]</span> рублей.
//                        </td>
//                    </tr>
//                    <tr>
//                        <td colspan='4'>
//                            Всего к оплате за текущий период: <span style='font-weight:bold'>[number_retantion]</span> рублей.
//                        </td>
//                    </tr>
//                </table>
//            </td>
//        </tr>
//
//        <tr>
//            <td>
//                <table width='640' border='1' cellpadding='5' cellspacing='0' id='table_calls'>
//                    <tr>
//                        <td style='width:70px;'>
//                            Дата и время
//                        </td>
//                        <td style='width:150px;'>
//                            Номер
//                        </td>
//                        <td style='width:70px;'>
//                            Зона ПС
//                        </td>
//                        <td style='width:70px;'>
//                            Зона ВТК
//                        </td>
//                        <td style='width:150px;'>
//                            Услуга
//                        </td>
//                        <td style='width:150px;'>
//                            Длительность
//                        </td>
//                        <td style='width:120px;'>
//                            Сумма
//                        </td>
//                    </tr>
//                </table>
//            </td>
//        </tr>
//    </table>
//
//	<br/>
//	<hr align='left' width='640' />
//	Конец письма.
//</body>
//</html>
//"
			//;            
	}
}
