/// <summary>
/// Tuzova L. Getting information about subscribers.
/// </summary>

using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using BillAnalyserWebApp.Code.IFlex.DB;
using CrossCompileSharing.Data;
using Oracle.DataAccess.Client;

namespace BillAnalyserWebApp.Code.Data
{
	public class SubscriberReports
	{
		#region SubscribersGettingMethod
		internal static ServiceResult<List<SubscriberReport>> SubscribersGettingReport(Int64? depGroupID, String accGroupID, String filter)
		{
			/* Set filters */
			String filterAbonent;
			String filterPhone;

			/* Define type of filter */
			if (filter != "")
			{
				var isPhone = true;
				foreach (var c in filter)
				{
					if (!(char.IsDigit(c) || c == '%' || c == '_'))
					{
						isPhone = false;
						break;
					}
				}
				if (isPhone)
				{
					filterAbonent = "%";
					filterPhone = "%" + filter + "%";
				}
				else
				{
					filterPhone = "%";
					filterAbonent = "%" + filter + "%";
				}
			}
			else
			{
				filterPhone = "%";
				filterAbonent = "%";
			}


			/* Define query string */
			var oCom = new OracleCommand { CommandType = CommandType.Text, BindByName = true };

			/* Department group */
			if (depGroupID != null)
			{
				oCom.CommandText =
					(@"SELECT AB.ID as ID, NVL(AB.NAME,'') as NAME, NVL(P.phone,'') as PHONE, p.id as PHONE_ID 
				   FROM (SELECT A.ID, A.NAME, A.IS_GROUP FROM V_BA_ABONENTS A
						 CONNECT BY ABONENT_ID = PRIOR ID
					     START WITH ID = :group_id) AB, V_BA_PHONES P
				   WHERE AB.IS_GROUP=0 AND 
						 UPPER(AB.NAME) LIKE UPPER(:filter_abonent) AND
						 P.phone LIKE :filter_phone AND
				         P.abonent_id = AB.ID
				   ORDER BY UPPER(NAME)");
				oCom.Parameters.Add("group_id", OracleDbType.Int64, depGroupID, ParameterDirection.Input);
				oCom.Parameters.Add("filter_abonent", OracleDbType.Varchar2, filterAbonent, ParameterDirection.Input);
				oCom.Parameters.Add("filter_phone", OracleDbType.Varchar2, filterPhone, ParameterDirection.Input);
			}
			/* Account group */
			else
			{
				oCom.CommandText =
					(@"SELECT a.id as ID, NVL(a.name,'') as NAME, NVL(P.phone,'') as PHONE, p.id as PHONE_ID
					FROM V_BA_ABONENTS A, V_BA_PHONES P
					WHERE a.id = p.abonent_id AND
						a.is_group=0 AND
						UPPER(a.NAME) LIKE UPPER(:filter_abonent) AND
						P.phone LIKE :filter_phone AND
						p.account like decode(:account_name,'  ','%',:account_name)
					ORDER BY UPPER(NAME)");

				oCom.Parameters.Add("filter_abonent", OracleDbType.Varchar2, filterAbonent, ParameterDirection.Input);
				oCom.Parameters.Add("filter_phone", OracleDbType.Varchar2, filterPhone, ParameterDirection.Input);
				oCom.Parameters.Add("account_name", OracleDbType.Varchar2, accGroupID, ParameterDirection.Input);
			}

			/* Get subscribers of the group */
			return OracleDbPull.GetDataTable<List<SubscriberReport>>(
				oCom,
				(resultDataTable, serviceResult) =>
				{
					var subsRaw = (
						from row in resultDataTable.AsEnumerable()
						select new
						{
							Id = (Int64)row.Field<Decimal>("ID"),
							Name = row.Field<String>("NAME"),
							Phone = row.Field<String>("PHONE"),
							PhoneId = (Int64)row.Field<Decimal>("PHONE_ID")
						}
						).ToList();

					var subsDict = new Dictionary<Int64, SubscriberReport>();

					foreach (var row in subsRaw)
					{
						if (
							!subsDict.ContainsKey(row.PhoneId))
						{
							subsDict.Add(row.PhoneId, new SubscriberReport
							{
								Id = row.Id,
								Name = row.Name,
								Phone = row.Phone,
								PhoneId = row.PhoneId
							});
						}
					}

					/* Full list of subscribers */
					var subs = new List<SubscriberReport>();
					foreach (KeyValuePair<Int64, SubscriberReport> keyVal in subsDict)
					{
						subs.Add(keyVal.Value);
					}
					serviceResult.DataOrNullOrDefValue = subs;
				}
				);
		}
		#endregion
	}
}