

//  it3xl.ru  .


using System;
using System.Web;
using CrossCompileSharing.Data;
using CrossCompileSharing.Log;
using CrossCompileSharing.UiStrings;
using Oracle.DataAccess.Client;
using System.Data;
using System.Web.Configuration;

namespace BillAnalyserWebApp.Code.IFlex.DB
{
	/// <summary>
	///            .
	///        OracleConnection
	/// </summary>
	public static class OracleDbPull
	{
		public static ServiceResult<TResultType> ExecuteNonQuery<TResultType>(OracleCommand oracleCommand, Action<ServiceResult<TResultType>> prepareResultOkAction)
		{
			return ExecuteNonQuery(oracleCommand, prepareResultOkAction, null);
		}

		public static ServiceResult<TResultType> ExecuteNonQuery<TResultType>(OracleCommand oracleCommand, Action<ServiceResult<TResultType>> prepareResultOkAction, Action<ServiceResult<TResultType>> prepareOnErrorAction)
		{
			oracleCommand.Connection = OracleDbUtils.GetClosedOracleConnectionInPull();
			try
			{
				oracleCommand.Connection.Open();
			}
			catch (Exception openDbConnectionException)
			{
				Logger.Write(openDbConnectionException, UiMessages.ConnectToDbBusinessErrorMessage);

				return new ServiceResult<TResultType>(BusinessError.DbConnectionErrorMessage);
			}

			var dbResult = new DbResult();
			var serviceResult = new ServiceResult<TResultType>();
			try
			{
				oracleCommand.ExecuteNonQuery();
			}
			catch (Exception ex)
			{
				Logger.Write(ex, UiMessages.UnknownDbBusinessErrorMessage);
				dbResult.SetDataExceptionAndBusinessError(ex, serviceResult);
			}
			finally
			{
				// it3xl.ru         -1  ExecuteNonQuery.

				try
				{
					//     , ..   OracleDataAdapter,     .
					oracleCommand.Connection.Close();
				}
				catch (Exception ex)
				{
					Logger.Write(ex);
				}

				//      .
				oracleCommand.Connection = null;
			}

			OracleDbUtils.ProcessDataOnDone(dbResult, serviceResult, prepareResultOkAction);

			OracleDbUtils.ProcessDataOnError(dbResult, serviceResult, prepareOnErrorAction);

			return serviceResult;
		}

		public static ServiceResult<TResultType> GetDataTable<TResultType>(OracleCommand oracleCommand, Action<DataTable, ServiceResult<TResultType>> prepareResultOkAction)
		{
			return GetDataTable(oracleCommand, prepareResultOkAction, null);
		}

		public static ServiceResult<TResultType> GetDataTable<TResultType>(OracleCommand oracleCommand, Action<DataTable, ServiceResult<TResultType>> prepareResultOkAction, Action<ServiceResult<TResultType>> prepareOnErrorAction)
		{
			oracleCommand.Connection = OracleDbUtils.GetClosedOracleConnectionInPull();

			var dbResult = new DbResult();
			var serviceResult = new ServiceResult<TResultType>();
			try
			{
				var dataTable = new DataTable();
				var oracleDataAdapter = new OracleDataAdapter(oracleCommand);
				oracleDataAdapter.Fill(dataTable);
				dbResult.DataTable = dataTable;
			}
			catch (Exception ex)
			{
				Logger.Write(ex, UiMessages.UnknownDbBusinessErrorMessage);
				dbResult.SetDataExceptionAndBusinessError(ex, serviceResult);
			}
			finally
			{
				try
				{
					//  OracleDataAdapter   ,         ,     ,     .
					oracleCommand.Connection.Close();
				}
				catch (Exception ex)
				{
					Logger.Write(ex);
				}

				//      .
				oracleCommand.Connection = null;
			}

			OracleDbUtils.ProcessDataOnDoneWithDataTable(dbResult, serviceResult, prepareResultOkAction);

			OracleDbUtils.ProcessDataOnError(dbResult, serviceResult, prepareOnErrorAction);

			return serviceResult;
		}
	}
}