

//  it3xl.ru  .


using System;
using CrossCompileSharing.Data;
using CrossCompileSharing.Log;
using CrossCompileSharing.UiStrings;
using Oracle.DataAccess.Client;
using System.Data;

namespace BillAnalyserWebApp.Code.IFlex.DB
{
	/// <summary>
	///            .
	///        OracleConnection
	/// </summary>
	public class OracleDbPullInTransaction : IDisposable
	{
		private readonly OracleConnection _oracleConnectionInPull;
		private readonly OracleTransaction _oracleTransaction;


		private Boolean WasConnected { get; set; }

		public OracleDbPullInTransaction()
		{
			_oracleConnectionInPull = OracleDbUtils.GetClosedOracleConnectionInPull();

			try
			{
				_oracleConnectionInPull.Open();
				WasConnected = true;
			}
			catch (Exception openDbConnectionException)
			{
				WasConnected = false;
				Logger.Write(openDbConnectionException, UiMessages.ConnectToDbBusinessErrorMessage);
			}

			if (WasConnected)
			{
				_oracleTransaction = _oracleConnectionInPull.BeginTransaction();
			}
		}

		public ServiceResult<TResultType> ExecuteNonQuery<TResultType>(OracleCommand oracleCommand, Action<ServiceResult<TResultType>> prepareResultOkAction)
		{
			return ExecuteNonQuery(oracleCommand, prepareResultOkAction, null);
		}

		public ServiceResult<TResultType> ExecuteNonQuery<TResultType>(OracleCommand oracleCommand, Action<ServiceResult<TResultType>> prepareResultOkAction, Action<ServiceResult<TResultType>> prepareOnErrorAction)
		{
			if (WasConnected == false)
			{
				return new ServiceResult<TResultType>( BusinessError.DbConnectionErrorMessage );
			}

			oracleCommand.Connection = _oracleConnectionInPull;

			var dbResult = new DbResult();
			var serviceResult = new ServiceResult<TResultType>();
			try
			{
				oracleCommand.ExecuteNonQuery();
			}
			catch (Exception ex)
			{
				Logger.Write(ex, UiMessages.UnknownDbBusinessErrorMessage);
				dbResult.SetDataExceptionAndBusinessError(ex, serviceResult);
			}
			finally
			{
				oracleCommand.Connection = null;
			}

			OracleDbUtils.ProcessDataOnDone(dbResult, serviceResult, prepareResultOkAction);

			OracleDbUtils.ProcessDataOnError(dbResult, serviceResult, prepareOnErrorAction);

			return serviceResult;
		}


		public ServiceResult<TResultType> GetDataTable<TResultType>(OracleCommand oracleCommand, Action<DataTable, ServiceResult<TResultType>> prepareResultOkAction)
		{
			return GetDataTable(oracleCommand, prepareResultOkAction, null);
		}

		public ServiceResult<TResultType> GetDataTable<TResultType>(OracleCommand oracleCommand, Action<DataTable, ServiceResult<TResultType>> prepareResultOkAction, Action<ServiceResult<TResultType>> prepareOnErrorAction)
		{
			if (WasConnected == false)
			{
				return new ServiceResult<TResultType>(BusinessError.DbConnectionErrorMessage );
			}

			oracleCommand.Connection = _oracleConnectionInPull;

			var dbResult = new DbResult();
			var serviceResult = new ServiceResult<TResultType>();
			try
			{
				var dataTable = new DataTable();
				var oracleDataAdapter = new OracleDataAdapter(oracleCommand);
				oracleDataAdapter.Fill(dataTable);
				dbResult.DataTable = dataTable;
			}
			catch (Exception ex)
			{
				Logger.Write(ex, UiMessages.UnknownDbBusinessErrorMessage);
				dbResult.SetDataExceptionAndBusinessError(ex, serviceResult);
			}
			finally
			{
				//      .
				oracleCommand.Connection = null;
			}

			OracleDbUtils.ProcessDataOnDoneWithDataTable(dbResult, serviceResult, prepareResultOkAction);

			OracleDbUtils.ProcessDataOnError(dbResult, serviceResult, prepareOnErrorAction);

			return serviceResult;
		}


		/// <summary>
		///          
		/// </summary>
		public void Rollback()
		{
			if (WasConnected)
			{
				_oracleTransaction.Rollback();
			}
			Dispose();
		}


		/// <summary>
		///          
		/// </summary>
		public void Commit()
		{
			if (WasConnected)
			{
				_oracleTransaction.Commit();
			}
			Dispose();
		}



		#region Disposing 

		/// <summary>
		///        ,     
		/// </summary>
		private bool _isDisposed = false;

		public void Dispose()
		{
			Dispose(DisposingCallFrom.ClientCode);
			GC.SuppressFinalize(this);
		}

		internal void Dispose(DisposingCallFrom disposingCallFrom)
		{
			if (!_isDisposed)
			{
				//     

				//       
				_isDisposed = true;

				//   
				if (disposingCallFrom == DisposingCallFrom.ClientCode)
				{
					//        ,       GC      

					try
					{
						//  ,      
						_oracleConnectionInPull.Dispose();
					}
					catch (Exception ex)
					{
						Logger.Write(ex);
					}
				}

				//   
				{
					//  ,  OracleConnection   IDisposable
				}
			}
		}

		~OracleDbPullInTransaction()
		{
			Dispose(DisposingCallFrom.Finalizer);
		}


		public enum DisposingCallFrom
		{
			ClientCode = 0,
			Finalizer = 1,
		}

		#endregion
	}
}