﻿

// Ответственный it3xl.ru Илья Третьяков.


using System;
using System.Data;
using CrossCompileSharing.Data;
using CrossCompileSharing.Log;
using CrossCompileSharing.UiStrings;
using Oracle.DataAccess.Client;
using System.Web.Configuration;

namespace BillAnalyserWebApp.Code.IFlex.DB
{
	public static class OracleDbUtils
	{
		internal static OracleConnection GetClosedOracleConnectionInPull()
		{
			OracleConnection connection;
			try
			{
				connection = new OracleConnection(WebConfigurationManager.ConnectionStrings["OracleConnectionString"].ConnectionString);
			}
			catch (Exception createDbConnectionObjectEx)
			{
				Logger.Write(createDbConnectionObjectEx, UiMessages.ConnectToDbBusinessErrorMessage);

				throw;
			}

			return connection;
		}

		internal static void ProcessDataOnDone<TResultType>(DbResult dbResult, ServiceResult<TResultType> serviceResult, Action<ServiceResult<TResultType>> prepareResultOkAction)
		{
			if (dbResult.IsDone
				&& prepareResultOkAction != null)
			{
				try
				{
					prepareResultOkAction(serviceResult);
				}
				catch (Exception prepareDataException)
				{
					Logger.Write(prepareDataException, UiMessages.ОшибкаОбработкиПолученныхИзБазыДанных);
					serviceResult.BusinessErrorNotNulled.Add(UiMessages.ОшибкаОбработкиПолученныхИзБазыДанных);
				}
			}
		}

		internal static void ProcessDataOnDoneWithDataTable<TResultType>(DbResult dbResult, ServiceResult<TResultType> serviceResult, Action<DataTable, ServiceResult<TResultType>> prepareResultOkAction)
		{
			if (dbResult.IsDone
				&& prepareResultOkAction != null)
			{
				try
				{
					prepareResultOkAction(dbResult.DataTable, serviceResult);
				}
				catch (Exception prepareDataException)
				{
					Logger.Write(prepareDataException, UiMessages.ОшибкаОбработкиПолученныхИзБазыДанных);
					serviceResult.BusinessErrorNotNulled.Add(UiMessages.ОшибкаОбработкиПолученныхИзБазыДанных);
				}
			}
		}

		internal static void ProcessDataOnError<TResultType>(DbResult dbResult, ServiceResult<TResultType> serviceResult, Action<ServiceResult<TResultType>> prepareOnErrorAction)
		{
			if (dbResult.IsDone == false
				&& prepareOnErrorAction != null)
			{
				try
				{
					prepareOnErrorAction(serviceResult);
				}
				catch (Exception prepareDataOnErrorException)
				{
					Logger.Write(prepareDataOnErrorException, "Ошибка обработки данных при сбое обращения к базе.");
				}
			}
		}


		//public static string GetOracleParametrSerialisation(OracleParameter op)
		//{
		//    string serialisation;
		//    if (op.Value != null)
		//    {
		//        if (
		//            op.OracleDbType == OracleDbType.Varchar2
		//                || op.OracleDbType == OracleDbType.Char
		//                    || op.OracleDbType == OracleDbType.NVarchar2
		//                        || op.OracleDbType == OracleDbType.NChar
		//                            || op.OracleDbType == OracleDbType.XmlType
		//            )
		//        {

		//            serialisation = "\t'" + op.Value.ToString().Replace("'", "''") + "'";
		//        }
		//        else if (op.OracleDbType == OracleDbType.Date)
		//        {
		//            serialisation = "\tTO_DATE('" + op.Value + "', 'DD.MM.YYYY HH24:MI:SS')";
		//        }
		//        else
		//        {
		//            serialisation = "\t" + op.Value;
		//        }
		//    }
		//    else
		//    {
		//        serialisation = "\tNULL";
		//    }

		//    return serialisation;
		//}

	}
}