﻿using System;
using System.IO;
using System.Linq;
using System.Text;
using System.Web.Hosting;
using CrossCompileSharing.Log;

namespace BillAnalyserWebApp.Code.Log
{
	public sealed class AspNetLogProvider : LogProviderBase
	{
		// ReSharper disable UnaccessedField.Local

		static AspNetLogProvider()
		{
			ApplicationName = HostingEnvironment.ApplicationVirtualPath.Replace('/', '-');
			// В некоторых случаях Path.GetTempPath() возвращает C:\WINDOWS\Temp. Думаю это при закрытии или при каком-то сбое, поэтому сделал FolderPath статическим и задаю только один раз, т.к. обычно ожидаю что запись пойдет в C:\Documents and Settings\ComputerName\ASPNET\Local Settings\Temp.
			FolderPath = Path.Combine(Path.GetTempPath(), LogProviderConfig.SaveFolderName);
		}

		public AspNetLogProvider()
		{
			// По закрытию приложения сохраним записи лога.
			Global.End += PrepareForShutdown;
		}

		public override void Write(LogItem logItem)
		{
			WriteInternaly(logItem);
		}

		protected override void EnsureLogFolderExist()
		{
			try
			{
				if (Directory.Exists(FolderPath))
				{
					return;
				}

				Directory.CreateDirectory(FolderPath);
			}
			catch (Exception ex)
			{
				Write(LogItem.GetLogItemFromException("Невозможно создать дирректорию для хранения логов.", ex));
			}
		}

		protected override void SaveLogs(LogProviderConfig.WriteMode writeMode)
		{
			EnsureLogFolderExist();
			lock (_logItemsBuffer)
			{
				if(_logItemsBuffer.Count==0)
				{
					return;
				}

				var accumulatedLogText = new StringBuilder();
				_logItemsBuffer.OrderBy(item => item.Occurrence).ToList().ForEach(logItem => accumulatedLogText.Append(logItem));

				var actualLogFileNamePath = GetActualLogFilePath(LogProviderConfig.FileNameWithEmergency.No);
#pragma warning disable 219
// ReSharper disable RedundantAssignment
				Boolean appendData;
				using (var streamWriter = new StreamWriter(actualLogFileNamePath, appendData = true))
// ReSharper restore RedundantAssignment
#pragma warning restore 219
				{
					streamWriter.Write(accumulatedLogText.ToString());
				}

				// Очистим коллекцию только после успешного сохранения.
				_logItemsBuffer.Clear();
			}
		}
	}
}
