//   .             !



using System;
using System.Linq;
using System.ServiceModel;
using System.ServiceModel.Activation;
using System.Collections.Generic;
using BillAnalyserWebApp.Code.Data;
using BillAnalyserWebApp.Code.IFlex.DB;
using CrossCompileSharing.Data;
using CrossCompileSharing.UiStrings;

namespace BillAnalyserWebApp
{
	[ServiceContract(Namespace = "iFlexCompensationService")]
	[AspNetCompatibilityRequirements(RequirementsMode = AspNetCompatibilityRequirementsMode.Allowed)]
	public class CompensationService
	{
		#region  

		[OperationContract]
		public ServiceResult<Object> PrepareCalcCompensation(DateTime period, bool recalc, long numberAbonents)
		{
			CompensationForCalc.listAbonents = new List<CompensationOfAbonent>();
			CompensationForCalc._numberAbonents = numberAbonents;
			CompensationForCalc.period = period;
			CompensationForCalc.recalc = recalc;
			CompensationForCalc._countPosted = 0;

			return new ServiceResult<object>();
		}

		[OperationContract]
		public ServiceResult<bool> AddCompensationsToCalc(List<CompensationOfAbonent> list)
		{
			if (CompensationForCalc.listAbonents == null || list == null)
				return ServiceResult.GetFalseServiceResult();

			CompensationForCalc.listAbonents.AddRange(list);

			//      
			if (CompensationForCalc.listAbonents.Count == CompensationForCalc._numberAbonents)
			{
				return ServiceResult.GetTrueServiceResult();
			}

			return ServiceResult.GetFalseServiceResult();
		}

		[OperationContract]
		public ServiceResult<int> GetPercentCalcCompensation()
		{
			return new ServiceResult<int> { DataOrNullOrDefValue = CompensationForCalc.Percent, };
		}

		[OperationContract]
		public ServiceResult<List<ServiceResult<CompensationOfAbonentResult>>> CalcCompensation()
		{
			var result = new List<ServiceResult<CompensationOfAbonentResult>>();

			foreach (var abonent in CompensationForCalc.listAbonents)
			{
				var compensationResult = new ServiceResult<CompensationOfAbonentResult> { DataOrNullOrDefValue = new CompensationOfAbonentResult { abonent = abonent.Name } };

				var res = Compensations.CalcCompensation(abonent.ID, CompensationForCalc.period, CompensationForCalc.recalc);

				compensationResult.AddResultStates(res);

				result.Add(compensationResult);
			}

			return new ServiceResult<List<ServiceResult<CompensationOfAbonentResult>>> { DataOrNullOrDefValue = result, };
		}
		#endregion

		#region   
		[OperationContract]
		public ServiceResult<Int64> ins_ba_compensations()
		{
			return Compensations.ins_ba_compensations();
		}
		#endregion

		#region  
		[OperationContract]
		public ServiceResult<Boolean> TrmCompensation(long id)
		{
			return Compensations.TrmCompensation(id);
		}
		#endregion

		#region   
		[OperationContract]
		public ServiceResult<List<Compensation>> GetCompensationGroups()
		{
			return Compensations.GroupCompensationsGet();
		}
		#endregion

		#region   () 
		[OperationContract]
		public ServiceResult<List<CompensationUnit>> GetCompensationUnits(long compensation_id)
		{
			return CompensationUnits.CompensationUnitsGet(compensation_id);
		}
		#endregion

		#region     
		[OperationContract]
		public ServiceResult<List<CompensationOfAbonent>> GetCompensationsOfAbonents(Int64 idCompensation)
		{
			return CompensationsOfAbonents.CompensationsOfAbonentsGet(idCompensation);
		}
		#endregion

		#region     
		[OperationContract]
		public ServiceResult<List<ServiceResult<CompensationOfAbonentResult>>> ChangeLinksPhoneToCompensation(List<CompensationOfAbonent> listForDelete, List<CompensationOfAbonent> listForAdd, long? idComp)
		{
			var processErrors = new List<ServiceResult<CompensationOfAbonentResult>>();

			foreach (var ab in listForDelete)
			{
				var trmPhoneCompsResult = Compensations.TrmPhoneComps(ab.CompensaionPhoneId);
				if (trmPhoneCompsResult.IsDone == false)
				{
					processErrors.Add(new ServiceResult<CompensationOfAbonentResult>(trmPhoneCompsResult.BusinessErrorNotNulled)
					{
						DataOrNullOrDefValue = new CompensationOfAbonentResult { abonent = ab.Name, phone = ab.Phone },
					});
				}
			}

			foreach (var ab in listForAdd)
			{
				var insBaPhoneCompsResult = Compensations.InsBaPhoneComps(ab.PhoneId, idComp);
				if (insBaPhoneCompsResult.IsDone == false)
				{
					processErrors.Add(new ServiceResult<CompensationOfAbonentResult>(insBaPhoneCompsResult.BusinessErrorNotNulled)
					{
						DataOrNullOrDefValue = new CompensationOfAbonentResult { abonent = ab.Name, phone = ab.Phone },
					});
				}
			}

			return new ServiceResult<List<ServiceResult<CompensationOfAbonentResult>>> { DataOrNullOrDefValue = processErrors, };
		}
		#endregion

		[OperationContract]
		public ServiceResult<List<ServiceResult<CompensationUnit>>> ChangServicesForComp(CompensationsData compensationsData)
		{
			var result = new ServiceResult<List<ServiceResult<CompensationUnit>>>
			{
				DataOrNullOrDefValue = new List<ServiceResult<CompensationUnit>>(),
			};

			using (var dbPullInTransaction = new OracleDbPullInTransaction())
			{
				//       ,        .
				result.DataOrNullOrDefValue.AddRange(compensationsData.UnitsForDelete.Select(unit => Compensations.TrmServiceComp(unit, dbPullInTransaction)));
				result.DataOrNullOrDefValue.AddRange(compensationsData.UnitsForUpdate.Select(unit => Compensations.UpdServiceComp(unit, dbPullInTransaction)));
				result.DataOrNullOrDefValue.AddRange(compensationsData.UnitsForInclude.Select(unit => Compensations.InsBaServiceComps(unit, dbPullInTransaction)));

				if (result.DataOrNullOrDefValue.Any(el => el.HasErrors))
				{
					//   ,                 .
					result.AddBusinessError(UiMessages.____);

					return result;
				}
				
				var updCompensationResult = Compensations.UpdCompensation(compensationsData.CommonCompensationData, dbPullInTransaction);
				//       .
				result.AddResultStates(updCompensationResult);
				if (updCompensationResult.HasErrors)
				{

					return result;
				}

				dbPullInTransaction.Commit();
			}

			return result;
		}
	}
}