﻿// Ответственный Новик Н.А.

using System;
using System.Reflection;
using System.ServiceModel;
using System.ServiceModel.Activation;
using System.Collections.Generic;
using BillAnalyserWebApp.Code.Data;
using BillAnalyserWebApp.Code.IFlex.DB.Oracle;
using BillAnalyserWebApp.Code.Interop1C;
using CrossCompileSharing.Data;
using CrossCompileSharing.Log;
using CrossCompileSharing.UiStrings;

namespace BillAnalyserWebApp
{
	[ServiceContract(Namespace = "")]
	[AspNetCompatibilityRequirements(RequirementsMode = AspNetCompatibilityRequirementsMode.Allowed)]
	public class ImportEmployees
	{
		private static Boolean _isStarted;

		[OperationContract]
		public ServiceResult<ResultImportEmployees> ImportFrom1cTxt(String fileUniqueKey)
		{
			var listEmployeeResult = ImportEmp.ProcessEmployeesFromUploadedFile(fileUniqueKey);
			var resultImportEmployees = new ServiceResult<ResultImportEmployees>
			{
				DataOrNullOrDefValue = GetResultImportEmployees(listEmployeeResult.DataOrNullOrDefValue)
			};

			return resultImportEmployees;
		}

		[OperationContract]
		public ServiceResult<ResultImportEmployees> ImportFromCsv()
		{
			var resultImportEmployees = new ServiceResult<ResultImportEmployees>() { DataOrNullOrDefValue = new ResultImportEmployees() };

			if (_isStarted)
			{
				return resultImportEmployees;
			}

			try
			{
				var listEmployeeResult = ImportEmp.SaveToDbFromStaticList();
				if (listEmployeeResult.HasErrors)
				{
					resultImportEmployees.AddBusinessError(listEmployeeResult.BusinessErrorNotNulled);
				}
				resultImportEmployees.DataOrNullOrDefValue = GetResultImportEmployees(listEmployeeResult.DataOrNullOrDefValue);
			}
			finally
			{
				_isStarted = false;
			}

			return resultImportEmployees;
		}

		#region Метод получения результата импорта
		private ResultImportEmployees GetResultImportEmployees(List<Employee> listResultImport)
		{
			var resultImportEmployees = new ResultImportEmployees() { IsDone = true, Result = false };

			if (listResultImport == null)
			{
				// Если импорт в БД завершился неудачно установим соответствующие флаги
				resultImportEmployees.Result = false;
				resultImportEmployees.IsDone = true;
				resultImportEmployees.ResultData = null;
			}
			else
			{
				resultImportEmployees.Result = true;
				resultImportEmployees.IsDone = true;
				resultImportEmployees.ResultData = GetListEmployeesReport(listResultImport); ;
			}

			return resultImportEmployees;
		}
		#endregion

		#region Метод получения списка для формирования результирующего окна
		private List<ImportEmployeesReport> GetListEmployeesReport(List<Employee> listResultImport)
		{
			var listImportEmployeesReport = new List<ImportEmployeesReport>();

			foreach (var item in listResultImport)
			{
				var importEmployeesReport = new ImportEmployeesReport();

				importEmployeesReport.N = listImportEmployeesReport.Count + 1;
				importEmployeesReport.isGroup = (item.IsGroup == 1) ? "Группа" : "";
				importEmployeesReport.Name = item.Name;
				importEmployeesReport.Post = item.Post;
				importEmployeesReport.Phone = item.Phone != null ? item.Phone.Trim() : null;
				importEmployeesReport.Email = item.Email;
				importEmployeesReport.Code = item.Code;
				importEmployeesReport.ParentCode = item.ParentCode;
				importEmployeesReport.Dsc = item.Dsc;
				importEmployeesReport.Id = item.Id;

				listImportEmployeesReport.Add(importEmployeesReport);
			}

			return listImportEmployeesReport;
		}
		#endregion

		[OperationContract]
		public ServiceResult<Boolean> PrepareImportFromCsv(int numberItems)
		{
			return ImportEmp.PrepareInsert(numberItems);
		}

		[OperationContract]
		public ServiceResult<bool> AddPartToList(List<ImportEmployeesReport> listEmployees)
		{
			return ImportEmp.AddPartToList(listEmployees);
		}

		[Obfuscation(Exclude = true)]
		public class ResultImportEmployees
		{
			public Boolean IsDone { get; set; }
			public List<ImportEmployeesReport> ResultData { get; set; }
			public Boolean Result { get; set; }
		}
	}


}