//   .             !



using System;
using System.Data;
using System.ServiceModel;
using System.ServiceModel.Activation;
using System.Collections.Generic;
using BillAnalyserWebApp.Code.Data;
using BillAnalyserWebApp.Code.IFlex.DB;
using System.Globalization;
using CrossCompileSharing.Data;
using CrossCompileSharing.Log;
using CrossCompileSharing.UiStrings;
using Oracle.DataAccess.Client;

namespace BillAnalyserWebApp
{
	[ServiceContract(Namespace = "iFlexImportFileService")]
	[AspNetCompatibilityRequirements(RequirementsMode = AspNetCompatibilityRequirementsMode.Allowed)]
	public class ImportFilesService
	{
		// it3xl.ru ,       ,               .
		//[OperationContract]
		//public ServiceResult<string> GetLastError()
		//{
		//    return new ServiceResult<string> { Data = _lastError, };
		//}

		#region CloseProtocol
		[OperationContract]
		public ServiceResult<Boolean> CloseProtocol()
		{
			return Protocols.CloseProtocol();
		}
		#endregion

		#region InsertBill
		[OperationContract]
		public ServiceResult<BillInfoAttributes> InsertBill(BillInfoAttributes billInfoAttributes)
		{
			Protocols.CreateProtocol(billInfoAttributes.ID);

			billInfoAttributes.ID = -1;
			billInfoAttributes.ImportKey = -1;

			var importKeyRes = ImportFilesServices.GetImportKey();
			if(importKeyRes.HasErrors)
			{
				return new ServiceResult<BillInfoAttributes>(importKeyRes.BusinessErrorNotNulled);
			}
			billInfoAttributes.ImportKey = importKeyRes.DataOrNullOrDefValue;

			var command = new OracleCommand("BA_BILLS_IAPI.ins_ba_bills") { CommandType = CommandType.StoredProcedure };

			command.Parameters.Add("i_bill_number", OracleDbType.Varchar2, billInfoAttributes.ReportBBn, ParameterDirection.Input);
			command.Parameters.Add("i_bill_date", OracleDbType.Date, billInfoAttributes.ReportBSd, ParameterDirection.Input);
			command.Parameters.Add("i_from_date", OracleDbType.Date, null, ParameterDirection.Input);
			command.Parameters.Add("i_to_date", OracleDbType.Date, null, ParameterDirection.Input);
			command.Parameters.Add("i_id", OracleDbType.Int32, ParameterDirection.Output);

			var result = OracleDbPull.ExecuteNonQuery<BillInfoAttributes>(
				command,
				serviceResult =>
				{
					if (command.Parameters["i_id"].Value != DBNull.Value)
					{
						try
						{
							billInfoAttributes.ID = Convert.ToInt32(command.Parameters["i_id"].Value);
						}
						catch (Exception ex)
						{
							Logger.Write(ex);
							Protocols.AddRecord("     : " + ex.Message);
						}
					}
					else
					{
						Protocols.AddRecord(UiMessages.____);
					}

					Protocols.AddRecord(String.Format(" {0}  ", billInfoAttributes.ID));

					serviceResult.DataOrNullOrDefValue = billInfoAttributes;
				},
				serviceResult =>
				{
					serviceResult.BusinessErrorNotNulled.Add(String.Format("     -  : {0},  : {1}.", billInfoAttributes.ReportBBn, billInfoAttributes.ReportBSd));
					Protocols.AddRecord(serviceResult.BusinessErrorNotNulled.GetAllErrorsText());
				}
				);

			return result;
		}
		#endregion

		#region InsertCalls

		#region PrepareInsertCalls
		[OperationContract]
		public ServiceResult<bool> PrepareInsertCalls(string numberPhone, string tariff, string personaBill, int numberCalls)
		{
			//_lastError = "";

			DetailCalls.numberPhone = numberPhone;
			DetailCalls.personalBill = personaBill;
			DetailCalls.tariff = tariff;
			DetailCalls._numberCallsForInsert = numberCalls;
			DetailCalls._listDetailed = new List<DetailCallAttributes>();

			return ServiceResult.GetTrueServiceResult();
		}
		#endregion

		#region AddCallsToListForInsert
		[OperationContract]
		public ServiceResult<bool> AddCallsToListForInsert(List<DetailCallAttributes> listCalls)
		{
			//_lastError = "";

			if (DetailCalls._listDetailed == null || listCalls == null)
				return ServiceResult.GetFalseServiceResult();

			DetailCalls._listDetailed.AddRange(listCalls);

			//      
			if (DetailCalls._listDetailed.Count == DetailCalls._numberCallsForInsert)
			{
				return ServiceResult.GetTrueServiceResult();
			}

			return ServiceResult.GetFalseServiceResult();
		}
		#endregion

		#region InsertCallsMass
		/// <summary>
		///    !!! 
		/// </summary>
		[OperationContract]
		public ServiceResult<ResultInsertCalls> InsertCallsMass()
		{
			var currentPhoneIdResult = GetPhoneId(DetailCalls.numberPhone, DetailCalls.tariff, DetailCalls.personalBill, false,null);
			if (currentPhoneIdResult.IsDone == false)
			{
				Protocols.AddRecord(currentPhoneIdResult.BusinessErrorNotNulled.GetAllErrorsText());

				return new ServiceResult<ResultInsertCalls>(currentPhoneIdResult.BusinessErrorNotNulled);
			}

			using (var dbInTransaction = new OracleDbPullInTransaction())
			{
				//      .
				var commandStartInsertCalls = new OracleCommand("BA_CALLS_IAPI.start_import") { CommandType = CommandType.StoredProcedure };
				var startInsertCallsResult = dbInTransaction.ExecuteNonQuery<ResultInsertCalls>(
					commandStartInsertCalls,
					serviceResult => Protocols.AddRecord("     - "),
					serviceResultOnError => Protocols.AddRecord(serviceResultOnError.BusinessErrorNotNulled.GetAllErrorsText())
					);

				if (startInsertCallsResult.IsDone == false)
				{

					return startInsertCallsResult;
				}

				//  .  ,        foreach  for.
				for (int i = 0; i < DetailCalls._listDetailed.Count; i++)
				{
					var currentDetail = DetailCalls._listDetailed[i];
					//      
					// !!!     

					//         

					var addCallForInsertResult = AddCallForInsert(
						dbInTransaction,
						currentPhoneIdResult.DataOrNullOrDefValue,
						DateTime.Parse(currentDetail.report_ds_i_d + " " + currentDetail.report_ds_i_t, new CultureInfo("Ru-RU", true)),
						currentDetail.report_ds_i_n,
						currentDetail.report_ds_i_s,
						currentDetail.report_ds_i_zp,
						currentDetail.report_ds_i_zv,
						currentDetail.report_ds_i_dup,
						currentDetail.report_ds_i_f,
						currentDetail.report_ds_i_,
						currentDetail.factor_nds,

						currentDetail.period_start_date
						);

					if (addCallForInsertResult.IsDone == false)
					{
						Protocols.AddRecord(String.Format(" {0}    {1}     . : {2}", DetailCalls._listDetailed.Count, DetailCalls.numberPhone, addCallForInsertResult.BusinessErrorNotNulled.GetAllErrorsText()));

						// it3xl.ru:       -   .
						continue;
					}
				}

				Protocols.AddRecord(String.Format(" {0}    {1}    - ", DetailCalls._listDetailed.Count, DetailCalls.numberPhone));

				//     .
				var result = FinishInsertCalls(dbInTransaction);
				if (result.IsDone)
				{
					dbInTransaction.Commit();
				}

				result.DataOrNullOrDefValue.NumberPhone = DetailCalls.numberPhone;

				return result;
			}
		}
		#endregion

		#region FinishInsertCalls
		private ServiceResult<ResultInsertCalls> FinishInsertCalls(OracleDbPullInTransaction trans)
		{
			//_lastError = "";

			var command = new OracleCommand("BA_CALLS_IAPI.finish_import") { CommandType = CommandType.StoredProcedure };

			command.Parameters.Add("o_canceled", OracleDbType.Int32, ParameterDirection.Output);
			command.Parameters.Add("o_loaded", OracleDbType.Int32, ParameterDirection.Output);
			command.Parameters.Add("o_errors", OracleDbType.Int32, ParameterDirection.Output);

			var result = trans.ExecuteNonQuery<ResultInsertCalls>(
				command,
				serviceResult =>
				{
					serviceResult.DataOrNullOrDefValue = new ResultInsertCalls();

					//    
					try
					{
						serviceResult.DataOrNullOrDefValue.CountAlreadyInserted = Convert.ToInt32(command.Parameters["o_canceled"].Value);
					}
					catch (Exception ex)
					{
						Logger.Write(ex);
						serviceResult.AddBusinessError(UiMessages.___);

						return;
					}

					//   
					try
					{
						serviceResult.DataOrNullOrDefValue.CountSucceffulInsertedCalls = Convert.ToInt32(command.Parameters["o_loaded"].Value);
					}
					catch (Exception ex)
					{
						Logger.Write(ex);
						serviceResult.AddBusinessError(UiMessages.___);

						return;
					}

					//     - 
					try
					{
						serviceResult.DataOrNullOrDefValue.CountUnsucceffulInsertedCalls = Convert.ToInt32(command.Parameters["o_errors"].Value);
					}
					catch (Exception ex)
					{
						Logger.Write(ex);
						serviceResult.AddBusinessError(UiMessages.___);

						return;
					}

					Protocols.AddRecord("   - ");
				},
				serviceResultOnError =>
				{
					Protocols.AddRecord("    ");
				}
				);

			if (result.DataOrNullOrDefValue==null)
			{
				// it3xl.ru       DataOrNullOrDefValue.     .
				result.DataOrNullOrDefValue = new ResultInsertCalls();
			}
			return result;
		}
		#endregion

		#region AddCallForInsert
		//procedure add_call_import(i_phones_id in ba_calls.phones_id%type,
		//                       i_call_date in ba_calls.call_date%type,
		//                       i_phone_number in ba_calls.phone_number%type,
		//                       i_service_name in ba_calls.service_name%type,
		//                       i_zone_ps in ba_calls.zone_ps%type,
		//                       i_zone_vtk in ba_calls.zone_vtk%type,
		//                       i_dur in ba_calls.dur%type,
		//                       i_cost in ba_calls.cost%type,
		//                       i_cost_no_vat in ba_calls.cost_no_vat%type)


		//  

		//Parameters
		//    i_phones_id   	  
		//    i_call_date   	 
		//    i_phone_number   	 
		//    i_service_name   	    
		//    i_zone_ps   	 
		//    i_zone_vtk   	 
		//    i_dur   	
		//    i_cost   	   
		//    i_cost_no_vat   	   

		private ServiceResult<bool> AddCallForInsert(OracleDbPullInTransaction dbInTransaction, Int64 idA, DateTime date_time, string number_B, string service_name,
			string zona_ps, string zona_vtk, string dur, string cost_nds, string cost, double factor_nds, DateTime period_start_date)
		{
			//_lastError = "";

//			DateTime dt = DateTime.Parse(date_time, new CultureInfo("Ru-RU", true));

			double d_cost_nds = 0;
			double d_cost = 0;

			//   
			d_cost = Convert.ToDouble(CommonFunctions.NormFloat(cost));

			if (cost_nds != null)
			{
				d_cost_nds = Convert.ToDouble(CommonFunctions.NormFloat(cost_nds));
			}
			else
			{
				d_cost_nds = d_cost * factor_nds;
			}

			var command = new OracleCommand("BA_CALLS_IAPI.add_call_import") { CommandType = CommandType.StoredProcedure };

			command.Parameters.Add("i_phone_id", OracleDbType.Int64, idA, ParameterDirection.Input);
			command.Parameters.Add("i_call_date", OracleDbType.Date, date_time, ParameterDirection.Input);
			command.Parameters.Add("i_phone_number", OracleDbType.Varchar2, number_B, ParameterDirection.Input);
			command.Parameters.Add("i_service_name", OracleDbType.Varchar2, service_name, ParameterDirection.Input);
			command.Parameters.Add("i_zona_ps", OracleDbType.Varchar2, zona_ps, ParameterDirection.Input);
			command.Parameters.Add("i_zona_vtk", OracleDbType.Varchar2, zona_vtk, ParameterDirection.Input);
			command.Parameters.Add("i_dur", OracleDbType.Varchar2, dur, ParameterDirection.Input);
			command.Parameters.Add("i_cost", OracleDbType.Double, d_cost_nds, ParameterDirection.Input);
			command.Parameters.Add("i_cost_no_vat", OracleDbType.Double, d_cost, ParameterDirection.Input);
			command.Parameters.Add("i_period_start_date", OracleDbType.Date, period_start_date, ParameterDirection.Input);
			// it3xl.ru:      .
			DateTime? period_end_date = null;
			command.Parameters.Add("i_period_end_date", OracleDbType.Date, period_end_date, ParameterDirection.Input);

			return dbInTransaction.ExecuteNonQuery<Boolean>(
				command,
				serviceResult =>
				{
					serviceResult.DataOrNullOrDefValue = true;
				},
				serviceResultOnError =>
				{
					serviceResultOnError.DataOrNullOrDefValue = false;
					Protocols.AddRecord(serviceResultOnError.BusinessErrorNotNulled.GetAllErrorsText());
				}
				);
		}
		#endregion

		#region InsertCall
		public ServiceResult<Int64> InsertCall(Int64 idA, DateTime date_time, string number_B, string service_name,
			string zona_ps, string zona_vtk, string dur, string cost_nds, string cost, double factor_nds)
		{
			//_lastError = "";

			//var dt = DateTime.Parse(date_time, new CultureInfo("Ru-RU", true));

			double d_cost_nds = 0;
			double d_cost = 0;

			//   
			d_cost = Convert.ToDouble(CommonFunctions.NormFloat(cost));

			if (cost_nds != null)
			{
				d_cost_nds = Convert.ToDouble(CommonFunctions.NormFloat(cost_nds));
			}
			else
			{
				d_cost_nds = d_cost * factor_nds;
			}

			var command = new OracleCommand("BA_CALLS_IAPI.ins_ba_calls") { CommandType = System.Data.CommandType.StoredProcedure };

			command.Parameters.Add("i_phone_id", OracleDbType.Int64, idA, System.Data.ParameterDirection.Input);
			command.Parameters.Add("i_call_date", OracleDbType.Date, date_time, System.Data.ParameterDirection.Input);
			command.Parameters.Add("i_phone_number", OracleDbType.Varchar2, number_B, System.Data.ParameterDirection.Input);
			command.Parameters.Add("i_service_name", OracleDbType.Varchar2, service_name, System.Data.ParameterDirection.Input);
			command.Parameters.Add("i_zona_ps", OracleDbType.Varchar2, zona_ps, System.Data.ParameterDirection.Input);
			command.Parameters.Add("i_zona_vtk", OracleDbType.Varchar2, zona_vtk, System.Data.ParameterDirection.Input);
			command.Parameters.Add("i_dur", OracleDbType.Varchar2, dur, System.Data.ParameterDirection.Input);
			command.Parameters.Add("i_cost", OracleDbType.Double, d_cost_nds, System.Data.ParameterDirection.Input);
			command.Parameters.Add("i_cost_no_vat", OracleDbType.Double, d_cost, System.Data.ParameterDirection.Input);
			command.Parameters.Add("i_from_date", OracleDbType.Date, null, System.Data.ParameterDirection.Input);
			command.Parameters.Add("i_to_date", OracleDbType.Date, null, System.Data.ParameterDirection.Input);
			command.Parameters.Add("o_id", OracleDbType.Int64, System.Data.ParameterDirection.Output);

			return OracleDbPull.ExecuteNonQuery<Int64>(
				command,
				serviceResult =>
				{
					try
					{
						serviceResult.DataOrNullOrDefValue = Convert.ToInt64(command.Parameters["o_id"].Value);
					}
					catch (Exception ex)
					{
						Logger.Write(ex);
						serviceResult.BusinessErrorNotNulled.Add(UiMessages.___);
						serviceResult.DataOrNullOrDefValue = -1;
					}
				},
				serviceResultOnError =>
				{
					serviceResultOnError.DataOrNullOrDefValue = -1;
				}
				);
		}
		#endregion

		#region InsertCallsCsv
		[OperationContract]
		public ServiceResult<int> InsertCallsCsv(List<DetailCallAttributesCsv> listCalls, string tariff, string personalBill)
		{







			// it3xl.ru > AKotov: ,  - .   (InsertCallsCsv)   -1.
			//      , ,   -1  .









			if (listCalls.Count == 0)
			{
				return new ServiceResult<int> { DataOrNullOrDefValue = -1 };
			}

			var currentPhoneId = GetPhoneId(listCalls[0].number_a, tariff, personalBill, false, null);
			if (currentPhoneId.HasErrors)
			{
				return new ServiceResult<int> { DataOrNullOrDefValue = -1 };
			}

			var result = new ServiceResult<int> { DataOrNullOrDefValue = -1 };


			foreach (var currentDetail in listCalls)
			{
				//      
				// !!!     

				//        
				//     . 
				//      : 
				//  ,    ,  ,  .
				var resultCheckCall = CheckCall(
					currentPhoneId.DataOrNullOrDefValue,
					currentDetail.date_call,
					currentDetail.number_b,
					currentDetail.dur
					);

				//           (CheckCall  1), 
				//       . 

				if (resultCheckCall.HasErrors)
				{
					//    
					return new ServiceResult<int> { DataOrNullOrDefValue = -1 };
				}

				if (resultCheckCall.IsDone)
				{
					//       .
					if (InsertCall(currentPhoneId.DataOrNullOrDefValue, currentDetail.date_call,
						currentDetail.number_b, currentDetail.service_name,
						currentDetail.rouming_zone, currentDetail.international_zone,
						currentDetail.dur, null, currentDetail.cost, currentDetail.factor_nds
						).HasErrors)
					{
						return new ServiceResult<int> { DataOrNullOrDefValue = -1 };
					}
				}
			}

			return result;
		}
		#endregion

		#region check_call
		//check_call

		//procedure check_call(i_phone_id in number,
		//                     i_phone_number in varchar2,
		//                     i_call_date in date,
		//                     i_dur in number,
		//                     o_res out number)


		//,    

		//Parameters
		//    i_phone_id   	  
		//    i_call_date   	 
		//    i_phone_number   	 
		//    i_dur   	
		//    o_res   	
		//    1   	  
		//    0   	  

		public ServiceResult<Object> CheckCall(Int64 idA, DateTime date_time, string number_B, string dur)
		{
			//_lastError = "";

			//DateTime dt;
			//try
			//{
			//    dt = DateTime.Parse(date_time, new CultureInfo("Ru-RU", true));
			//}
			//catch (FormatException ex)
			//{
			//    Logger.Write(ex, UiMessages.___);
			//    return new ServiceResult<object>(UiMessages.___);
			//}

			var command = new OracleCommand("BA_CALLS_IAPI.check_call") { CommandType = CommandType.StoredProcedure };

			command.Parameters.Add("i_phone_id", OracleDbType.Int64, idA, ParameterDirection.Input);
			command.Parameters.Add("i_phone_number", OracleDbType.Varchar2, number_B, ParameterDirection.Input);
			command.Parameters.Add("i_call_date", OracleDbType.Date, date_time, ParameterDirection.Input);
			command.Parameters.Add("i_dur", OracleDbType.Varchar2, dur, ParameterDirection.Input);

			command.Parameters.Add("o_res", OracleDbType.Int32, ParameterDirection.Output);

			return OracleDbPull.ExecuteNonQuery<Object>(
				command,
				serviceResult =>
				{
					try
					{
						serviceResult.DataOrNullOrDefValue = Convert.ToInt32(command.Parameters["o_res"].Value);
					}
					catch (Exception ex)
					{
						Logger.Write(ex, UiMessages.___);
						serviceResult.BusinessErrorNotNulled.Add(UiMessages.___);
					}
				}
				);
		}
		#endregion

		#endregion







		#region InsertServices Group

		#region UploadListServices()
		[OperationContract]
		public ServiceResult<bool> UploadListServices(BillInfoAttributes currentBill, List<ListServicesInfo> listServicesInfo, bool replaceService)
		{
			try
			{
				UploadServices._listServices = new List<ListServicesInfo>();
				UploadServices._listServices.AddRange(listServicesInfo);
				UploadServices._currentBill = new BillInfoAttributes(currentBill);
				UploadServices._replaceService = replaceService;

				return new ServiceResult<bool> { DataOrNullOrDefValue = true, };
			}
			catch (Exception ex)
			{
				Logger.Write(ex);

				return new ServiceResult<bool>(ServiceResult.AddBusinessErrorWithDefaultError.Yes) { DataOrNullOrDefValue = true, }; ;
			}
		}
		#endregion

		#region InsertServices
		/// <summary>
		///  
		/// </summary>
		[OperationContract]
		public ServiceResult<Boolean> InsertServices(Int64 importKey)
		{
			//  
			foreach (var listServicesInfo in UploadServices._listServices)
			{
				//         
				// !!!     
				var currentPhoneId = GetPhoneId(listServicesInfo._numberPhone, listServicesInfo._nameTarif, UploadServices._currentBill.ReportBAn, false, null);

				if (currentPhoneId.HasErrors)
				{
					/// !!!
					return new ServiceResult<Boolean>(currentPhoneId.BusinessErrorNotNulled);
				}
				
				var updatePhoneResult = UpdatePhone(currentPhoneId.DataOrNullOrDefValue, listServicesInfo._nameTarif, UploadServices._currentBill.ReportBAn);
				if (updatePhoneResult.HasErrors)
				{
					// !!!
					return new ServiceResult<Boolean>(updatePhoneResult.BusinessErrorNotNulled);
				}

				long idBill = UploadServices._currentBill.ID;

				var resultCheck = CheckService(currentPhoneId.DataOrNullOrDefValue, listServicesInfo._period, importKey);
				if (resultCheck.HasErrors)
				{
					//    

					// !!!
					return new ServiceResult<bool>(resultCheck.BusinessErrorNotNulled);
				}
				else if (resultCheck.DataOrNullOrDefValue.ResultCheck == 1)
				{
					//   .     .
					if (UploadServices._replaceService)
					{
						//    (/)
						// TODO: it3xl.ru:  .    ,    .
						var deleteServiceResult = DeleteService(currentPhoneId.DataOrNullOrDefValue, listServicesInfo._period, importKey);

						if (deleteServiceResult.HasErrors)
						{
							// !!!
							return new ServiceResult<bool>(deleteServiceResult.BusinessErrorNotNulled);
						}
					}
					else
					{
						//  CheckService  1   ,
						//           CheckService
						idBill = resultCheck.DataOrNullOrDefValue.IDBilling.HasValue ? resultCheck.DataOrNullOrDefValue.IDBilling.Value : 0;
					}
				}
				else if (resultCheck.DataOrNullOrDefValue.ResultCheck == 2)
				{
					//         (    ).
					//  CheckService  2,           CheckService
					idBill = resultCheck.DataOrNullOrDefValue.IDBilling.HasValue ? resultCheck.DataOrNullOrDefValue.IDBilling.Value : 0;
				}
				else if (resultCheck.DataOrNullOrDefValue.ResultCheck == -1)
				{
					//    

					// !!!
					return ServiceResult.GetFalseServiceResult();
				}
				else
				{
					// it@it3xl.ru:      ,  .. 0.
					//   .
				}


				//   
				foreach (var currentService in listServicesInfo._listPeriodicalServices)
				{
					//       - 
					//    .
					var insertServiceResult = InsertService
						(
							idBill,
							currentService.report_tp_n,
							1,
							currentPhoneId.DataOrNullOrDefValue,
							currentService.report_tp_c,
							currentService.report_tp_a,
							currentService.report_tp_awt,

							importKey
						);

					if (insertServiceResult.HasErrors)
					{
						// !!!
						return new ServiceResult<bool>(insertServiceResult.BusinessErrorNotNulled);
					}

                //   OD

                    if (currentService.report_tp_od_a != null || currentService.report_tp_od_awt != null)
                    {
                        insertServiceResult = InsertService
                                                                (
                                                                 idBill,
                                                                 currentService.report_tp_n + "  offline",
                                                                 1,
                                                                 currentPhoneId.DataOrNullOrDefValue,
                                                                 currentService.report_tp_c,
                                                                 currentService.report_tp_od_a,
                                                                 currentService.report_tp_od_awt,

                                                                 importKey
                                                                );

                        if (insertServiceResult.HasErrors)
                        {
                            // !!!
                            return new ServiceResult<bool>(insertServiceResult.BusinessErrorNotNulled);
                        }
                    }
				}

				//   .
				foreach (var currentService in listServicesInfo._listNotPeriodicalServices)
				{
					//       - 
					//    .
					var insertServiceResult = InsertService
						(
							idBill,
							currentService.report_tp_n,
							0,
							currentPhoneId.DataOrNullOrDefValue,
							currentService.report_tp_c,
							currentService.report_tp_a,
							currentService.report_tp_awt,

							importKey
						);

					if (insertServiceResult.HasErrors)
					{
						// !!!
						return new ServiceResult<bool>(insertServiceResult.BusinessErrorNotNulled);
					}

                    if (currentService.report_tp_od_a != null || currentService.report_tp_od_awt != null)
                    {
                        insertServiceResult = InsertService
                                                                (
                                                                 idBill,
                                                                 currentService.report_tp_n + "  offline",
                                                                 0,
                                                                 currentPhoneId.DataOrNullOrDefValue,
                                                                 currentService.report_tp_c,
                                                                 currentService.report_tp_od_a,
                                                                 currentService.report_tp_od_awt,

                                                                 importKey
                                                                );

                        if (insertServiceResult.HasErrors)
                        {
                            // !!!
                            return new ServiceResult<bool>(insertServiceResult.BusinessErrorNotNulled);
                        }
                    }
				}
			}

			return ServiceResult.GetTrueServiceResult();
		}
		#endregion

		#region CheckService
		//check_service

		//procedure check_service(i_phone_id in number,
		//                        i_date in date,
		//                        o_serv out number)


		//,      

		//Parameters
		//    i_phone_id   	
		//    i_date   	
		//    o_serv
		//    1   	  
		//    0   	  

		//    o_bill -  


		internal class CheckServiceData
		{
			public Int32 ResultCheck { get; set; }
			public Int64? IDBilling { get; set; }
		}

		private static ServiceResult<CheckServiceData> CheckService(Int64 idPhone, DateTime date, Int64 importKey)
		{

			//DateTime dt;
			//try
			//{
			//    dt = DateTime.Parse(date, new CultureInfo("Ru-RU", true));
			//}
			//catch (Exception ex)
			//{
			//    Logger.Write(ex);
			//    var checkResult = new ServiceResult<CheckServiceData>(UiMessages.___)
			//    {
			//        DataOrNullOrDefValue = new CheckServiceData
			//        {
			//            ResultCheck = -1,
			//        },
			//    };

			//    return checkResult;
			//}

			SetDebug(2);

			var command = new OracleCommand("BA_BILL_SERVICES_IAPI.check_service") { CommandType = CommandType.StoredProcedure };

			command.Parameters.Add("i_phone_id", OracleDbType.Int64, idPhone, ParameterDirection.Input);
			command.Parameters.Add("i_date", OracleDbType.Date, date, ParameterDirection.Input);
			command.Parameters.Add("o_serv", OracleDbType.Int32, ParameterDirection.Output);
			command.Parameters.Add("o_bill", OracleDbType.Int64, ParameterDirection.Output);
			command.Parameters.Add("i_import_key", OracleDbType.Varchar2, importKey, ParameterDirection.Output);

			try
			{
				return
					OracleDbPull.ExecuteNonQuery<CheckServiceData>(
					command,
					serviceResult =>
					{
						serviceResult.DataOrNullOrDefValue = new CheckServiceData
						{
							ResultCheck = 0,
						};

						if (command.Parameters["o_serv"].Value != DBNull.Value)
						{
							try
							{
								serviceResult.DataOrNullOrDefValue.ResultCheck = Convert.ToInt32(command.Parameters["o_serv"].Value);
							}
							catch (Exception ex)
							{
								Logger.Write(ex);
								serviceResult.BusinessErrorNotNulled.Add(UiMessages.___);

								return;
							}
						}
						else
						{
							serviceResult.BusinessErrorNotNulled.Add(UiMessages.___);
							serviceResult.DataOrNullOrDefValue.ResultCheck = -1;

							return;
						}


						if (command.Parameters["o_bill"].Value != DBNull.Value)
						{
							try
							{
								serviceResult.DataOrNullOrDefValue.IDBilling = Convert.ToInt64(command.Parameters["o_bill"].Value);
							}
							catch (Exception ex)
							{
								Logger.Write(ex);
								serviceResult.BusinessErrorNotNulled.Add(UiMessages.___);
								serviceResult.DataOrNullOrDefValue.ResultCheck = -1;
							}
						}
						else
						{
							serviceResult.DataOrNullOrDefValue.IDBilling = null;
						}
					},
					serviceResultOnError =>
					{
						serviceResultOnError.DataOrNullOrDefValue = new CheckServiceData
						{
							ResultCheck = -1,
						};
					}
					);
			}
			finally
			{
				SetDebug(0);
			}
		}

		#endregion

		#region DeleteService
		//trm_service_by_phone

		//procedure trm_service_by_phone(i_phone_id in number,
		//                               i_date in date)


		//    

		//Parameters
		//    i_phone_id   	
		//    i_date   	

		public ServiceResult<Object> DeleteService(Int64 idPhone, DateTime date, Int64 importKey)
		{
			//DateTime dt;
			//try
			//{
			//    dt = DateTime.Parse(date, new CultureInfo("Ru-RU", true));
			//}
			//catch (Exception ex)
			//{
			//    Logger.Write(ex);

			//    return new ServiceResult<object>(UiMessages.__);
			//}

			var command = new OracleCommand("BA_BILL_SERVICES_IAPI.trm_service_by_phone")
			{
				CommandType = CommandType.StoredProcedure
			};

			command.Parameters.Add("i_phone_id", OracleDbType.Int64, idPhone, ParameterDirection.Input);
			command.Parameters.Add("i_date", OracleDbType.Date, date, ParameterDirection.Input);
			command.Parameters.Add("i_import_key ", OracleDbType.Varchar2, importKey, ParameterDirection.Input);

			return OracleDbPull.ExecuteNonQuery<Object>(command,null);
		}
		#endregion

        #region InsertService
        //procedure ins_ba_bill_services(i_bill_id in ba_bill_services.bill_id%type,
        //                               i_service_name in varchar2,
        //                               i_service_type in number,
        //                               i_phone_id in ba_bill_services.phone_id%type,
        //                               i_dur in ba_bill_services.dur%type,
        //                               i_dur_dim in ba_bill_services.dur_dim%type,
        //                               i_cost in ba_bill_services.cost%type,
        //                               i_cost_no_vat in ba_bill_services.cost_no_vat%type,
        //                               i_from_date in ba_bill_services.from_date%type,
        //                               i_to_date in ba_bill_services.to_date%type,
        //                               o_id out number)


        //  ""

        //Parameters
        //    i_bill_id   	  
        //    i_service_name   	 
        //    i_service_type   	 
        //    1   	
        //    0   	 

        //    i_phone_id   	 
        //    i_dur   	  - , 
        //    i_dur_dim   	 (,,,,)
        //    i_cost   	   
        //    i_cost_no_vat   	   
        //    i_from_date   	 
        //    i_to_date   	 
        //    o_id   	  

        /// <summary>
        /// Date:21.09.2012, 
        /// changed by: Evseev,
        /// Reason:   ""    .    .
        ///   string fromDate -  
        /// string fromDate -   . 
        /// </summary>
        [OperationContract]
        public ServiceResult<object> insertServicePublic(long idBill, string service_name, int service_type,
            Int64 idPhone, string dur, string cost_nds, string cost, Int64 importKey, DateTime period)
        {
            var deleteServiceResult = DeleteService(idPhone, period, importKey);

            if (deleteServiceResult.HasErrors)
            {
                // !!!
                return new ServiceResult<object>(deleteServiceResult.BusinessErrorNotNulled);
            }

            return InsertService(idBill, service_name, service_type,
             idPhone, dur, cost_nds, cost, importKey);
        }


		private ServiceResult<Object> InsertService(long idBill, string service_name, int service_type,
			Int64 idPhone, string dur, string cost_nds, string cost, Int64 importKey)
		{
			SetDebug(2);

			string dur_dim = "";
			string dt_dur = GetDur(dur, ref dur_dim);
			double d_cost_nds = Convert.ToDouble(CommonFunctions.NormFloat(cost_nds));
			double d_cost = Convert.ToDouble(CommonFunctions.NormFloat(cost));

			var command = new OracleCommand("BA_BILL_SERVICES_IAPI.ins_ba_bill_services")
			{
				CommandType = CommandType.StoredProcedure
			};

			command.Parameters.Add("i_bill_id", OracleDbType.Int64, idBill, ParameterDirection.Input);
			command.Parameters.Add("i_service_name", OracleDbType.Varchar2, service_name, ParameterDirection.Input);
			command.Parameters.Add("i_service_type", OracleDbType.Int32, service_type, ParameterDirection.Input);
			command.Parameters.Add("i_phone_id", OracleDbType.Int64, idPhone, ParameterDirection.Input);
			command.Parameters.Add("i_dur", OracleDbType.Varchar2, dt_dur, ParameterDirection.Input);
			command.Parameters.Add("i_dur_dim", OracleDbType.Varchar2, dur_dim, ParameterDirection.Input);
			command.Parameters.Add("i_cost", OracleDbType.Double, d_cost_nds, ParameterDirection.Input);
			command.Parameters.Add("i_cost_no_vat", OracleDbType.Double, d_cost, ParameterDirection.Input);
			command.Parameters.Add("i_from_date", OracleDbType.Date, null, ParameterDirection.Input);
			command.Parameters.Add("i_to_date", OracleDbType.Date, null, ParameterDirection.Input);
			command.Parameters.Add("i_import_key", OracleDbType.Varchar2, importKey, ParameterDirection.Input);
			command.Parameters.Add("o_id", OracleDbType.Int32, ParameterDirection.Output);

			ServiceResult<Object> result;
			try
			{
				result = OracleDbPull.ExecuteNonQuery<Object>(
						command,
						serviceResult =>
						{
							try
							{
								Convert.ToInt32(command.Parameters["o_id"].Value);
							}
							catch (Exception ex)
							{
								Logger.Write(ex);
								serviceResult.BusinessErrorNotNulled.Add(UiMessages.___);
							}
						}
						);
			}
			catch (Exception)
			{
				throw;
			}

			SetDebug(0);

			return result;
		}

		#endregion

		#endregion END InsertServices Group




		#region InsertPhone
        private ServiceResult<Int64> InsertPhone(string numberPhone, string tariff, string numberPersonalBill, string abonentId)
		{
			var command = new OracleCommand("BA_PHONES_IAPI.ins_ba_phones")
			{
				CommandType = CommandType.StoredProcedure
			};

			command.Parameters.Add("i_phone", OracleDbType.Varchar2, numberPhone, ParameterDirection.Input);
            if (string.IsNullOrEmpty(abonentId))
            {
                command.Parameters.Add("i_abonent_id", OracleDbType.Int32, null, ParameterDirection.Input);
            }
            else
            {
                command.Parameters.Add("i_abonent_id", OracleDbType.Int32, Convert.ToInt32(abonentId), ParameterDirection.Input);
            }
			command.Parameters.Add("i_account", OracleDbType.Varchar2, numberPersonalBill, ParameterDirection.Input);
			command.Parameters.Add("i_tarif", OracleDbType.Varchar2, tariff, ParameterDirection.Input);
			command.Parameters.Add("i_from_date", OracleDbType.Date, null, ParameterDirection.Input);
			command.Parameters.Add("i_to_date", OracleDbType.Date, null, ParameterDirection.Input);
			command.Parameters.Add("i_phone_id", OracleDbType.Int32, ParameterDirection.Output);

			return OracleDbPull.ExecuteNonQuery<Int64>(
				command,

				serviceResult =>
				{
					try
					{
						serviceResult.DataOrNullOrDefValue = Convert.ToInt32(command.Parameters["i_phone_id"].Value);
					}
					catch (Exception ex)
					{
						Logger.Write(ex);
						serviceResult.DataOrNullOrDefValue = -1;
						serviceResult.BusinessErrorNotNulled.Add(UiMessages.___);
					}
				},

				serviceResultOnError =>
				{
					serviceResultOnError.DataOrNullOrDefValue = -1;
					serviceResultOnError.AddBusinessError(UiMessages.__);
				}
				);
		}
		#endregion

		#region UpdatePhone
		private ServiceResult<Object> UpdatePhone(Int64 idPhone, string tariff, string personalBill)
		{
			// it3xl.ru  .  BA_PHONES_IAPI.upd_phone  BA_PHONES_IAPI.upd_phones.
			var command = new OracleCommand("BA_PHONES_IAPI.upd_phone") { CommandType = CommandType.StoredProcedure };

			command.Parameters.Add("i_id", OracleDbType.Int64, idPhone, ParameterDirection.Input);
			command.Parameters.Add("i_tarif", OracleDbType.Varchar2, tariff, ParameterDirection.Input);
			command.Parameters.Add("i_account", OracleDbType.Varchar2, personalBill, ParameterDirection.Input);
			command.Parameters.Add("i_from_date", OracleDbType.Date, null, ParameterDirection.Input);

			return OracleDbPull.ExecuteNonQuery<Object>(
				command,
				null,
				prepareOnErrorAction =>
				{
					prepareOnErrorAction.BusinessErrorNotNulled.Add(UiMessages.____);
				}
				);
		}

		#endregion

		#region GetDur
		private string GetDur(string dur, ref string dur_dim)
		{
			if (dur == null)
				return "";

			string dt_dur = "";

			string dt_value = dur.ToLower();

			string[] splits = { " " };

			string[] parts = dt_value.Split(splits, StringSplitOptions.None);

			if (parts.Length >= 2)
			{
				dt_dur = parts[0];
				dur_dim = parts[1];
			}
			else if (parts.Length == 0)
			{
				dt_dur = "";
				dur_dim = "";
			}
			else if (parts.Length == 1)
			{
				dt_dur = parts[0];
				dur_dim = "";
			}


			//int indexKey = -1;
			//if ((indexKey = dt_value.IndexOf("")) != -1)
			//{
			//    dt_dur = Convert.ToInt32(dt_value.Substring(0, indexKey - 1));
			//    dur_dim = dt_value.Substring(indexKey);
			//}
			//else if ((indexKey = dt_value.IndexOf("")) != -1)
			//{
			//    dt_dur = Convert.ToInt32(dt_value.Substring(0, indexKey - 1));
			//    dur_dim = dt_value.Substring(indexKey);
			//}
			//else if ((indexKey = dt_value.IndexOf("")) != -1)
			//{
			//    dt_dur = Convert.ToInt32(dt_value.Substring(0, indexKey - 1));
			//    dur_dim = dt_value.Substring(indexKey);
			//}
			//else if ((indexKey = dt_value.IndexOf("")) != -1)
			//{
			//    dt_dur = Convert.ToInt32(dt_value.Substring(0, indexKey - 1));
			//    dur_dim = dt_value.Substring(indexKey);
			//}
			//else if ((indexKey = dt_value.IndexOf("")) != -1)
			//{
			//    dt_dur = Convert.ToInt32(dt_value.Substring(0, indexKey - 1));
			//    dur_dim = dt_value.Substring(indexKey);
			//}
			//else
			//{
			//    dt_dur = Convert.ToInt32(dt_value);
			//    dur_dim = "";
			//}

			return dt_dur;
		}
		#endregion

			#region GetPhoneId
        /// <summary>
        /// Date:21.09.2012, 
        /// changed by: Evseev,
        /// Reason:   ""       .
        ///   :bool isUpdated -     ,     .
        /// string abonentId - id .
        /// </summary>
        [OperationContract]
        public ServiceResult<Int64> getPhoneIdPublic(string account, string phone, string tariff,  bool isUpdate, string abonentId)
        {
            return GetPhoneId(phone, tariff, account, isUpdate, abonentId);
        }

        private ServiceResult<Int64> GetPhoneId(string numberPhone, string tariff, string personalBill, bool isUpdate, string abonentId)
		{
			SetDebug(2);

			var command = new OracleCommand("BA_PHONES_IAPI.get_phone_id") { CommandType = CommandType.StoredProcedure };

			command.Parameters.Add("i_phone", OracleDbType.Varchar2, numberPhone, ParameterDirection.Input);
			command.Parameters.Add("i_phone_id", OracleDbType.Int64, ParameterDirection.Output);

			var result = OracleDbPull.ExecuteNonQuery<Int64>(
				command,
				serviceResult =>
				{
					Int64 idPhone = -1;
					if (command.Parameters["i_phone_id"].Value != DBNull.Value)
					{
						try
						{
							idPhone = Convert.ToInt64(command.Parameters["i_phone_id"].Value);
						}
						catch (Exception ex)
						{
							Logger.Write(ex);
							serviceResult.BusinessErrorNotNulled.Add(UiMessages.___);
						}
					}

					if (idPhone == -1)
					{
						//         .
						// it3xl.ru   .
                        var idPhoneResult = InsertPhone(numberPhone, tariff, personalBill, abonentId);
						if (idPhoneResult.HasErrors)
						{
							serviceResult.AddResultStates(idPhoneResult);

							// !!!
							return;
						}
						idPhone = idPhoneResult.DataOrNullOrDefValue;
					}
                    /// <summary>
                    /// Date:21.09.2012, 
                    /// changed by: Evseev,
                    /// Reason:       
                    /// </summary>
                    else
                    {
                        if (isUpdate)
                        {

                            var updatePhoneResult = UpdatePhone(idPhone, tariff, personalBill);
                            if (updatePhoneResult.HasErrors)
                            {
                                serviceResult.AddResultStates(updatePhoneResult);
                            }

                        }
                    }

					serviceResult.DataOrNullOrDefValue = idPhone;
				},
				serviceResultOnError =>
				{
					serviceResultOnError.AddBusinessError(UiMessages.___);
					serviceResultOnError.DataOrNullOrDefValue = -1;
				}
				);

			SetDebug(0);

			return result;

		}
		#endregion

		#region SetDebug

		private static void SetDebug(int t)
		{
			//OracleCommand command = new OracleCommand("krn_debug.SetMode");
			//command.CommandType = System.Data.CommandType.StoredProcedure;

			//command.Parameters.Add("i_t", OracleDbType.Int32, t, System.Data.ParameterDirection.Input);

			//OracleDbPull.ExecuteNonQuery(command);
		}
		#endregion



	}
}