//   .

using System;
using System.Linq;
using System.Linq.Expressions;
using System.Net;
using System.Net.Mail;
using System.ServiceModel;
using System.ServiceModel.Activation;
using System.Collections.Generic;
using BillAnalyserWebApp.Code.Data;
using CrossCompileSharing.Data;
using CrossCompileSharing.Log;
using CrossCompileSharing.UiStrings;

namespace BillAnalyserWebApp
{
	[ServiceContract(Namespace = "")]
	[AspNetCompatibilityRequirements(RequirementsMode = AspNetCompatibilityRequirementsMode.Allowed)]
	public class SmtpService
	{
		[OperationContract]
		public ServiceResult<List<PostResult>> PostBills(
			List<Abonent> abonents,
			DateTime period,
			String stateKey,

			Boolean useCostsForServices,
			Boolean useSubtotal,
			Boolean useDetalization,
			
			Calls.ViewModFor viewModFor,
			SmtpParameters smtpParameters
			)
		{
			if (String.IsNullOrEmpty(stateKey))
			{
				Logger.Write("         .");

				return new ServiceResult<List<PostResult>>(UiMessages.__);
			}

			if (smtpParameters ==null)
			{
				Logger.Write(UiMessages._SMTP___);

				return new ServiceResult<List<PostResult>>(UiMessages._SMTP___);
			}

			if (abonents == null ||abonents.Count==0)
			{
				return new ServiceResult<List<PostResult>>(UiMessages._____eMail);
			}

			var oparationState = new OperationState
			{
				NumberPosts = abonents.Count
			};
			if (Smtp.SetOperationStateTry(stateKey, oparationState) == false)
			{
				Logger.Write("       .");

				return new ServiceResult<List<PostResult>>(UiMessages.__);
			}

			SmtpClient smtpClient;
			try
			{
				smtpClient = new SmtpClient(smtpParameters.Host, smtpParameters.Port);
			}
			catch (Exception ex)
			{
				Logger.Write(ex, UiMessages.__);

				return new ServiceResult<List<PostResult>>(UiMessages.__);
			}



			smtpClient.DeliveryMethod = SmtpDeliveryMethod.Network;

			var settings = AnalyserSettings.UnsafeSettingsGet(0, false);

			var subjectSetting = settings.DataOrNullOrDefValue.Settings.FirstOrDefault((setting) => setting.Id == AnalyserSettings.EmailSubjectSettingMetadata.Id);
			smtpParameters.Subject = subjectSetting != null && String.IsNullOrEmpty(subjectSetting.Value_V) == false
				? subjectSetting.Value_V
				: "RE: billing info";

			// it3xl.ru:         ,     .
			var passSetting = settings.DataOrNullOrDefValue.Settings.FirstOrDefault((setting) => setting.Id == AnalyserSettings.SmtpServerPasswordSettingMetadata.Id);
			String pass = null;
			if (passSetting != null)
			{
				pass = passSetting.Value_V;

				if(String.IsNullOrEmpty(pass))
				{
					//     (   ),      null,     , ..   ,   .
					pass = null;
				}
			}

			// it3xl.ru:   ,       Windows,   .
			if (String.IsNullOrEmpty(smtpParameters.User))
			{
				//        .   .    .
				smtpClient.Credentials = CredentialCache.DefaultNetworkCredentials;
			}
			else
			{
				//       .
				if (smtpParameters.User.Contains("@"))
				{
					//   ,     smtpClient.Credentials = new NetworkCredential("_", null, "_");,
					//    new NetworkCredential("_@_", null)  NetworkCredential("_", null).
					var login = smtpParameters.User.Split(new []{"@"}, StringSplitOptions.None);
					var computerName = login.First();
					// it3xl.ru: ,              @,  ,  login.length == 2   .
					var domainName = login[1];
					smtpClient.Credentials = new NetworkCredential(computerName, pass, domainName);
				}
				else
				{
					// it3xl.ru:    (null),           @,
					//                 (  ),       ,
					//         .
					smtpClient.Credentials = new NetworkCredential(smtpParameters.User, pass);
				}
			}



			var result = new ServiceResult<List<PostResult>>
			{
				DataOrNullOrDefValue = new List<PostResult>(),
			};

			foreach (var abonent in abonents)
			{
				var sendMessageResult = Smtp.SendMessage(abonent, period, useCostsForServices, useSubtotal, useDetalization, viewModFor, smtpParameters, smtpClient, stateKey);
				result.DataOrNullOrDefValue.Add(sendMessageResult.DataOrNullOrDefValue);
				if (sendMessageResult.DataOrNullOrDefValue.Terminated)
				{
					break;
				}
				Smtp.IncreaseOperationStateProcessedCount(stateKey);
			}

			Smtp.ReleaseOperationStateTry(stateKey);

			return result;
		}

		[OperationContract]
		public ServiceResult<int> GetPercent(String stateKey)
		{
			return new ServiceResult<int> { DataOrNullOrDefValue = Smtp.GetOperationPercent(stateKey), };
		}

		[OperationContract]
		public ServiceResult<Boolean> TerminatePostBills(String stateKey)
		{
			Smtp.TerminatePostBills(stateKey);
			return ServiceResult.GetTrueServiceResult();
		}

	}
}