

//  it3xl.ru  .


#region using

using System;
using System.Data;
using System.Linq;
using System.Linq.Expressions;
using System.Reflection;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.ServiceModel.Activation;
using System.Collections.Generic;
using System.Text;
using BillAnalyserWebApp.Code.Data;
using BillAnalyserWebApp.Code.IFlex.DB;
using CrossCompileSharing.Data;
using iFlexBillAnalyserResources.Data;
using Oracle.DataAccess.Client;

#endregion

namespace BillAnalyserWebApp
{
	[ServiceContract(Namespace = "iFlexServices")]
	[AspNetCompatibilityRequirements(RequirementsMode = AspNetCompatibilityRequirementsMode.Allowed)]
	public class SubscriptionService
	{
		#region GetSubscriptionData OperationContract

		[OperationContract]
		public ServiceResult<SubscriptionsData> GetSubscriptionData()
		{
			var result = new ServiceResult<SubscriptionsData>();

			var orgUnitsResult = OrgUnits.OrgUnitsGet();
			if (orgUnitsResult.IsDone == false)
			{
				result.BusinessErrorNotNulled.Add(orgUnitsResult);

				return result;
			}
			var notUsedPhonesResult = GetNotUsedPhones();
			if (notUsedPhonesResult.IsDone == false)
			{
				result.BusinessErrorNotNulled.Add(notUsedPhonesResult);

				return result;
			}

			result.DataOrNullOrDefValue = new SubscriptionsData
				{
					OrgUnits = orgUnitsResult.DataOrNullOrDefValue,
					NotUsedPhones = notUsedPhonesResult.DataOrNullOrDefValue,
				};

			return result;
		}

		#endregion

		#region GetNotUsedPhones

		private static ServiceResult<List<Phone>> GetNotUsedPhones()
		{
			//SubscriptionDataSet.V_BA_PHONES_UNUSEDDataTable notUsedPhonesDT;

			//var dataSet = new SubscriptionDataSet();
			//// it3xl.ru   Oracle.net,    DataSet    ,      using
			//using (dataSet)
			//{
			//    var notUsedPhonesTableAdapter = new V_BA_PHONES_UNUSEDTableAdapter();
			//    notUsedPhonesDT = notUsedPhonesTableAdapter.GetData();
			//}


			var oCom = new OracleCommand(@"
SELECT ID, PHONE, TARIF FROM V_BA_PHONES_UNUSED ORDER BY PHONE
"
				);

			return OracleDbPull.GetDataTable<List<Phone>>(
				oCom,
				(resultDataTable, serviceResult) =>
				{
					serviceResult.DataOrNullOrDefValue = (
						from row in resultDataTable.AsEnumerable()
						select new Phone
						{
							Id = (Int64)row.Field<Decimal>("ID"),
							PhoneStr = row.Field<String>("PHONE"),
							Tarif = row.Field<String>("TARIF"),
						}
						).ToList();
				}
				);

		}

		#endregion

		#region GetSubscriptionData DataClasses

		[Obfuscation(Exclude = true)]
		public class SubscriptionsData
		{
			public List<OrgUnit> OrgUnits { get; set; }
			public List<Phone> NotUsedPhones { get; set; }
		}

		#endregion


		#region OrgUnits

		[OperationContract]
		public ServiceResult<Boolean> OrgUnitUpdate(OrgUnit orgUnit)
		{
			ServiceResult<Boolean> result;
			using (var dbInTransaction = new OracleDbPullInTransaction())
			{
				result = OrgUnits.OrgUnitUpdate(orgUnit, dbInTransaction);
				if(result.IsDone)
				{
					dbInTransaction.Commit();
				}
			}

			return result;
		}


		[OperationContract]
		public ServiceResult<OrgUnit> OrgUnitCreate(OrgUnit orgUnit)
		{
			return OrgUnits.OrgUnitCreate(orgUnit);
		}


		[OperationContract]
		public ServiceResult<Boolean> OrgUnitDelete(IEnumerable<OrgUnit> orgUnit)
		{
			return OrgUnits.OrgUnitDelete(orgUnit);
		}


		[OperationContract]
		public ServiceResult<Boolean> OrgUnitAddToChilds(IEnumerable<OrgUnit> childOrgUnits, OrgUnit parentOrgUnit)
		{
			return OrgUnits.OrgUnitAddToChilds(childOrgUnits, parentOrgUnit);
		}

		#endregion


		#region Compensations

		[OperationContract]
		public ServiceResult<List<Compensation>> CompensationsGet(Int64 phoneId)
		{
			return Compensations.CompensationsGet(phoneId);
		}

		[OperationContract]
		public ServiceResult<Boolean> CompensationsRemove(Compensation compensation)
		{
			return Compensations.TrmPhoneComps(compensation.PhoneCompsId);
		}

		#endregion


		#region Phones

		[OperationContract]
		public ServiceResult<Boolean> UserPhoneLinkSet(Phone phone, OrgUnit orgUnit)
		{
			return Phones.UserPhoneLinkSet(phone, orgUnit);
		}


		[OperationContract]
		public ServiceResult<Boolean> UserPhoneLinksSet(List<Phone> phones, OrgUnit orgUnit)
		{
			return Phones.UserPhoneLinksSetInTransaction(phones, orgUnit);
		}

		#endregion

	}
}
