
//  it3xl.ru  .


using System;
using System.Linq;
using System.ServiceModel;
using System.ServiceModel.Activation;
using System.Collections.Generic;
using BillAnalyserWebApp.Code.Data;
using BillAnalyserWebApp.Code.IFlex.DB;
using System.Data;
using CrossCompileSharing.Data;
using Oracle.DataAccess.Client;

namespace BillAnalyserWebApp
{
	[ServiceContract(Namespace = "iFlex")]
	[AspNetCompatibilityRequirements(RequirementsMode = AspNetCompatibilityRequirementsMode.Allowed)]
	public class Withholdings
	{
		[OperationContract]
		public ServiceResult<List<Withholding>> GetAllWithholdingsByPeriod(DateTime period)
		{
			var oCom = new OracleCommand { CommandText = @"
select a.id,a.abonent_id,a.name as abonent_name,a.is_group,a.code_1c,s.sum_add,s.sum_ded from 
(select ph.abonent_id
      ,sum(ch.sum_add) sum_add
      ,sum(ch.sum_ded) sum_ded
  from v_ba_bill_charges ch
      ,v_ba_phones       ph
      ,v_ba_bills        bi
 where ch.phones_id = ph.id
   and ch.bill_id = bi.id
   and bi.bill_date = :bill_date /* to_date('01.09.2009','DD.MM.RRRR') */
 group by ph.abonent_id) s,
         v_ba_abonents a
where a.id=s.abonent_id(+)    
" };

			oCom.Parameters.Add("bill_date", OracleDbType.Date, period, ParameterDirection.Input);

			return OracleDbPull.GetDataTable<List<Withholding>>(
				oCom,
				(resultDataTable, serviceResult) =>
				{
					serviceResult.DataOrNullOrDefValue =
						(
							from row in resultDataTable.AsEnumerable()
							select
								new Withholding
								{
									Id = (Int64)row.Field<Decimal>("ID"),
									ParentId = (Int64?)row.Field<Decimal?>("ABONENT_ID"),
									Name = row.Field<String>("abonent_name"),
									Code1C = row.Field<String>("CODE_1C") ,
									WithholdingAmmount = row.Field<Decimal?>("sum_ded"),
									IsGroup = row.Field<Int16>("IS_GROUP") == 1,
								}
							).ToList();
				}
				);
		}
	}
}